/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.apportal.server.imp ;

import java.util.logging.Logger;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.*;

import org.azuki.bean.apportal.AppPortalBean;
import org.azuki.bean.apportal.server.PortalAdvice;
import org.azuki.bean.apportal.server.imp.resource.RsrcPortal;
import org.azuki.common.console.*;
import org.azuki.common.console.html.HtmlBuilder;
import org.azuki.common.console.http.HttpRequest;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.Framework;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.Locator;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class PService
{
	final static public String UA_XMLHttpRequest = "UA-XMLHttpRequest" ;
	
	// URI :  ..../portal.html   for main view
	// URI :  ..../portal.html/<BeanID.hasString('.'))>/  for a bean
	
	final static public void p(AppPortalBean pa, HttpServlet svt, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		String ruag = req.getHeader("User-Agent") ;
		
		// .../console/something/... --> /something/...
		String ruri = PortalUtil.relativeURI(pa, req) ;
		
		// /something/... --> something
		//String tokn = StringUtil.nextToken(ruri, "/") ;
		//if(tokn.equals(PortalUtil.rsrcCtx))		doResource(pa, svt, req, res) ;
		
		if(ruri.indexOf(PortalUtil.rsrcCtx)!=-1)	doResource(pa, svt, req, res) ;
		else if(UA_XMLHttpRequest.equals(ruag)) 	doBean(pa, svt, req, res) ;
		else					 		  			doPortal(pa, svt, req, res) ;
	}
	
	final static public void doPortal(AppPortalBean pa, HttpServlet svt, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Logger lgr = Log.getLogger() ;
		
		lgr.finest(">> doPortal()") ;
		
		Template tmpl = new Template(RsrcPortal.getInputStream("portal.html")) ;
		tmpl.replace("${portal:name}", pa.getName()) ;
		tmpl.replace("${portal:title}", pa.getTitle()) ;
		tmpl.replace("${portal.css}", PortalUtil.mkRsrcURL(pa, req, RsrcPortal.PORTAL_CSS)) ;
		tmpl.replace("${portal.js}", PortalUtil.mkRsrcURL(pa, req, RsrcPortal.PORTAL_JS)) ;
		tmpl.replace("${favicon.ico}", PortalUtil.mkRsrcURL(pa, req, RsrcPortal.FAVICO_ICO)) ;
		
		Template.Part portalHtmlTop = tmpl.getFirst() ;
		Template.Part portalHtmlBot = tmpl.getLast() ;
		
		String portalHtmlMenu = PPortalMenu.p(pa, req) ;
		portalHtmlTop.replace("${portal:menu}", portalHtmlMenu) ;
		portalHtmlTop.replace("${portal:notice}", "Azuki framework/"+Framework.getVersion()) ;
		
		res.setContentType(HttpRequest.isIE(req) ? "text/html" : "application/xhtml+xml") ;
		
		ServletOutputStream sos = res.getOutputStream() ;
		sos.println(portalHtmlTop.toString()) ;
		
		doBean(pa, svt, req, res) ;
		
		sos.println(portalHtmlBot.toString()) ;
	}
	
	final static public void doBean(AppPortalBean pa, HttpServlet svt, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Logger lgr = Log.getLogger() ;
		
		lgr.finest(">> doBean()") ;
		
		PortalID pid = PortalUtil.getID(pa, req) ;
		if(pid==null) return ;
		
		Integer bid = pid.getBeanID() ;
		if(bid==null) return ;
		
		try
		{
			IMetadata meta = Bean.getMetadata(bid) ;

			IPointcut ipc = meta.getPointcut(pid.getPointcutID()) ;
			
			PortalAdvice cad = (PortalAdvice)ipc.getAdvice() ;
			
			IMethodProxy mpxy = (IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, meta.getName(), meta.getLabel(), cad.getMethod()) ; 
			mpxy.call(svt, req, res) ;
		}
		catch(Throwable t)
		{
			ServletOutputStream sos = res.getOutputStream() ;
			sos.println("<pre><![CDATA[") ;
			sos.println(Log.getStack(t)) ;
			sos.println("]]></pre>") ;
		}		
	}
	
	final static public void doBeanCard(Integer bid, HttpServlet svt, HttpServletRequest req, HttpServletResponse res)
	{
		HtmlBuilder html = new HtmlBuilder() ;
		
		html.println("<table>") ;
		html.println("<tr>") ;
		html.println("<td>") ;
		html.println("</td>") ;
		html.println("<td>") ;
		html.println("</td>") ;
		html.println("<td>") ;
		html.println("</td>") ;
		html.println("</tr>") ;
		html.println("</table>") ;
		
	}
	
	final static public void doResource(AppPortalBean pa, HttpServlet svt, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Logger lgr = Log.getLogger() ;
		
		lgr.finest(">> doResource()") ;
		
		String ruri = req.getRequestURI() ;
		
		int idx = ruri.indexOf(PortalUtil.rsrcCtx) ;
		if(idx==-1) return ; // impossible !
		
		String rnam = StringUtil.unslashed(ruri.substring(idx+1)) ;
		lgr.finest("RNAM ("+rnam+")") ;
		
//		String ctyp = Mime.getContentTypeFor(rnam) ;
		String ctyp = svt.getServletContext().getMimeType(rnam) ;
		byte[] byts = Resource.getBytes(PService.class.getClassLoader(), rnam) ;
		
		// pas de "Content-Length": possible pb d'encodage == taille erronee !!
		if(ctyp!=null) res.setContentType(ctyp) ;
		res.getOutputStream().write(byts) ;
	}
}