/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.apportal.server.imp ;

import javax.servlet.http.HttpServletRequest;

import org.azuki.bean.apportal.AppPortalBean;
import org.azuki.common.console.http.HttpRequest;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.context.Log;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class PortalUtil
{
	final static public String rsrcCtx = "!" ;
	
	final static public String mkRsrcURL(AppPortalBean pa, HttpServletRequest req, String res)
	{
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(portalURI(pa, req)) ;
		bdr.append('/') ;
		bdr.append(rsrcCtx) ;
		bdr.append('/').append(res) ;
		
		return bdr.toString() ;
	}
	
//	final static public String mkRsrcURL(AppPortalBean pa, HttpServletRequest req, String res, Map<String,String> para)
//	{
//		StringBuilder bdr = new StringBuilder() ;
//		bdr.append(consoleURI(pa, req)) ;
//		bdr.append(rsrcCtx) ;
//		bdr.append("/").append(res) ;
//		
//		if(para==null || para.isEmpty()) return bdr.toString() ;
//		
//		bdr.append('?') ;
//		
//		for(Map.Entry<String,String> ety : para.entrySet())
//			bdr.append('&').append(ety.getKey()).append('=').append(ety.getValue()) ;
//		
//		return bdr.toString() ;
//	}
	
	final static public String mkBeanURL(AppPortalBean pa, HttpServletRequest req, Integer bid, String pid)
	{	
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(portalURI(pa, req)) ;
		bdr.append('/') ;
		bdr.append(bid.toString()) ;
		bdr.append(PortalID.pcutSep).append(pid) ;
		bdr.append('/') ;
		
		return bdr.toString() ;
	}
	
	// /.../portal
	final static public String portalURI(AppPortalBean pa, HttpServletRequest req)
	{
		String puri = pa.getURI() ;
		String ruri = HttpRequest.decodeURI(req) ;
		
		int idx = ruri.indexOf(puri) ;
		if(idx==-1) return null ; // impossible !
		
		String uri = puri.substring(0, idx+puri.length()) ;
		return StringUtil.slashStart(uri) ;
	}
	
	// .../portal/something/... --> /something/...
	final static protected String relativeURI(AppPortalBean pa, HttpServletRequest req)
	{
		String puri = pa.getURI() ;
		String ruri = HttpRequest.decodeURI(req) ;
		
		int idx = ruri.indexOf(puri) ;
		if(idx==-1) return null ; // pas possible !
		
		String uri = ruri.substring(idx+puri.length()) ;
		return StringUtil.slashStart(uri) ;
	}
	
	// Returns null if no portalID in the url (main page)
	final static public PortalID getID(AppPortalBean pa, HttpServletRequest req)
	{
		String puri = pa.getURI() ; // "/portal" ;
		String ruri = HttpRequest.decodeURI(req) ;
				
		Log.finest("RURI ("+ruri+")") ;
		Log.finest("PURI ("+puri+")") ;

		if(ruri.endsWith(puri)) return null ; // main page
		
		int idx = ruri.indexOf(puri) ; // indexof "/portal/..."
		if(idx==-1) return null ; // ? never occur
		
		String bean = ruri.substring(idx+puri.length()) ; // "/<beanid> + ..."
		if(bean.isEmpty()) return null ;
		
		bean = StringUtil.unslashed(bean) ;
		if(bean.isEmpty()) return null ;
		Log.finest("TEMP ("+bean+")") ;
		
		idx = bean.indexOf("/") ; // Find next slash, after <beanid>
		if(idx!=-1) bean = bean.substring(0,idx) ;
		
		Log.finest("PORTALID ("+bean+")") ;
	
		return PortalID.fromString(bean) ;
	}
}