/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.chrono.server;

import java.lang.reflect.Method;
import java.util.*;

import org.azuki.bean.chrono.ChronoBean;
import org.azuki.bean.chrono.server.imp.ChronoLine;
import org.azuki.framework.injection.*;

final public class ChronoAdvice implements IController
{
	final private ChronoBean apst ;
	final private Method[] meths ;
	
	// contains chrono values for each method selected in the pointcut
	final private Map<String,ChronoLine> lines = new HashMap<String,ChronoLine>() ;
	final public  Map<String,ChronoLine> getLines() { return lines ; }
	
	public ChronoAdvice(ChronoBean aspt, Method[] meths)
	{
		this.apst = aspt ;
		this.meths = meths ;
	}

	// IController
	final public Object intercept(InvocationTarget it) 
	throws Exception
	{
		long sta = System.currentTimeMillis() ;
		Object ret = it.proceed() ;
		long end = System.currentTimeMillis() ;
		
		if(!apst.isRunning()) return ret ;
		
		String mnam = it.getMethodName() ;
		
		for(Method meth : meths)
		{
			if(!meth.getName().equals(mnam)) continue ;
			
			ChronoLine line = lines.get(mnam) ;
			
			if(line==null)
			{	line = new ChronoLine() ;
				lines.put(mnam, line) ; }
			
			line.add(sta, end) ;
		}
		
		return ret ;
	}
}
