/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.command.server.imp.queue ;

import java.util.*;

import org.azuki.bean.command.server.imp.core.CPU;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class CmdQF
{
	final static private CPU cpu = new CPU() ;
	static
	{
    	// Creation of main "CPU" thread.
		Thread thread = new Thread(cpu, "KERNEL");
		thread.setPriority(Thread.NORM_PRIORITY) ;
		thread.start();
	}
	
	final static private CmdQF cmdqf = new CmdQF() ;
	final static public CmdQF instance() { return cmdqf; }
	
	final static private SCmdQ mainQ = new SCmdQ(cpu) ;
	final static public SCmdQ main() { return mainQ; }
	
    final private HashMap<String, CmdQ> queues = new HashMap<String, CmdQ>();

    @SuppressWarnings("unchecked")
	final public Map<String, CmdQ> get(boolean clone)
    {
        if(!clone) return queues ;
        return (Map<String, CmdQ>)queues.clone();
    }

    final synchronized public CmdQ get(String name)
	{
    	CmdQ cmdq = queues.get(name);

		if(cmdq == null)
		{
			cmdq = new CmdQ(cpu, name) ;
			queues.put(name, cmdq);
		}

		return cmdq ;
	}

	final synchronized public void remove(String name)
	{
		CmdQ cmdq = queues.get(name);
		queues.remove(name) ;
		if(cmdq == null) return ;
		cmdq.destroy() ;
	}	
}