/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.command.server.imp.queue ;


import java.util.*;

//
// Faster implementation for OQP when no priority is needed
// and no other stuff

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class OQ<T>
{
	private int count = 0 ; // conteur de message recu
	final public int count() { return count ; } // total queued

	final private LinkedList<T> oq = new LinkedList<T>();

	@SuppressWarnings("unchecked")
	final public List<T> asList()
	{
		return (List<T>)oq.clone() ; // Give a shallow copy to  prevent modification
	}

	final public synchronized void clear()
	{
		oq.clear();
	}

	final public synchronized void put(T o)
	{
		count++;
		oq.addLast(o);
		notify();
	}

	final public synchronized void push(T o)
	{
		oq.addFirst(o);
		notify();
	}

	final public int size()
	{
		return oq.size();
	}

	// Lecture blocante
	final public synchronized T remove()
	{
		if(oq.size() == 0)
		{
			try { wait() ; }
			catch (InterruptedException e) { e.printStackTrace(); }
		}

		return oq.removeFirst();
	}

	// Lecture non-blocante
	final public synchronized T remove(long timeout)
	{
		if(oq.size() > 0) return oq.removeFirst();

		if(timeout>0)
		{
			try { wait(timeout) ; }
			catch (InterruptedException e) { }
		}

		if(oq.size() > 0) return oq.removeFirst();

		return null ;
	}

	final public synchronized boolean remove(T o)
	{
		return oq.remove(o) ;
	}

	// Lecture blocante
	final public synchronized T get()
	{
		if(oq.size() == 0)
		{
			try { wait() ; }
			catch (InterruptedException e) { e.printStackTrace(); }
		}

		return oq.getFirst();
	}

	// Lecture non-blocante
    //
	final public synchronized T get(long timeout)
	{
		if(oq.size() > 0) return oq.getFirst();

		if(timeout>0)
		{
			try { wait(timeout) ; }
			catch (InterruptedException e) { }
		}
        else return null; // Avoid next test

		if(oq.size() > 0) return oq.getFirst();
		return null ;
	}
}
