/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.command.server.imp.queue ;

import java.util.*;

import org.azuki.framework.command.ICommand.Priority;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class OQP<T> 
{
    final private LinkedList<T> oq3 = new LinkedList<T>(); // Normal priority

    private LinkedList<T> oq1 = null ; // Max   priority (lazy)
    private LinkedList<T> oq2 = null ; // High  priority (lazy)
    private LinkedList<T> oq4 = null ; // Low priority (lazy)
    private LinkedList<T> oq5 = null ; // Min priority (lazy)

    private int s1 = 0 ; // Faster acces than "if (o!=null) s1=o.size(); else s1=0;
    private int s2 = 0 ;
    private int s3 = 0 ;
    private int s4 = 0 ;
    private int s5 = 0 ;

	private int count = 0 ; // conteur de message recu

	private int max = 999999999; // Default to no limit
	private boolean shift = false;
		 // if shift then the first object is remove when max is reached

	final public synchronized void setLimit(int max)
	{
		this.max = max ;
	}
	
	final public synchronized int getLimit()
	{
		return max;
	}
	
	final public synchronized void setShift(boolean shift)
	{
		this.shift = shift ;
	}

    final public synchronized List<T> asList()
    {
        List<T> list = new LinkedList<T>() ;

        if(oq1!=null) list.addAll(oq1) ;
        if(oq2!=null) list.addAll(oq2) ;
                      list.addAll(oq3) ;
        if(oq4!=null) list.addAll(oq4) ;
        if(oq5!=null) list.addAll(oq5) ;

        return list ;
    }

    @SuppressWarnings("unchecked")
	final public synchronized List<T> asList(Priority p)
    {
        if(p==Priority.NORMAL) return (List<T>)oq3.clone() ; // Gives a shallow copy to  prevent modification

        if(oq1!=null && p==Priority.HIGHEST) return (List<T>)oq1.clone() ;
        if(oq2!=null && p==Priority.HIGH)    return (List<T>)oq2.clone() ;
        if(oq4!=null && p==Priority.LOW)     return (List<T>)oq4.clone() ;
		if(oq5!=null && p==Priority.LOWEST)  return (List<T>)oq5.clone();
        return new LinkedList<T>() ;
    }

    final public synchronized void clear()
    {
        if(oq1!=null) { oq1.clear(); s1 = 0 ;}
        if(oq2!=null) { oq2.clear(); s2 = 0 ;}
                        oq3.clear(); s3 = 0 ;
        if(oq4!=null) { oq4.clear(); s4 = 0 ;}
        if(oq5!=null) { oq5.clear(); s5 = 0 ;}
    }

    final public synchronized void clear(Priority p)
    {
        if(p==Priority.NORMAL) { oq3.clear(); s3 = 0 ; return; }

        if(oq1!=null && p==Priority.HIGHEST)  { oq1.clear(); s1 = 0 ; return; }
        if(oq2!=null && p==Priority.HIGH) { oq2.clear(); s2 = 0 ; return; }
        if(oq4!=null && p==Priority.LOW)  { oq4.clear(); s4 = 0 ; return; }
        if(oq5!=null && p==Priority.HIGHEST)  { oq5.clear(); s5 = 0 ; return; }
    }

    final public void put(T o)
    {
        put(o, Priority.NORMAL) ;
    }

    final public void push(T o)
    {
        put(o, Priority.HIGHEST) ;
    }
    
    final public synchronized void put(T o, Priority p)
    {
        if(p==Priority.NORMAL) // Normal
        {
        	if(max>size())
        	{
        		oq3.addLast(o);
	            s3++; 
	      	}
        	else
        	{	if(shift) shift(o, p) ;
        		else insert(o, p) ;
        	}
        }
        
        else if(p==Priority.HIGHEST)
        {
            if(oq1==null) oq1 = new LinkedList<T>() ;
        
			if(max>size())
			{
	            oq1.addLast(o);
	            s1++;
			}
			else
			{	if(shift) shift(o, p) ;
				else insert(o, p) ;
			}
        }
        else if(p==Priority.HIGH)
        {
            if(oq2==null) oq2 = new LinkedList<T>() ;

			if(max>size())
			{
	            oq2.addLast(o);
	            s2++;
			}
			else
			{	if(shift) shift(o, p) ;
				else insert(o, p) ;
			}
        }
        else if(p==Priority.LOW)
        {
            if(oq4==null) oq4 = new LinkedList<T>() ;

			if(max>size())
			{
	            oq4.addLast(o);
	            s4++;
			}
			else
			{	if(shift) shift(o, p) ;
				else insert(o, p) ;
			}
        }
        else if(p==Priority.LOWEST)
        {
            if(oq5==null) oq5 = new LinkedList<T>() ;

			if(max>size())
			{
	            oq5.addLast(o);
	            s5++;
			}
			else
			{	if(shift) shift(o, p) ;
				else insert(o, p) ;
			}
       }

	   count++;
       notify();// Race condition between readers...
    }

	final private void shift(T o, Priority p)
	{
		removeNext() ;

		if(p==Priority.NORMAL) // test start with normal priority
		{	oq3.addLast(o);
			s3++;
			return ;
		}
		if(p==Priority.HIGH)
		{	oq2.addLast(o);
			s2++;
			return ;
		}
		if(p==Priority.HIGHEST)
		{	oq1.addLast(o);
			s1++;
			return ;
		}
		if(p==Priority.LOWEST)
		{	oq5.addLast(o);
			s5++;
			return ;
		}
		if(p==Priority.LOW)
		{	oq4.addLast(o);
			s4++;
			return ;
		}
	}

	final private void insert(T o, Priority p)
	{
		if(p==Priority.NORMAL) // test start with normal priority
		{
			if(s5>0)
			{
				oq5.removeLast() ; 
				s5--;
				put(o, p) ;
				return ;
			}
			if(s4>0)
			{
				oq4.removeLast() ; 
				s4--;
				put(o, p) ;
				return ;
			}
			return ; // o est le maillon faible !
		}

		if (p==Priority.LOWEST) return ; // o est le maillon faible !
		if (p==Priority.LOW)
		{
			if(s5>0)
			{
				oq5.removeLast() ; 
				s5--;
				put(o, p) ;
				return ;
			}
			return ; // o est le maillon faible !
		}
		if(p==Priority.HIGH)
		{
			if(s5>0)
			{
				oq5.removeLast() ; 
				s5--;
				put(o, p) ;
				return ;
			}
			if(s4>0)
			{
				oq4.removeLast() ; 
				s4--;
				put(o, p) ;
				return ;
			}
			if(s3>0)
			{
				oq3.removeLast() ; 
				s3--;
				put(o, p) ;
				return ;
			}
			return ; // o est le maillon faible !
		}

		if(p==Priority.HIGHEST) // Max priority
		{
			if(s5>0)
			{
				oq5.removeLast() ; 
				s5--;
				put(o, p) ;
				return ;
			}
			if(s4>0)
			{
				oq4.removeLast() ; 
				s4--;
				put(o, p) ;
				return ;
			}
			if(s3>0)
			{
				oq3.removeLast() ; 
				s3--;
				put(o, p) ;
				return ;
			}
			if(s2>0)
			{
				oq2.removeLast() ; 
				s2--;
				put(o, p) ;
				return ;
			}
			return ; // o est le maillon faible !
		}
	}

    final public int size()
    {
        return (s1+s2+s3+s4+s5) ;
    }
    
	final public int count() { return count ; } // total queued

    // Lecture blocante
    final public synchronized T remove()
    {
        if((s1+s2+s3+s4+s5) > 0) return removeNext();

        try { wait() ; }
        catch (InterruptedException e) { e.printStackTrace(); }

        return removeNext() ; // Always not null if wake up !
    }

    final private T removeNext()
    {
        if(s1>0) { s1--; return oq1.removeFirst() ; }
        if(s2>0) { s2--; return oq2.removeFirst() ; }
        if(s3>0) { s3--; return oq3.removeFirst() ; }
        if(s4>0) { s4--; return oq4.removeFirst() ; }
        if(s5>0) { s5--; return oq5.removeFirst() ; }
        return null;
    }

    // Lecture non-blocante
    final public synchronized T remove(long timeout)
    {
        if((s1+s2+s3+s4+s5) > 0) return removeNext();

        if(timeout>0)
        {
            try { wait(timeout) ; }
            catch (InterruptedException e) { }
        }
        else return null; // Avoid next test

        return removeNext(); // maybe null
    }

    final public synchronized boolean remove(T o)
    {
        if(oq3.remove(o)) { s3=oq3.size(); return true ; }

        if(oq1!=null && oq1.remove(o)) { s1=oq1.size(); return true ; }
        if(oq2!=null && oq2.remove(o)) { s2=oq2.size(); return true ; }
        if(oq4!=null && oq4.remove(o)) { s4=oq4.size(); return true ; }
        if(oq5!=null && oq5.remove(o)) { s5=oq5.size(); return true ; }

        return false ; // Not found
    }

    // Lecture blocante
    final public synchronized T get()
    {
        if((s1+s2+s3+s4+s5) > 0) return getNext();

        try { wait() ; }
        catch (InterruptedException e) { e.printStackTrace(); }

        return getNext(); // Never return null normally !
    }

    final private T getNext()
    {
        if(s1>0)  return oq1.getFirst() ;
        if(s2>0)  return oq2.getFirst() ;
        if(s3>0)  return oq3.getFirst() ;
        if(s4>0)  return oq4.getFirst() ;
        if(s5>0)  return oq5.getFirst() ;
        return null;
    }

    // Lecture non-blocante
    final public synchronized T get(long timeout)
    {
        if((s1+s2+s3+s4+s5) > 0) return getNext();

        if(timeout>0)
        {
            try { wait(timeout) ; }
            catch (InterruptedException e) { }
        }
        else return null ; // // Avoid next test

        return getNext(); // maybe null
    }
}
