/*
 *   Created the 1 August 2007
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.compiler ;

import java.io.File;
import java.util.*;

import javax.tools.*;
import javax.tools.JavaCompiler.CompilationTask;

import org.azuki.bean.compiler.server.imp.*;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class CompilerBean
{
	// Sun's doc says it's thread safe
	final static JavaCompiler compiler = ToolProvider.getSystemJavaCompiler() ;
	
	final static RAMClassLoader cl = new RAMClassLoader() ;
	
	@Export
	@MethodInfo(desc="Compile source code")	
	// name ex :  com.societe.projet.myClass
	final public Class compile(@ArgInfo(name="name", desc="The class name") String name,
							   @ArgInfo(name="src", desc="The source code") String src,
							   @ArgInfo(name="cpat", desc="The compilation classpath") List<String> cpat)
	throws Exception
	{
		List<String> opts = null ;
		if(cpat!=null)
		{
			StringBuilder bdr = new StringBuilder() ;
			bdr.append(StringUtil.toString(cpat, File.pathSeparator)) ;
			bdr.append(File.pathSeparator) ;
			bdr.append(StringUtil.toString(getRuntimeClasspath(), File.pathSeparator)) ;
			
			opts = Arrays.asList("-cp", bdr.toString()) ;
		}
		
		DiagnosticCollector<JavaFileObject> diags = new DiagnosticCollector<JavaFileObject>() ;	
		
		JavaFileManager jfm = new RAMFileManager(compiler.getStandardFileManager(diags, null, null), cl) ;
		
		String file = name.replace('.', '/')+".java" ; 
		
		JavaFileObject jfo = new JavaObjectFromString(file, src) ;
		CompilationTask task = compiler.getTask(null, jfm, diags, opts, null, Arrays.asList(jfo)) ;

		// Compilation
		if(task.call().booleanValue())
			return Class.forName(name, true, cl) ;
		
		try
		{ jfm.close() ; }
		catch(Exception ex)
		{ Log.severe(Log.getStack(ex)) ; }
		
		// Print all the information here.
		for(Diagnostic<? extends JavaFileObject> d : diags.getDiagnostics())
			Log.severe(d.toString()) ;			

		throw new Exception("Compilation fails.") ;
	}
	
	final static public List<String> getRuntimeClasspath()
	{
		String cp = System.getProperty("java.class.path") ;
		return StringUtil.asList(cp, File.pathSeparator) ;
	}
}