/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.context ;

import java.util.*;

import org.azuki.bean.context.server.ContextObjectAdvice;
import org.azuki.bean.context.server.imp.Stub;
import org.azuki.bean.context.shared.ConstContext;
import org.azuki.common.editor.*;
import org.azuki.common.util.ProcessLifecycle;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.context.IContext.IStub;
import org.azuki.framework.object.ILifecycle;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class ContextParameterBean implements IAspect<ContextObjectAdvice>
{
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public ContextObjectAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		// Get and add new variables declared in the bean Global.
		List<Map<String,String>> param = EditorUtil.tableToList(ipc.getParameter(), TagCommon.ROW);		

		Map<String, IStub> stubs = new HashMap<String,IStub>() ;
		
		for (Map<String,String> datas : param)
		{
			String   name = datas.get(AttrCommon.NAME) ;
			Object   valu = datas.get(AttrCommon.VALU) ;
			
			String sval = datas.get(ConstContext.SCOP) ;
			boolean  scop = sval==null || sval.equals(ConstContext.SCOP_PROCESS) ; // Inherited or not
			
			ILifecycle life = null ;
			if(scop) life = new ProcessLifecycle() ;
			
			stubs.put(name, new Stub(valu, life)) ;
		}
		
		return new ContextObjectAdvice(stubs) ;
	}	
}