/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.context ;

import org.azuki.bean.context.server.ObjectFactoryAdvice;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.object.*;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class ObjectFactoryBean implements IAspect<ObjectFactoryAdvice>
{	
	private IMetadata meta = null ; // cached
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	// Ne genere pas de code advice (ou alors un petit pour monitoring)
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public ObjectFactoryAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		IMetadata tmeta = ipc.getTargetMetadata() ;
		Object target = tmeta.getBean();
		
		getMetadata().getContract().checkBindable(target.getClass()) ;
		
		IObjectFactory irf = (IObjectFactory)target;

		String onam = irf.getObjectName() ;
		if(onam==null || onam.trim().isEmpty())
			throw new Exception("Invalid value for object name: "+onam) ;
		
		Locator.addObjectFactory(onam, irf) ;
		
		Log.info("\t (*) Adding ("+onam+") Thread's Context Object Factory") ;
		
		return new ObjectFactoryAdvice(onam) ;
	}	
	
	/* Not used 
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		ObjectFactoryConsole.service(srv, req, res) ;
	}
	*/
}