/*
 *   Created the 1 August 2007
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.cxf.server.imp ;

import java.lang.reflect.Method;
import java.util.Collection;

import org.azuki.bean.cxf.CXFWSBean.Service;
import org.azuki.common.reflect.*;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class PSource
{
	final static public String mkCode(String clas, Collection<Service> srvs, String pkag)
	{
		StringBuilder code = new StringBuilder() ;
		
		if(pkag!=null)
			code.append("package ").append(pkag).append(";\n") ;
		
		code.append("import javax.jws.*;\n");
		code.append("import org.azuki.framework.injection.IMethodProxy;\n");
		code.append("import org.azuki.framework.object.Locator;\n\n");
		
		code.append("@WebService\n") ;
		code.append("public class ") ;
		code.append(clas).append("\n{\n") ;
		
		// TODO : Gerer les conflits de nom de method
		for(Service serv : srvs)
			mkLine(code, serv) ;
		
		code.append("}") ;
		
		return code.toString() ;
	}
	
	final static public void mkLine(StringBuilder code, Service serv)
	{
		Method meth = serv.meth ;
		
		// Info method
		Class<?> rtyp = meth.getReturnType() ;
		String   rnam = ReflectUtil.getClassName(rtyp) ;
		
		// JAX-WS required annotation
		code.append("@WebMethod").append('\n') ;
		
		// Signature
		code.append("final public ").append(rnam).append(" ").append(serv.serv).append("(") ;
		
		boolean first = true ;
		
		Parameter[] para = Annotations.getParameters(meth) ;
		for(Parameter p : para)
		{
			if(!first) code.append(", ") ;
			first = false ;
			
			String pcla = ReflectUtil.getClassName(p.getType()) ;
			
			code.append("@WebParam(name=\"").append(p.getName()).append("\") ") ;
			code.append(pcla).append(' ').append(p.getName()) ;
		}
		
		code.append(")\n") ;
		code.append("throws Exception\n{\n\t") ;
		
		// Appel
		if(rtyp!=void.class)
		{
			code.append("return ") ;
			
			// cast
			if(!rtyp.equals(Object.class))
			{
				String cast = ReflectUtil.getClassName(rtyp) ;
				if(rtyp.isPrimitive())
				{
					Class<?> tmp = ReflectUtil.getBoxedClass(rtyp) ;
					cast = tmp.getName() ;
				}
				
				code.append("(").append(cast).append(") ") ;	
			}
		}
		
		code.append("((IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, \"");
		code.append(serv.name).append("\", \"").append(serv.labl).append("\", \"").append(meth.getName()).append("\")).call(") ;
		
		first = true ;
		
		for(Parameter p : para)
		{
			if(!first) code.append(", ") ;
			first = false ;
			
			code.append(p.getName()) ;
		}
		
		code.append(");\n}\n");
	}
}