/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.documentation.server.imp ;

import java.util.*;

import javax.servlet.http.HttpServletRequest;

import org.azuki.bean.documentation.DocumentationBean;
import org.azuki.bean.documentation.server.imp.resource.RsrcDocumentation;
import org.azuki.common.console.html.HtmlBuilder;
import org.azuki.common.framework.*;
import org.azuki.common.icon.CommonIcons;
import org.azuki.common.util.StringUtil;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class DocumentationMenu
{
	final static public String p(DocumentationBean da, HttpServletRequest req)
	throws Exception
	{
		try
		{
			return pp(da, req) ;
		}
		catch(Exception e)
		{
			e.printStackTrace() ;
			throw e ;
		}
	}
	
	final static public String pp(DocumentationBean da, HttpServletRequest req)
	throws Exception
	{
		Collection<BeanDefinition> bds = BeanRepository.instance().all() ;
		
		GroupNode root = new GroupNode("ROOT") ;
		
		for(BeanDefinition bd : bds)
		{
			String grup = bd.getGroup() ;
			if(grup==null)
			{	root.addBean(bd) ;
				continue ; }
			
			GroupNode beanGrupNode = root ;
			
			StringTokenizer st = new StringTokenizer(grup, ".") ;
			while(st.hasMoreTokens())
				beanGrupNode = beanGrupNode.addGroup(st.nextToken()) ;
			
			beanGrupNode.addBean(bd) ;
		}
		
		HtmlBuilder bdr = new HtmlBuilder() ;
		
		bdr.println("<table>") ;
		
		for(GroupNode grup : root.getGroups())
		{
			bdr.println("<tr><td class=\"menu\">") ;
			
			bdr.println("<table class=\"menu\">") ;
			bdr.print("<tr><th class=\"menu\">").print(grup.name).println("</th></tr>") ;
			bdr.println("<tr><td>") ;
			bdr.println(grup.contentAsString(da, req)) ;
			bdr.println("</td></tr>") ;
			bdr.println("</table>") ;
			
			bdr.println("</td></tr>") ;
		}
		
		for(BeanNode bean : root.getBeans())
		{
			bdr.println("<tr><td class=\"menu\">") ;
			bdr.println(bean.asString(da, req)) ;
			bdr.println("</td></tr>") ;
		}
		
		bdr.println("</table>") ;
	
		return bdr.toString() ; //root.asString(ca, req) ;
	} 
	
	final static class GroupNode implements Comparable<GroupNode>
	{
		final private SortedSet<GroupNode> tasks = new TreeSet<GroupNode>() ;
		final private SortedSet<BeanNode> beans = new TreeSet<BeanNode>() ;
		
		final public String name ;
		
		public GroupNode(String name)
		{
			this.name = name ;
		}
		
		final public Collection<GroupNode> getGroups()
		{
			return Collections.unmodifiableCollection(tasks) ;
		}
		
		final public Collection<BeanNode> getBeans()
		{
			return Collections.unmodifiableCollection(beans) ;
		}
	
		final private GroupNode getGroup(String name)
		{
			for(GroupNode task : tasks)
				if(task.name.equals(name)) return task ;
			return null ;
		}
		
		final public GroupNode addGroup(String name)
		{
			GroupNode task = getGroup(name) ;
			if(task==null)
			{	task = new GroupNode(name) ;
				tasks.add(task) ; }
			
			return task ;
		}
		
		final public void addBean(BeanDefinition bdef)
		{
			beans.add(new BeanNode(bdef)) ;
		}
		
		@Override
		public boolean equals(Object obj)
		{
			return obj instanceof GroupNode
					? name.equals(((GroupNode)obj).name) 
					: super.equals(obj) ;
		}
		
		public String asString(DocumentationBean da, HttpServletRequest req)
		{
			HtmlBuilder bdr = new HtmlBuilder() ;
			bdr.println("<table>") ;
			bdr.println("<tr>") ;
			bdr.print("<td><img src=\"").print(DocumentationUtil.mkRsrcURL(da, req, RsrcDocumentation.FOLDER_PNG)).println("\"/></td>") ;
			bdr.print("<td>").print(name).println("</td>") ;
			bdr.println("</tr>") ;
			bdr.println("<tr>") ;
			bdr.println("<td/>") ;
			bdr.println("<td>") ;
			bdr.println(contentAsString(da, req)) ;
			bdr.println("</td>") ;
			bdr.println("</tr>") ;
			bdr.println("</table>") ;
			
			return bdr.toString() ;
		}

		final public String contentAsString(DocumentationBean da, HttpServletRequest req)
		{
			HtmlBuilder bdr = new HtmlBuilder() ;
			
			bdr.println("<table>") ;
			
			for(GroupNode task : tasks)
			{
				bdr.println("<tr>") ;
				bdr.println("<td>") ;
				bdr.println(task.asString(da, req)) ;
				bdr.println("</td>") ;
				bdr.println("</tr>") ;
			}
			
			for(BeanNode bean : beans)
			{
				bdr.println("<tr>") ;
				bdr.println("<td>") ;
				bdr.println(bean.asString(da, req)) ;
				bdr.println("</td>") ;
				bdr.println("</tr>") ;
			}
			
			bdr.println("</table>") ;
			
			return bdr.toString() ;
		}
		
		/*
		 * Comparable
		 */
		final public int compareTo(GroupNode tn)
		{
			return name.compareTo(tn.name) ;
		}
	}
	
	final static class BeanNode implements Comparable<BeanNode>
	{
		final public BeanDefinition bdef ;
		
		public BeanNode(BeanDefinition bdef)
		{
			this.bdef = bdef ;
		}
		
		final public String asString(DocumentationBean da, HttpServletRequest req)
		{
			String sico = bdef.getIcon() ;
			if(StringUtil.isEmpty(sico)) sico = CommonIcons.BEAN_22 ;
			
			HtmlBuilder bdr = new HtmlBuilder() ;
			
			bdr.println("<table cellspacing=\"2\" cellpadding=\"0\">") ;
			bdr.println("<tr>") ;
			
			bdr.print("<td>") ;
			try
			{ bdr.print("<img src=\"").print(DocumentationUtil.mkRsrcURL(da, req, sico)).print("\" width=\"22\" height=\"22\" />") ; }
			catch(Exception ex){}			
			bdr.print("</td>") ;
			bdr.println() ;
			
			bdr.print("<td>") ;
			bdr.print("<a href=\"").print(DocumentationUtil.mkBeanURL(da, req, bdef)).print("\">") ;
			bdr.print(bdef.getName()) ;
			bdr.print("</a>") ;
			bdr.print("</td>") ;
			bdr.println() ;
			
			bdr.println("</tr>") ;
			bdr.println("</table>") ;
			
			return bdr.toString() ;
		}

		/*
		 * Comparable
		 */
		final public int compareTo(BeanNode bn)
		{
			return bdef.getName().compareTo(bn.bdef.getName()) ;
		}
	}
}