package org.azuki.bean.documentation.server.imp ;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.*;

import org.azuki.bean.documentation.DocumentationBean;
import org.azuki.bean.documentation.server.imp.resource.RsrcDocumentation;
import org.azuki.common.beandoc.BeanDocumentation;
import org.azuki.common.console.*;
import org.azuki.common.console.http.HttpRequest;
import org.azuki.common.framework.*;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.Framework;
import org.azuki.framework.context.Log;

public class PService
{
	final static public void doMain(DocumentationBean da, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		Template tmpl = new Template(RsrcDocumentation.getInputStream("portal.html")) ;
		tmpl.replace("${portal:name}", "Azuki on-line documentation") ;
		tmpl.replace("${portal:title}", "Azuki on-line documentation") ;
		tmpl.replace("${portal.css}", DocumentationUtil.mkRsrcURL(da, req, RsrcDocumentation.CSS)) ;
		tmpl.replace("${portal.js}", DocumentationUtil.mkRsrcURL(da, req, RsrcDocumentation.JS)) ;
		tmpl.replace("${favicon.ico}", DocumentationUtil.mkRsrcURL(da, req, RsrcDocumentation.FAVICO_ICO)) ;
		
		Template.Part portalHtmlTop = tmpl.getFirst() ;
		Template.Part portalHtmlBot = tmpl.getLast() ;
		
		String menu = DocumentationMenu.p(da, req) ;
		portalHtmlTop.replace("${portal:menu}", menu) ;
		portalHtmlTop.replace("${portal:notice}", "Azuki framework/"+Framework.getVersion()) ;
		
		res.setCharacterEncoding("utf-8") ;
		res.setContentType(HttpRequest.isIE(req) ? "text/html" : "application/xhtml+xml") ;
		
		ServletOutputStream sos = res.getOutputStream() ;
		sos.println(portalHtmlTop.toString()) ;
		
		doBean(da, srv, req, res) ;
		
		sos.println(portalHtmlBot.toString()) ;
	}
	
	final static public void doBean(DocumentationBean da, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		String ruri = DocumentationUtil.relativeURI(da, req) ;
		
		String name = ruri.substring(ruri.lastIndexOf('/')+1) ;
		if(StringUtil.isEmpty(name)) return ;
		
		ServletOutputStream sos = res.getOutputStream() ;
		
		BeanDefinition bdef = BeanRepository.instance().find(name) ;
		if(bdef==null)
		{	sos.println("Bean not found: "+name) ;
			return ; }
		
		try
		{
			String html = BeanDocumentation.asHtml(bdef) ;
			
			sos.write(html.getBytes("utf-8")) ;
		}
		catch(Throwable t)
		{
			sos.println("<pre><![CDATA[") ;
			sos.println(Log.getStack(t)) ;
			sos.println("]]></pre>") ;
		}		
	}
	
	final static public void doResource(DocumentationBean da, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		String ruri = req.getRequestURI() ;
		
		int idx = ruri.indexOf(DocumentationUtil.rsrcCtx) ;
		if(idx==-1) return ; // impossible !
		
		String rnam = StringUtil.unslashed(ruri.substring(idx+1)) ;
		
		String ctyp = srv.getServletContext().getMimeType(rnam) ;
		byte[] byts = Resource.getBytes(PService.class.getClassLoader(), rnam) ;
		
		// pas de "Content-Length": possible pb d'encodage == taille erronee !!
		if(ctyp!=null) res.setContentType(ctyp) ;
		res.getOutputStream().write(byts) ;
	}	
}