/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.event ;

import java.lang.reflect.Method;
import java.util.*;

import org.azuki.bean.event.server.EventHandlerAdvice;
import org.azuki.bean.event.server.imp.EventHandler;
import org.azuki.bean.event.shared.*;
import org.azuki.common.consts.Tag;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.command.ICommand.Priority;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.Element;

/*
        <pointcut name="EventHandlerBean">
          <para />
          <method name="processEvent">
            <para>
              <priority name="high" />
              <topic name="TOPIC1" />
      	      <link name="bean origine" label="label bean origin" ouput="nom sortie bean origin" />
      	    </para>
          </method>
        </pointcut>
*/

//
// Rem : name (et label) sont ajouté seulement si on coche "bean event"
//

final public class EventHandlerBean implements IAspect<EventHandlerAdvice>
{
	@DependencyInjectionInfo(desc="The method used to register a topic's listener", param={String.class, Object.class})
	public IMethodProxy<Void> register ; 

	//
	// TODO : Console : afficher la liste des listeners et aussi des bean a l'ecoute
	//

	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public EventHandlerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		Method[] methods = ipc.getSelectedMethods() ;
		
		if(methods.length<1) 
			throw new Exception("No method selected") ;

		List<EventHandler> list = mkListeners(ipc) ;
		
		return new EventHandlerAdvice(list) ;

	}
	
	// Rem : On listener for each method
	final private List<EventHandler> mkListeners(IPointcut ipc)
	throws Exception
	{
		List<EventHandler> list = new ArrayList<EventHandler>() ;
	
		for(Method meth : ipc.getSelectedMethods())
		{
			EventHandler listener = mkListener(ipc, meth) ;
			if(listener!=null)
				list.add(listener) ;
		}
		
		return list ;
	}
	
	@SuppressWarnings("unchecked")
	final private EventHandler mkListener(IPointcut ipc, Method meth)
	throws Exception
	{
		Element lmen = ipc.getElement(meth) ;
		Element para = lmen.getChild(Tag.PARA) ;
		if(para==null) return null ; // no parameter !
		
		IMetadata meta = ipc.getTargetMetadata() ;
		
		EventHandler lsn = new EventHandler(meta, meth) ;
		
		// Priority
		///////////
		Element epty = para.getChild(TagEvent.PRIO) ;		
		if(epty!=null)
		{
			String spty = epty.getAttributeValue(AttrEvent.NAME, Priority.NORMAL.name()) ;
			
			Priority prio = Priority.NORMAL ;
			try
			{ prio = Priority.valueOf(spty) ; }
			catch(Exception ex)
			{ Log.warning("Invalid priority("+spty+"), use default") ; }
			
			lsn.setPriority(prio) ;
		}
		
		// Register the advice to topic (bean output and global topic)
		// -----------------------------------------------------------

		// Topic
		////////
		List<Element> list = para.getChildren(TagEvent.TOPC) ;
		for(Element lm : list)
		{
			String topicname = lm.getAttributeValue(AttrEvent.NAME) ;
		
			try
			{
				register.call(topicname, lsn) ;
			}
			catch(Exception ce) // Normally already tested
			{
				Log.severe(ce.getMessage()) ;
				return null ;
			}
		}

		// Link
		/////////
		list = para.getChildren(TagEvent.LINK) ;
		for(Element lm : list)
		{
			String name = lm.getAttributeValue(AttrEvent.NAME) ;
			String labl = lm.getAttributeValue(AttrEvent.LABL) ;
			String outp = lm.getAttributeValue(AttrEvent.OUTP) ;
			
			Integer bid = Bean.searchFirst(name, labl).getID(); 
			
			
			String topic = new StringBuilder("link#").append(bid).append("#").append(outp).toString() ;

			try
			{
				register.call(topic, lsn) ;
			}
			catch(Exception ce) // Normally already tested
			{
				Log.severe(ce.getMessage()) ;
				return null ;
			}
		}				
		
		return lsn ;
	}
	
	/* Before change
	final public EventHandlerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		Method[] methods = ipc.getSelectedMethods() ;
		
		if(methods.length<1)
			throw new Exception("No method selected") ;

//		if(methods.length>1)
//			throw new Exception("This aspect can be applied to one method only") ;
		
		Method sel = methods[0] ;
				
		EventHandlerAdvice lsn =  new EventHandlerAdvice(ipc.getAspectMetadata(), sel) ;

		Element para = ipc.getParameter() ;

		if(para==null) return lsn ; // no parameter !
		
		// Priority
		///////////
		Element epty = para.getChild(TagEvent.PRIO) ;		
		if(epty!=null)
		{
			String spty = epty.getAttributeValue(AttrEvent.NAME, Priority.NORMAL.name()) ;
			
			Priority prio = Priority.NORMAL ;
			try
			{ prio = Priority.valueOf(spty) ; }
			catch(Exception ex)
			{ Log.warning("Invalid priority("+spty+"), use default") ; }
			
			lsn.setPriority(prio) ;
		}
		
		// Register the advice to topic (bean output and global topic)
		// -----------------------------------------------------------

		// Topic
		////////
		List<Element> list = para.getChildren(TagEvent.TOPC) ;
		for(Element lm : list)
		{
			String topicname = lm.getAttributeValue(AttrEvent.NAME) ;
		
			try
			{
				register.call(topicname, lsn) ;
			}
			catch(Exception ce) // Normally already tested
			{
				Log.severe(ce.getMessage()) ;
				return null ;
			}
		}

		// Link
		/////////
		list = para.getChildren(TagEvent.LINK) ;
		for(Element lm : list)
		{
			String name = lm.getAttributeValue(AttrEvent.NAME) ;
			String labl = lm.getAttributeValue(AttrEvent.LABL) ;
			String outp = lm.getAttributeValue(AttrEvent.OUTP) ;
			
			Integer bid = Bean.searchFirst(name, labl).getID(); 
			
			
			String topic = new StringBuilder("link#").append(bid).append("#").append(outp).toString() ;

			try
			{
				register.call(topic, lsn) ;
			}
			catch(Exception ce) // Normally already tested
			{
				Log.severe(ce.getMessage()) ;
				return null ;
			}
		}				
		
		return lsn ;
	}
	*/
}