/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

/**
 *   @author Mathieu Jucker
 */

package org.azuki.bean.event;

import java.util.Map;

import org.azuki.common.editor.EditorUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;
import org.jdom.Element;

public class EventProducerBean
{
	// EventPublisherBean injection
	@DependencyInjectionInfo(desc="The method used to publish a message to an output connector", param={Object.class, String.class, Object.class})
	public IMethodProxy<Void> publish ;
	
	private String text = null ;
	
	final static private String OUTPUT_NAME = "out" ;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
    final public void postCreate(Element para)
	throws Exception
	{
		Map<String,String> lst = EditorUtil.formToMap(para) ;
		if(lst!=null) text = lst.get("text") ;
	}
    
    @Export
	@MethodInfo(desc = "Publish a 'message' into the 'out' ouput connector of this bean")	
	final public void publishEvent(
	@ArgInfo(name="message", desc="The message to be sent by this bean into his 'out' output connector") Object message)
	{
		try
		{
			publish.call(this, OUTPUT_NAME, message) ;
		}
		catch(Exception e)
		{
			e.printStackTrace() ;
			Log.severe(Log.getStack(e)) ;
		}
	}

    @Export
	@MethodInfo(desc = "Publish the message (defined as parameter) into the 'out' output connector of this bean" )
	final public void publishEventMsg()
	{
		publishEvent(text) ;
	}
}
