/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.event ;

import java.util.List;

import javax.servlet.http.*;

import org.azuki.bean.event.server.EventPublisherAdvice;
import org.azuki.bean.event.server.console.EventPublisherConsole;
import org.azuki.bean.event.server.shell.EventPublisherShell;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;

/*
  <pointcut pcid="pcEventPublisher" name="EventPublisherBean" rlvl="10" enab="yes">    
    <para>
      <output name="s1">
         <topic name="TOPIC1" />
         <topic name="TOPIC2" />
      </output>
      <output name="s2">
        ...
      </output>
    </para> 
  </pointcut>
*/

//
// Rem : name (et label) sont ajouté seulement si on coche "bean event"
//

final public class EventPublisherBean implements IAspect<EventPublisherAdvice>
{
	@DependencyInjectionInfo(desc="The method used to publish a message to a topic", param={String.class,Object.class})
	public IMethodProxy<Void> publish ;
	
	IMetadata meta = null ;
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public EventPublisherAdvice createAdvice(IPointcut ipc)
	throws Exception
	{		
		return new EventPublisherAdvice(ipc.getTargetMetadata().getID(), ipc.getParameter()) ;
	}

	@Export
	@MethodInfo(desc = "This method must be imported in order for a bean to publish a 'message' into one of his 'output' connector")	
	final public void publish(
			@ArgInfo(name="bean", desc="The bean component holding the specified 'output' connector") Object bean,
			@ArgInfo(name="output", desc="The name of the output connector")                          String output,
			@ArgInfo(name="message", desc="The message to be sent into the 'output' connector")       Object mess)
	throws Exception
    {
		IMetadata meta = Bean.getMetadata(bean) ;
		
		// Recuperation des codes advice de type output
		List<IPointcut> lipc = meta.getPointcuts("EventPublisher") ;
		
		for(IPointcut ipc : lipc)
		{
			EventPublisherAdvice epa = (EventPublisherAdvice)ipc.getAdvice() ;

			if(epa==null)
				throw new Exception("Problem with EventPublisherBean Advice.") ;
			
			List<String> topics = epa.getTopics(output) ;
			if(topics==null) continue ;

			for(String topic : topics)
			{
				publish.call(topic, mess) ;
			}
		}
    }	
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		EventPublisherConsole.service(Bean.getMetadata(this).getID(), srv, req, res) ;
	}
	
	@MethodInfo(desc="Build the list of all EventPublisherBean advice",
				retu="The advices listing")
	final public String listEventPublishers()
	throws Exception
	{
		return EventPublisherShell.listPointcuts(this) ;
	}
}