/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.event ;

import javax.servlet.http.*;

import org.azuki.bean.event.server.console.EventServiceConsole;
import org.azuki.bean.event.server.imp.*;
import org.azuki.bean.event.server.shell.EventServiceShell;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;

final public class EventServiceBean
{
	final private EventServiceImp imp = new EventServiceImp() ;

	@Export
	@MethodInfo(desc="Publish an object 'message' as a event to a 'topic'")
	final public void publish(
			@ArgInfo(name="topic"  , desc="The destination topic") String topic,
			@ArgInfo(name="message", desc="The message published") Object message)
    {
        imp.publish(topic, message) ;
    }

	@Export
	@MethodInfo(desc="Registers a 'listener' (implementing the IListener interface) to the 'topic'")
    final public void register(
			@ArgInfo(name="topic"   , desc="The source topic") String topic,
			@ArgInfo(name="listener", desc="The event listener") IListener listener)
    {
		Log.info("\t (*) Register topic="+topic) ;
    	imp.register(topic, listener) ;
    }

	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		EventServiceConsole.service(imp, srv, req, res) ;
	}
	
	@MethodInfo(desc="Build the list of all EventServiceBean advice",
				retu="The advices listing")
	final public String listTopics()
	throws Exception
	{
		return EventServiceShell.listPointcuts(imp) ;
	}	
}