/**
*   Created the 2 April 2006
*
*   Azuki Framework
*   Copyright 2006 the original author or authors.
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*   
*   Contact : support@azuki-framework.org
*
*   @author Yoann Perrin (yoann.perrin@gmail.com)
*/
package org.azuki.bean.event.editor ;

import java.awt.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;

import org.azuki.bean.event.editor.imp.*;
import org.azuki.common.editor.pointcut.AbstractPointcutEditor;
import org.jdom.Element;

@SuppressWarnings("serial")
public class EventPublisherEditor extends AbstractPointcutEditor implements ListSelectionListener
{
	protected JSplitPane splitPane ;
	
	protected OutputSelectionPane outpSelectionPane ;
	protected TopicSelectionPane topcSelectionPane ;
	
	/*
	 * AbstractPointcutEditor
	 */
	@Override
	final public void initParameters(Element para)
	{
		outpSelectionPane = new OutputSelectionPane(getContext(), para) ;
		outpSelectionPane.setPreferredSize(new Dimension(200, 200)) ;
		outpSelectionPane.addListSelectionListener(this) ;
		
		outpSelectionPane.setSelectionMode(ListSelectionModel.SINGLE_SELECTION) ;
		
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT) ;
		splitPane.setLeftComponent(outpSelectionPane) ;
		splitPane.setRightComponent(new JPanel()) ;
		splitPane.setDividerLocation(0.5f) ;
		splitPane.setBorder(BorderFactory.createEmptyBorder()) ;
		
		setDialogTitle("Event publisher editor") ;
		
		setBorder(BorderFactory.createEmptyBorder(0,5,5,5)) ;
		setPreferredSize(new Dimension(450,200)) ;
		add(splitPane, BorderLayout.CENTER) ;
	}
	
	@Override
	final public void initFields(List<Element> flds)
	{}

	@Override
	final public void initMethods(List<Element> mths)
	{}

	/*
	 * ListSelectionListener
	 */
	final public void valueChanged(ListSelectionEvent e)
	{
		if(e.getValueIsAdjusting()) return ;
		
		if(outpSelectionPane.isSelectionEmpty())
		{	splitPane.setRightComponent(new JPanel()) ;
			return ; }
		
		Element outpLmen = outpSelectionPane.getSelectedElement() ;
		
		if(topcSelectionPane==null)
		{
			topcSelectionPane = new TopicSelectionPane(getContext(), outpLmen) ;
			topcSelectionPane.setPreferredSize(new Dimension(200, 200)) ;
		}
		else
			topcSelectionPane.setElement(outpLmen) ;
		
		splitPane.setRightComponent(topcSelectionPane) ;
		splitPane.setDividerLocation(splitPane.getLastDividerLocation()) ;
	}
}