/**
*   Created the 2 April 2006
*
*   Azuki Framework
*   Copyright 2006 the original author or authors.
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*   
*   Contact : support@azuki-framework.org
*
*   @author Yoann Perrin (yoann.perrin@gmail.com)
*/
package org.azuki.bean.event.editor.imp ;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.azuki.common.consts.Attr;
import org.azuki.common.editor.IEditorContext;
import org.azuki.common.editor.jdom.JDOMAttributeListModel;
import org.azuki.common.icon.CommonIcons;
import org.jdom.Element;

@SuppressWarnings("serial")
public class TopicSelectionPane extends JPanel implements ActionListener
{
	final protected IEditorContext ectx ;
	
	protected Element lmen ;
	
	protected JLabel topcLabel ;
	
	protected JComboBox topcSelectionBox ;
	protected JLabel topcSelectionLabel ;
	protected JPanel topcSelectionMessage ;
	
	protected JDOMAttributeListModel topcListModel ;
	protected JList topcList ;
	protected JScrollPane topcListScroll ;
	
	protected JButton addButton ;
	protected JButton delButton ;
	protected JToolBar actionBar ;
	
	public TopicSelectionPane(IEditorContext ectx, Element lmen)
	{
		this.ectx = ectx ;
		this.lmen = lmen ;
		
		// topics selection
		topcSelectionBox = new JComboBox(new Object[0]) ;
		topcSelectionBox.setEditable(true) ;
		
		topcSelectionLabel = new JLabel("Enter topic name: ") ;
		topcSelectionLabel.setLabelFor(topcSelectionBox) ;
		
		topcSelectionMessage = new JPanel() ;
		topcSelectionMessage.add(topcSelectionLabel) ;
		topcSelectionMessage.add(topcSelectionBox) ;
		
		// topics list
		topcLabel = new JLabel("Topics:") ;
		topcLabel.setBorder(BorderFactory.createEmptyBorder(2,2,2,2)) ;
		
		topcListModel = new JDOMAttributeListModel(lmen, "topic", Attr.NAME) ;
		topcList = new JList(topcListModel) ;
		
		topcListScroll = new JScrollPane(topcList) ;
		
		// actions
		addButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_ADD_16)));
		addButton.addActionListener(this) ;
		
		delButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_REMOVE_16)));
		delButton.addActionListener(this) ;
		
		actionBar = new JToolBar(SwingConstants.VERTICAL) ;
		actionBar.setFloatable(false) ;
		actionBar.add(addButton) ;
		actionBar.add(delButton) ;
		
		setLayout(new BorderLayout()) ;
		add(topcLabel, BorderLayout.NORTH) ;
		add(topcListScroll, BorderLayout.CENTER) ;
		add(actionBar, BorderLayout.EAST) ;
	}
	
	final public void setElement(Element lmen)
	{
		this.lmen = lmen ;
		topcListModel.setElement(lmen) ;
	}
	
	/*
	 * ActionListener
	 */
	final public void actionPerformed(ActionEvent e)
	{
		Object src = e.getSource() ;
		
		if(src==addButton)
		{
			Collection<String> topics = ectx.getDeclaredTopics() ;
			topics.removeAll(topcListModel.getNames()) ;
			
			topcSelectionBox.setModel(new DefaultComboBoxModel(topics.toArray())) ;
			topcSelectionBox.setSelectedIndex(-1) ;
			
			int opt = JOptionPane.showConfirmDialog(this, topcSelectionMessage, "New topic", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE) ;
			if(opt==JOptionPane.OK_OPTION)
				topcListModel.addItem((String)topcSelectionBox.getSelectedItem()) ;
		}
		else if(src==delButton)
		{
			int[] indexes = topcList.getSelectedIndices() ;
			
			Arrays.sort(indexes) ;
			
			for(int i=indexes.length-1 ; i>=0 ; i--)
				topcListModel.removeItem(indexes[i]) ;
			
			topcList.clearSelection() ;
		}
	}
}