/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.event.server.console;

import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.event.server.EventPublisherAdvice;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.Table;
import org.azuki.common.console.http.HttpResponse;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

public class EventPublisherConsole
{
	final static public String[] tableHeaders = {"Bean", "Output", "Topic"} ;
	
	final public static void service(Integer aspectID, HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		List<List> data = new ArrayList<List>() ;
		
		for (IPointcut pc : Bean.getPointcuts(aspectID))
		{
			EventPublisherAdvice advc = (EventPublisherAdvice)pc.getAdvice() ;
			if(advc==null) continue ;
			
			IMetadata tmeta = pc.getTargetMetadata() ;
			
			Map<String,List<String>> mapOutpTopc = advc.getTopics() ;
			
			for(Map.Entry<String,List<String>> ety : mapOutpTopc.entrySet())
			{
				String outp = ety.getKey() ;
				
				for(String topc : ety.getValue())
					data.add(ListUtil.toList(Util.oneString(tmeta.getName(), tmeta.getLabel()), outp, topc)) ;
			}
		}
		
		Table table = HtmlFactory.sortedTable(req, data, tableHeaders) ;
		
		HttpResponse.print(res, table) ;
	}
}
