/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.event.server.imp ;

import java.lang.reflect.Method;

import org.azuki.common.util.Util;
import org.azuki.framework.bean.IMetadata;
import org.azuki.framework.command.Command;
import org.azuki.framework.command.ICommand.Priority;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.Locator;

final public class EventHandler implements IListener
{
	final private IMethodProxy mp ;
	
	final private IMetadata meta ;
	final private Method meth ;
	final private int nbargs ; // 0, 1, 2
	
	private Priority pty = Priority.NORMAL ;
	final public void setPriority(Priority pty) { this.pty = pty ; }
	
	public EventHandler(IMetadata meta, Method meth)
	throws Exception
	{
		this.meta = meta ;
		this.meth = meth ;
		
		nbargs = meth.getParameterTypes().length ;
	    
	    // Creation of all event handler (IListener)
		mp = (IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, meta.getName(), meta.getLabel(), meth.getName()) ;
	}
	
	@Override
	final public String toString()
	{
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(Util.oneString(meta.getName(), meta.getLabel())) ;
		bdr.append(Util.methodSeparatorChar).append(meth==null ? "null" : meth.getName()) ;
		return bdr.toString() ;
	}

	// IListener
    final public void processEvent(String topic, Object event)
    throws Exception 
    {
    	Command.sendQueued(new EventCommand(mp, nbargs, topic, event), meta.getQueue(), pty) ;
    }

	final public boolean isQueued() { return true; } // Already Queued
}