/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.event.server.imp ;

import java.util.*;

import org.azuki.framework.command.Command;

public class EventServiceImp
{
    final private HashMap<String, List<IListener>> llsn = new HashMap<String, List<IListener>>() ; 
	
    // For service... 
	final public Map<String, List<IListener>> getTopicListeners() { return llsn; }     
    
	// Topic management 
    final public void publish(String topic, Object mess)
    {
    	List<IListener> list = llsn.get(topic) ;
        if(list==null) return ; // No Listener

        for(IListener lsn : list)
        {
        	if(!lsn.isQueued())
        	{
            	// If listener not queued, then create a command
            	// to prevent processing lock
            	Command.mainQueue().put(new JEvent(lsn, topic, mess)) ;
            	continue ;
        	}

            try
			{
        		// This is the case for bean input
        		lsn.processEvent(topic, mess);           		
			}
        	catch(Exception e)
			{
        		e.printStackTrace() ;
			}
        }
    }
        
    // The listener key must be unique (can use hashcode or more explicit key).
    final public synchronized void register(String topic, IListener lsn)
    {
    	getListeners(topic).add(lsn) ;
    }

    // Unregister topic listener for current app only.
    final synchronized public void unregister(String topic, IListener lsn)
    {
	    // TODO : Il faudrait idealement supprimer les Queues qui ont été
        // crées pour ces listeners....
	    List<IListener> list = llsn.get(topic) ;
        if(list==null) return ;
        list.remove(lsn) ;
    }    

    
    // Permet aux "publisher" de s'enregistrer
    final private synchronized List<IListener> getListeners(String topic)
    {
	    List<IListener> list = llsn.get(topic) ;
        if(list==null)
        {   list = new ArrayList<IListener>() ;
        	llsn.put(topic, list) ; }
        return list ;
    }
}
