/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.exec.server.imp.mngr ;

import java.io.OutputStream;
import java.util.*;

import org.azuki.bean.exec.ProcessFactoryBean;
import org.azuki.bean.exec.server.imp.ExecUtil;

final public class PError
{
	final static private int SIZE = 100 ;
	
	// Last process with errors 
	final private Vector<GErr> errs = new Vector<GErr>() ;

	final private ProcessFactoryBean es ;
	
	public PError(ProcessFactoryBean es)
	{
		this.es = es ;
	}
	
	final public void add(Integer ID, String name, String[] cmd,
						String errmess,	long start, long end, Map<String, String> env)
	{
		if(errs.size()>=SIZE)
			errs.remove(0) ; // Remove first element
		
		GErr gerr = new GErr(ID, name, cmd, errmess, start, end, env) ;
		errs.add(gerr) ;			
	}
	
	final public List list()
	{
		ArrayList<Object[]> list = new ArrayList<Object[]>() ;
		
		Enumeration enu = errs.elements() ;
		while(enu.hasMoreElements())
		{
			GErr gerr = (GErr)enu.nextElement() ;
			Object[] o7 = new Object[7] ;
			
			o7[0] = gerr.ID ;
			o7[1] = gerr.name ;
			o7[2] = ExecUtil.toString(gerr.cmd) ;
			o7[3] = gerr.err ;
			o7[4] = new Long(gerr.star) ;
			o7[5] = new Long(gerr.end) ;
			o7[6] = gerr.env ;
			
			list.add(o7) ;
		}
		
		return list ;
	}
	
	final public void restartNoWait(Integer ID)
	{
		GErr gerr = find(ID) ;
		if(gerr==null) return ;
		
		// Path already set in cmd
		es.send(gerr.name, gerr.cmd, gerr.env, null); 
	}

	final public void restartWait(Integer ID, OutputStream out)
	throws Exception
	{
		GErr gerr = find(ID) ;
		if(gerr==null) return ;
		
		// Path already set in cmd
	    es.invokeStreamed(gerr.name, gerr.cmd, gerr.env, null, out) ;
	}
	
	final private GErr find(Integer ID)
	{
		Enumeration enu = errs.elements() ;
		while(enu.hasMoreElements())
		{
			GErr gerr = (GErr)enu.nextElement() ;
			if(ID.equals(gerr.ID)) return gerr ;
		}
		return null;
	}
}