/*
  GRANITE DATA SERVICES
  Copyright (C) 2007 ADEQUATE SYSTEMS SARL

  This file is part of Granite Data Services.

  Granite Data Services is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 3 of the License, or (at your
  option) any later version.
 
  Granite Data Services is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.
 
  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, see <http://www.gnu.org/licenses/>.
*/

package org.azuki.bean.graniteds.server.imp ;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.azuki.bean.graniteds.GraniteDSBean;
import org.azuki.bean.graniteds.shared.ConstGranite;
import org.azuki.framework.context.*;
import org.granite.config.GraniteConfig;
import org.granite.config.flex.ServicesConfig;
import org.granite.context.*;
import org.granite.messaging.amf.AMF0Message;
import org.granite.messaging.amf.io.*;
import org.granite.messaging.amf.process.AMF0MessageProcessor;
import org.granite.messaging.webapp.*;

/**
 * @author Franck Wolff (franck.wolf@graniteds.org )
 * @author Robert Bakic
 */

public class PService
{
    static private GraniteConfig graniteConfig = null;
    static private ServicesConfig servicesConfig = null;
	
    // filter part
	final static public void p(GraniteDSBean bean, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws ServletException
	{
        //Log.info(">> Incoming AMF0 request from: " + req.getRequestURL());

		ServletContext srvctx = srv.getServletContext() ;

		if(graniteConfig==null) // First time, todo : post-create
		{	
			graniteConfig = loadConfig(srvctx);	
			servicesConfig = loadSrvConfig(srvctx);
		}
		
		//Logger.getLogger(AMF3Deserializer.class).setLevel(Level.ALL) ;
		
        try 
        {
            GraniteContext context = HttpGraniteContext.createThreadIntance(
                graniteConfig, servicesConfig, srvctx, req, res);

            AMFContextImpl amf = (AMFContextImpl)context.getAMFContext();

            //Log.info(">> Deserializing AMF0 request...");

            AMF0Deserializer deserializer = new AMF0Deserializer(new DataInputStream(req.getInputStream()));
            AMF0Message amf0Request = deserializer.getAMFMessage();

            amf.setAmf0Request(amf0Request);

           	//Log.info(">> Chaining AMF0 request:" + amf0Request);
           	
           	// Servlet part
           	chain(bean, srv, req, res) ;
            
            AMF0Message amf0Response = amf.getAmf0Response();

            //Log.info("<< Serializing AMF0 response:" + amf0Response);

            res.setContentType(AMF0Message.CONTENT_TYPE);
            AMF0Serializer serializer = new AMF0Serializer(new DataOutputStream(res.getOutputStream()));
            serializer.serializeMessage(amf0Response);
        }
        catch(Exception e)
        {
            Log.severe(e);
            throw new ServletException(e);
        }
        finally
        {
            GraniteContext.release();
        }
	}

	final static private void chain(GraniteDSBean bean, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
        GraniteContext context = GraniteContext.getCurrentInstance(); // TODO : pass as parameter
    
        if (context == null)
            throw new ServletException(
                "No GraniteContext (" + AMFMessageFilter.class.getName() + " not configured in web.xml ?)");

        AMFContextImpl amf = (AMFContextImpl)context.getAMFContext();

        AMF0Message amf0Request = amf.getAMF0Request();

        // Log.info(">> Processing AMF0 request:" + amf0Request);

        
        // L'instance du bean "GraniteDS" est stocké dans le context d'execution 
        // afin que le PojoInvoker puisse savoir a partir de quelle instance de "GraniteDS"
        // il a été invoqué
		ThreadContext.get().bind(ConstGranite.BIND, bean, null) ;
        
        
        AMF0Message amf0Response = AMF0MessageProcessor.process(amf0Request);

       	// Log.info("<< Returning AMF0 response:" + amf0Response);

        amf.setAmf0Response(amf0Response);		
	}
	
	
    static private final String GRANITE_CONFIG_KEY = GraniteConfig.class.getName() + "_CACHE";
    
    final static private GraniteConfig loadConfig(ServletContext context)
    throws ServletException
    {
        GraniteConfig graniteConfig = (GraniteConfig)context.getAttribute(GRANITE_CONFIG_KEY);
        if (graniteConfig != null)
            return graniteConfig;

        // BOB
    	InputStream is = RscGranite.getInputStream("granite-config.xml") ;
    	
    	/*
        String path = context.getInitParameter("graniteConfigPath");
        if (path == null)
            path = "/WEB-INF/granite/granite-config.xml";
        
        InputStream is = context.getResourceAsStream(path);
        if (is == null) {
            log.warn("Could not load custom granite-config.xml: " + path + " (file does not exists)");
            path = null;
        }
        */
        try 
        {
            return GraniteConfig.loadConfig(is);
        }
        catch (Exception e) {
            throw new ServletException("Could not load custom granite-config.xml", e);
        }
    }	
    
    private static final String SERVICES_CONFIG_KEY = ServicesConfig.class.getName() + "_CACHE";
    final static private ServicesConfig loadSrvConfig(ServletContext context)
    throws ServletException
    {
        ServicesConfig servicesConfig = (ServicesConfig)context.getAttribute(SERVICES_CONFIG_KEY);
        if (servicesConfig != null)
            return servicesConfig;
        
        // BOB
    	InputStream is = RscGranite.getInputStream("services-config.xml") ;
        /*
        String path = context.getInitParameter("servicesConfigPath");
        if (path == null)
            path = "/WEB-INF/flex/services-config.xml";
        
        InputStream is = context.getResourceAsStream(path);
        if (is == null)
            throw new ServletException("Could not load: " + path + " (file does not exists)");
        */
        try {
            return ServicesConfig.loadConfig(is);
        }
        catch (Exception e) {
            throw new ServletException("Could not load custom granite-config.xml", e);
        }
    }    
}