/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.hibernate;

import java.io.InputStream;
import java.util.*;

import org.azuki.bean.hibernate.shared.ConstHibernate;
import org.azuki.common.consts.Attr;
import org.azuki.common.editor.*;
import org.azuki.framework.annotation.*;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.jdom.Element;

public class HibernateBean
{
	/** Hibernate Configuration properties **/	
	private Configuration conf;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		List<Map<String,String>> listprop = EditorUtil.tableToList(para, TagCommon.ROW) ;
		List<Map<String,String>> listmap  = EditorUtil.tableToList(para, ConstHibernate.MAP) ;
		
		/** configure Hibernate with properties **/
		Properties confProps = new Properties() ;
		
		for (Map<String,String> m : listprop)
			confProps.put(m.get(Attr.NAME), m.get(AttrCommon.VALU)) ;
		
		conf = new Configuration().setProperties(confProps) ;
		addResource(listmap) ;
	}
	
	final private void addResource(List<Map<String,String>> list)
	throws Exception
	{
		ClassLoader cl = Thread.currentThread().getContextClassLoader() ;
		
		for (Map<String,String> m : list)
		{
			String mapClass = m.get(ConstHibernate.FILE) ;
			InputStream i = cl.getResourceAsStream(mapClass) ;
			//TODO: si InputStream==NULL --> MESSAGE !
			
			try
			{ conf.addInputStream(i) ; }
			catch(Exception e)
			{ e.printStackTrace() ; }
		}
	}
	
	@Export
	final public SessionFactory getSessionFactory()
	throws Exception
	{
		return conf.buildSessionFactory();
	}
}