/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.injection;

import java.lang.reflect.Field;

import org.azuki.bean.injection.server.MethodProxyAdvice;
import org.azuki.bean.injection.server.imp.MethodProxyImp;
import org.azuki.bean.injection.shared.*;
import org.azuki.common.consts.*;
import org.azuki.common.util.Util;
import org.azuki.framework.aop.*;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.*;
import org.jdom.Element;

/**
 *   @author Robert Bakic
 */
final public class MethodProxyBean implements IAspect<MethodProxyAdvice>, IObjectFactory<IMethodProxy>
{
	final public String getObjectName()
	{
		return Locator.OBJECT_METHOD_PROXY ;
	}
	
	// if bean label==null returns the first instance found
	final public IMethodProxy createObject(Object... args)
	{
		int len = args.length ;
		// todo : check args
		
		String name = (String)args[0] ;
		String labl = (String)args[1] ;
		String meth = (String)args[2] ;
		
		boolean asyn = false ;
		if(len>3) asyn = ((Boolean)args[3]).booleanValue() ;

		boolean fork = false ;
		if(len>4) fork = ((Boolean)args[4]).booleanValue() ;
		
		return new MethodProxyImp(name, labl, meth, asyn, fork) ;
	}	

	/*
	<!--methodProxy attr="publish" name="EventBasedProcessing" labl="" meth="publish" asyn="no" fork="no" /-->
	<pointcut name="MethodProxy">
	  <field name="publish">
		<para>
		  <proxy name="EventBasedProcessing" labl="" meth="publish" asyn="no" fork="no" />
	    </para>
	  </field>
	</pointcut>        
	*/

	final public MethodProxyAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		Object target = ipc.getTargetMetadata().getBean() ;
		
		Field[] fds = ipc.getSelectedFields() ;
		
		for(Field fd : fds)
		{
			try
			{
				Element lmen = ipc.getElement(fd) ;
				lmen = lmen.getChild(Tag.PARA) ;
				if(lmen==null) continue ;
				lmen = lmen.getChild(TagMethodProxy.PROXY) ;
				if(lmen==null) continue ;
				
				String name = lmen.getAttributeValue(AttrMethodProxy.NAME) ;
				String labl = lmen.getAttributeValue(AttrMethodProxy.LABL, "") ;
				String meth = lmen.getAttributeValue(AttrMethodProxy.METH) ;
	
				// service() equals to send()
				String asynAttr = lmen.getAttributeValue(AttrMethodProxy.ASYN, Const.NO) ; // Asynchronous 
				boolean asyn = false ;
				if(Util.isOK(asynAttr))	asyn = true ;
				
				// service() equals to invoke()
				String forkAttr = lmen.getAttributeValue(AttrMethodProxy.FORK, Const.NO) ; // Preserve context 
				boolean fork = false ;
				if(Util.isOK(forkAttr)) fork = true ;
	
				IMethodProxy mp = new MethodProxyImp(name, labl, meth, asyn, fork) ;
				
				StringBuilder sb = new StringBuilder("\t(*) Proxy [") ;				
				sb.append(fd.getName()).append("] --> ").append(mp).append("()") ;
				if(asyn) sb.append(" (asyn)");
				if(fork) sb.append(" (fork)");
				
				Log.info(sb.toString()) ;
			
				fd.set(target, mp) ;
			}
			catch(Exception e)
			{
				Log.severe(Log.getStack(e)) ;
			}
		}
		
		return new MethodProxyAdvice();
	}	
}