/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.injection.editor.imp ;

import java.awt.BorderLayout;
import java.awt.event.*;

import javax.swing.*;

import org.azuki.bean.injection.shared.*;
import org.azuki.common.consts.*;
import org.azuki.common.editor.*;
import org.azuki.common.editor.page.Page;
import org.azuki.common.editor.weaver.MethodProxySelector;
import org.azuki.common.util.*;
import org.azuki.common.widget.JFormPanel;
import org.jdom.Element;

/**
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */
@SuppressWarnings("serial")
public class MethodProxyFieldPage extends Page implements ActionListener, ItemListener
{
	private IPointcutEditorContext pctx ;
	private String fnam ;
	private Element prxy ;
	
	JTextField bnamField ;
	JTextField blblField ;
	JTextField bmthField ;
	
	JButton chooseButton ;
	
	JCheckBox asynBox ;
	JCheckBox forkBox ;
	
	JFormPanel form ;
	
	/*
	 * Page
	 */
	@Override
	final public void init(IEditorContext ectx, Element para)
	{
		pctx = (IPointcutEditorContext)ectx ;							// editor context
		prxy = JDOMUtil.getElement(para, TagMethodProxy.PROXY) ;		// <proxy/> element
		
		//FIXME: a parametrer !!
		fnam = para.getParentElement().getAttributeValue(Attr.NAME) ;	// selected field
		
		bnamField = new JTextField() ;
		bnamField.setText(prxy.getAttributeValue(AttrMethodProxy.NAME)) ;
		bnamField.setEditable(false) ;
		
		blblField = new JTextField() ;
		blblField.setText(prxy.getAttributeValue(AttrMethodProxy.LABL)) ;
		blblField.setEditable(false) ;
		
		bmthField = new JTextField() ;
		bmthField.setText(prxy.getAttributeValue(AttrMethodProxy.METH)) ;
		bmthField.setEditable(false) ;
		
		chooseButton = new JButton("Choose method") ;
		chooseButton.addActionListener(this) ;
		
		asynBox = new JCheckBox() ;
		asynBox.setSelected(Util.isOK(prxy.getAttributeValue(AttrMethodProxy.ASYN))) ;
		asynBox.addItemListener(this) ;
		
		forkBox = new JCheckBox() ;
		forkBox.setSelected(Util.isOK(prxy.getAttributeValue(AttrMethodProxy.FORK))) ;
		forkBox.addItemListener(this) ;
		
		form = new JFormPanel() ;
		form.addField(bnamField, "Bean name") ;
		form.addField(blblField, "Bean label") ;
		form.addField(bmthField, "Bean method") ;
		form.addField(chooseButton) ;
		form.addField(asynBox, "Asynchonous ?") ;
		form.addField(forkBox, "Fork ?") ;
		
		setLayout(new BorderLayout()) ;
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		add(form, BorderLayout.NORTH) ;
	}
	
	/*
	 * ActionListener
	 */
	final public void actionPerformed(ActionEvent e)
	{
		String bnam = prxy.getAttributeValue(AttrMethodProxy.NAME) ;
		String blbl = prxy.getAttributeValue(AttrMethodProxy.LABL, "") ;
		String bmth = prxy.getAttributeValue(AttrMethodProxy.METH) ;
		
		MethodProxySelector mps = new MethodProxySelector(pctx, fnam) ;
		mps.setSelection(bnam, blbl, bmth) ;
		
		if(!mps.showDialog(this, "Select method ...")) return ;
		
		bnam = mps.getSelectedBeanName() ;
		blbl = mps.getSelectedBeanLabel() ;
		bmth = mps.getSelectedMethodName() ;
		
		bnamField.setText(bnam) ;
		prxy.setAttribute(AttrMethodProxy.NAME, bnam) ;
		
		blblField.setText(blbl) ;
		prxy.setAttribute(AttrMethodProxy.LABL, blbl) ;
		
		bmthField.setText(bmth) ;
		prxy.setAttribute(AttrMethodProxy.METH, bmth) ;
	}

	/*
	 * ItemListener
	 */
	final public void itemStateChanged(ItemEvent e)
	{
		if(e.getSource()==asynBox)
		{
			if(asynBox.isSelected()) 	prxy.setAttribute(AttrMethodProxy.ASYN, Const.YES) ;
			else						prxy.removeAttribute(AttrMethodProxy.ASYN) ;
		}
		else if(e.getSource()==forkBox)
		{
			if(forkBox.isSelected()) 	prxy.setAttribute(AttrMethodProxy.FORK, Const.YES) ;
			else						prxy.removeAttribute(AttrMethodProxy.FORK) ;
		}
	}
}