/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.javarpc.server.imp ;

import java.io.*;

import javax.servlet.http.*;

import org.azuki.client.javarpc.imp.PZip;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
public class Response
{
	final static public void  write(HttpServletRequest req, HttpServletResponse res, Object obj)
	throws IOException
	{
		if(obj==null)
		{	Response.write(req, res, HttpServletResponse.SC_NO_CONTENT, null, -1, null) ;
			return ; }

		byte[] content = PZip.serialize(obj); 

		write(req, res, HttpServletResponse.SC_OK, "application/octet-stream", System.currentTimeMillis(), content) ;
	}	
	
	final static private void write(HttpServletRequest req, HttpServletResponse res, int status, String ctype, long lmod, byte[] content)
	throws IOException
	{
		boolean cz = false ; // compressed
		boolean allowgzip = Http.allowGzip(req) ;
		boolean headOnly = req.getMethod().equals("HEAD") ;
		
		// Pas de compression si fichier zip
		if(ctype != null)
		{
			if(ctype.equals("application/zip")) allowgzip = false ;
			if(ctype.equals("image/jpeg")) allowgzip = false ; // BUG IE
		}

		if(!headOnly && allowgzip)
		{
			if(content!=null && PZip.COMPRESS > 0) // if 0 then no compression
			if(content.length >= PZip.COMPRESS) // si seuil atteint
			{
				content = PZip.compress(content) ;
				cz = true ;
			}
		}
		
		if(HttpServletResponse.SC_OK==status && content==null)
		    status = HttpServletResponse.SC_NO_CONTENT ;
		
		res.setStatus(status) ; // ex : 200 OK
		
		if(content!=null)
			res.setContentLength(content.length) ;
		
		res.setContentType(ctype) ;
		
		if(lmod>1) res.setDateHeader("Last-Modified", lmod) ;
		
		if(cz)	res.setHeader("Content-Encoding", "gzip") ;		
		
		// Utilisation d'un serveur Proxy-Cache
		// suppression de la mise en cache des infos.
		// Le proxy devra tester soit la date soit le ETag du fichier.
		//////////////////////////////////////////////////////////////
		if(req.getHeader("Via")!=null && content != null)
		{
			res.setHeader("Cache-Control", "max-age=1") ; // no-cache
			res.setHeader("Expires", "Thu, 01 Dec 2003 00:00:00 GMT") ;
			res.setHeader("Pragma", "no-cache") ;
//			 EX 5.0 ? :("Proxy-Connection: Keep-Alive\r\n");
		}
		
		if(content!=null)
		{
			BufferedOutputStream bos = new BufferedOutputStream(res.getOutputStream()) ;
			try
			{
				bos.write(content);
				bos.flush();
			}
			catch(IOException ioe){}
			bos.close();
		}

		res.flushBuffer() ;

		// Close
        req.getInputStream().close();
        res.getOutputStream().close();
	}
}