/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.javarpc.server.imp.nouse ;

import java.io.*;
import java.util.*;
import java.util.zip.CRC32;

import org.azuki.common.util.FileTool;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class FileCache
{
	static private Map<String, Object[]> pool = new Hashtable<String, Object[]>();
							// Object[] : (Long/byte[]/String)

	final static public byte[] getContent(String filename) // Absolute
	throws IOException
	{
		Object[] poolfile = getFileTime(filename) ;

		if(poolfile == null) return null ;

		return (byte[])poolfile[1] ;
	}

	final static public Object[] getFileTime(String fname)  // Absolute
	throws IOException
	{
		File f = new File(fname);
		long ftime = f.lastModified() ;

		boolean cache = (fname.endsWith(".jar") || fname.endsWith(".class")) ;

		if(cache)
		{
			Object[] res = pool.get(fname);

			if(res != null) // fichier en cache
			{
				long lastime = ((Long)res[0]).longValue() ;

				if(lastime == ftime)
				{
					//Log.debug("Cache hit : " + filename) ;
					return res ;
				}
			}
		}
		
		byte[] content = FileTool.readFile(f) ;
		
		String etag = null ;
		// Calcul de l'ETag pour les jars seulement
		if(fname.endsWith(".jar"))  etag = calculETag(fname, content) ;
		
		Object[] res = new Object[]{ new Long(ftime), content, etag} ;

		if(cache)
		{
			//Log.debug("Cache miss : " + filename) ;
			pool.put(fname, res);
		}

		return res;
	}


	// Cache ETag pour performance
	final static private Map<String, String> ceta = new Hashtable<String, String>() ;
	final static public String getETag(String fname)
	{
		return ceta.get(fname) ;
	}
	
	// Calcul de l'ETag (CRC32) plus mise en cache
	final static private String calculETag(String fname, byte[] content)
	{
		CRC32 chks = new CRC32();
		chks.update(content);
		String etag = String.valueOf(chks.getValue());		
		ceta.put(fname, etag) ;
		return etag ;
	}
}