/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.jms.server.console;

import java.io.*;
import java.text.DateFormat;
import java.util.*;

import javax.jms.*;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.*;

import org.azuki.bean.jms.JMSReaderBean;

public class JMSReaderConsole
{
	final static public void service(JMSReaderBean reader,HttpServlet srv, HttpServletRequest req, HttpServletResponse hsr)
	throws Exception
	{
		ServletOutputStream sos = hsr.getOutputStream() ;
		
		// read queue Messages without remove it
		Vector<Message> v = null;
		try
		{
			 v = reader.dumpQueue();
		}
		catch(Exception e)
		{
			dumpException(sos,e);
			return;
		}
		
		// print queue elements
		sos.println("<h2> Number of messages : "+v.size()+"</h2><br/>");
	    int j = 1;
	    if (v.size()>0)
	    {
	    	for (Message m : v)
	    	{
    			sos.println("<h2> "+j+":Message header<br/></h2>");
	    		DateFormat df = DateFormat.getDateTimeInstance();
	    		sos.println("<dd>ID="+m.getJMSMessageID()+"</dd>");
	    		sos.println("<dd>Date="+df.format(new Date(m.getJMSTimestamp()))+"</dd>");
	    		if (m.getJMSExpiration()!=0)
	    			sos.println("<dd>Expiration="+df.format(new Date(m.getJMSExpiration()))+"</dd>");
	    		else
	    			sos.println("<dd>Expiration=no delay</dd>");
	    		sos.println("<dd>Priority="+m.getJMSPriority()+"</dd>");
	    		sos.println("<dd>Delivery="+m.getJMSDeliveryMode()+"</dd>");
	    		sos.println("<dd>Type="+m.getJMSType()+"</dd><br/>");
	    		j++;
	    	}
	    }
	}
	
	
	final static private void dumpException(ServletOutputStream sos, Exception e)
	{
		try
		{
			sos.print("<i>") ;		
			sos.print(e.getMessage()+"<br/>") ;
			sos.println("</i>") ;
		}
		catch(IOException io)
		{
			io.printStackTrace();
		}
	}
}
