/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.jms.server.console;

import java.io.IOException;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.*;

import org.azuki.bean.jms.JMSTestToolBean;

public class JMSTestToolConsole
{
	static public void service(JMSTestToolBean jmstt, HttpServlet servlet, HttpServletRequest req, HttpServletResponse hsr)
	throws Exception
	{
		
		ServletOutputStream sos = hsr.getOutputStream() ;
		//sos.println("<html>") ;
		//sos.println("<body>") ;

		try
		{
			sos.print("test JNDI ConnectionFactory to JMS broker...");
			jmstt.testConnectionFactory();
			sos.println("[Success]<br/>");
		}
		catch(Exception e)
		{
			e.printStackTrace();
			dumpException(sos,e);
		}
		
		try
		{
			sos.print("test JMS Connection...");
			jmstt.testConnection();
			sos.println("[Success]<br/>");
		}
		catch(Exception e)
		{
			e.printStackTrace();
			dumpException(sos,e);
		}
		
		try
		{
			sos.print("test JMS Session...");
			jmstt.testSession();
			sos.println("[Success]<br/>");
		}
		catch(Exception e)
		{
			e.printStackTrace();
			dumpException(sos,e);
		}
		
		try
		{
			sos.print("test JMS Queue Messaging...");
			int i = jmstt.testQueueConnection();
			switch(i)
			{
				case(0):
					sos.println("[Success]<br/>");
					break;
				case(2):
					sos.println("[Failed] cause : Send Message failed<br/>");
					break;
				case(4):
					sos.println("[Failed] cause : Read Message failed<br/>");
					break;
				case(16):
					sos.println("[Failed] cause : error : see azuki log file<br/>");
					break;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			sos.println("[Failed]<br/>");
			dumpException(sos,e);
		}
		
		try
		{
			sos.println("test JMS Topic Messaging...");
			int i = jmstt.testTopicConnection();
			switch(i)
			{
				case(0):
					sos.println("[Success]<br/>");
					break;
				case(2):
					sos.println("[Failed] cause : Send Message failed<br/>");
					break;
				case(4):
					sos.println("[Failed] cause : Read Message failed<br/>");
					break;
				case(16):
					sos.println("[Failed] cause : error : see azuki log file<br/>");
					break;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			dumpException(sos,e);
		}
		//sos.println("</body>");
		//sos.println("</html>");

	}
	
	// Tools class
	final static private void dumpException(ServletOutputStream sos, Exception e)
	{
		try
		{
			sos.println("[Failed]");
			sos.print(" ");
			sos.print("<i>") ;		
			sos.print(e.getMessage()) ;
			sos.println("</i> ") ;
		}
		catch(IOException io)
		{
			io.printStackTrace();
		}
	}	
}
