/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.jmx;

import java.util.*;

import javax.management.*;
import javax.servlet.http.*;

import org.azuki.bean.jmx.server.JMXExportAdvice;
import org.azuki.bean.jmx.server.imp.console.JMXExportConsole;
import org.azuki.bean.jmx.shared.ConstJMX;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.Locator;
import org.jdom.Element;

public class JMXExportBean implements IAspect<JMXExportAdvice>
{
	static private boolean stop = false ;
	
	private String domn ;
	
	@Export
	final public String getDomain() { return domn ; }
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		stop = false ;
		
//    	Element jmxs = JDOMUtil.getElement(para, ConstJMX.SERVICE) ;    	
//    	agid = jmxs.getAttributeValue(ConstJMX.SERVICE_AGENT) ;
//    	if(agid!=null && agid.trim().isEmpty()) agid = null ;
		
		Element jmxe = JDOMUtil.getElement(para, ConstJMX.EXPORT) ;
		domn = jmxe.getAttributeValue(ConstJMX.EXPORT_DOMAIN, ConstJMX.defaultDomain) ;	
		Log.info("\tDomain sets to ("+domn+")") ;
		
		if(mbs==null) mbs = java.lang.management.ManagementFactory.getPlatformMBeanServer() ;
		
////    	if(agid==null)
////    	{
////    		mbs = java.lang.management.ManagementFactory.getPlatformMBeanServer() ;
////    	}
////    	else
////    	{
//    		List<MBeanServer> servers = MBeanServerFactory.findMBeanServer(agid) ;
//    		if(servers.isEmpty())
//    		{
////    			mbs = MBeanServerFactory.createMBeanServer(domn) ;
////    			// javax.management.AttributeNotFoundException: MBeanServerId not accessible
////    			//mbs.setAttribute(MBeanServerDelegate.DELEGATE_NAME, new Attribute(mbeanServerIdKey, agid)) ; //"MBeanServerId"
////    			
//////    			Log.info("\tCreate a new MBeanServer with ID ("+agid+")") ;
////    			Log.info("\tCreate a new MBeanServer") ;
//    			
//    			mbs = java.lang.management.ManagementFactory.getPlatformMBeanServer() ;
//    			Log.info("\tUse platform MBeanServer") ;
//    		}
//    		else
//    		{
//    			Log.info("Nb. MBeanServer found with ID ("+agid+"): "+servers.size()) ;
//    			
//    			for(MBeanServer server : servers)
//    				Log.info("    found: "+server.getAttribute(MBeanServerDelegate.DELEGATE_NAME, mbeanServerIdKey)) ;
//    			
//    			mbs = servers.get(0) ;
//    		}	
////    	}

		Log.info("\tUsing MBeanServer with ID ("+mbs.getAttribute(MBeanServerDelegate.DELEGATE_NAME, mbeanServerIdKey)+")") ;	
	}	
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public JMXExportAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		JMXExportAdvice advc = new JMXExportAdvice(ipc, domn) ;
		
		// register MBean
		registerMBean(advc, advc.getObjectName()) ;
		
		return advc ;
	}
	

	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		JMXExportConsole.service(this, srv, req, res) ;
	}
	
	final public void stop()
	{
		if(stop) return ;
		
		stop = true ;
		
		System.out.println("Unregistering Azuki MBeans...") ;
		
		try
		{
			String name = Bean.getMetadata(this).getName() ;
			
			Collection<IPointcut> pcuts = Bean.getPointcuts(name) ;
			
			for(IPointcut pcut : pcuts)
			{
				JMXExportAdvice advc = (JMXExportAdvice)pcut.getAdvice() ;
				
				try
				{ mbs.unregisterMBean(advc.getObjectName()) ; }
				catch(Exception ex)
				{ Log.severe(Log.getStack(ex)) ; }
			}
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
		}
	}
	
	final public Object invokeImpl(String name, String labl, String meth, Object... args)
	throws Exception
	{
		return ((IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, name, labl, meth)).call(args) ; 
	}
	
///////////////////// IMPORTED FROM JMXService	
	final static private String mbeanServerIdKey = "MBeanServerId" ;
	
	private MBeanServer mbs ;
	
	private String agid = null ;
	
	@Export
    final public String getAgentID()
    {
		if(agid!=null) return agid ;
		
		try
		{ agid = (String)mbs.getAttribute(MBeanServerDelegate.DELEGATE_NAME, mbeanServerIdKey) ; }
		catch(Exception ex)
		{}
		
    	return agid ;
    }
	
	@Export
	final public void registerMBean(Object mbean, ObjectName name)
	throws MBeanRegistrationException, NotCompliantMBeanException, InstanceAlreadyExistsException
	{
		mbs.registerMBean(mbean, name) ;
	}
	
	@Export
	final public void addNotificationListener(ObjectName name, NotificationListener listener, NotificationFilter filter, Object handback) 
	throws InstanceNotFoundException
	{
		mbs.addNotificationListener(name,listener,filter,handback) ;
	}
	
	@Export
	final public void removeNotificationListener(ObjectName name, NotificationListener listener) 
	throws InstanceNotFoundException, ListenerNotFoundException
	{
		mbs.removeNotificationListener(name, listener) ;
	}
}
