/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.jmx;

import java.lang.reflect.Method;

import javax.management.*;

import org.azuki.bean.jmx.server.JMXListenerAdvice;
import org.azuki.bean.jmx.shared.ConstJMX;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.Element;

public class JMXListenerBean implements IAspect<JMXListenerAdvice>
{
	@DependencyInjectionInfo(desc="domain")
	public IMethodProxy<String> domain ;
	
	@DependencyInjectionInfo(desc="addListener", param=
					{ObjectName.class,NotificationListener.class,NotificationFilter.class,Object.class})
	public IMethodProxy<Void> addListener ;
		
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public JMXListenerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		// determine method
		Method[] methods = ipc.getSelectedMethods() ;
		if(methods.length>1)
			throw new Exception("This aspect can be applied to one method only") ;
		
		// retrieve JMX domain name
		String domn = domain.call() ;
		if(domn==null)
			throw new IllegalArgumentException("The management domain cannot be null") ;
		
		// parsing pointcuts parameters
		Element jmxl = JDOMUtil.getElement(ipc.getParameter(), ConstJMX.LISTEN) ;
		
		String name = jmxl.getAttributeValue(ConstJMX.LISTEN_NAME) ;
		if(name==null || name.trim().isEmpty())
			throw new IllegalArgumentException("Invalid bean name ("+name+")") ;
		
		String labl = jmxl.getAttributeValue(ConstJMX.LISTEN_LABEL, "").trim() ;
		
		Integer bid = Bean.searchFirst(name, labl).getID(); 

		JMXListenerAdvice advc = new JMXListenerAdvice(domn, ipc.getTargetMetadata().getID(), methods[0], name, labl, bid) ;
		
		// register interest on JMX Notification for the targetted MBean
		addListener.call(advc.getTargetObjectName(), advc, null, null);

		return advc ;
	}
}
