/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.jndi ;

import java.util.Collection;

import javax.naming.*;

import org.azuki.bean.jndi.server.JNDIContextAdvice;
import org.azuki.bean.jndi.server.imp.PInitialContext;
import org.azuki.common.util.Util;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */
final public class JNDIContextBean implements IAspect<JNDIContextAdvice>
{
	public enum InitStatus{ NOT_INITED, INIT_FAILED, INIT_OK }
	
	static private InitStatus init = InitStatus.NOT_INITED ;
	static private boolean stop = false ;
	
	private IMetadata meta = null ; // cached
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	// IAspect
	final public JNDIContextAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		getMetadata().getContract().checkBindable(ipc.getTargetMetadata().getBeanClass()) ;
		
		stop = false ;
		
		if(init==InitStatus.NOT_INITED)
		{	boolean isok = PInitialContext.init() ;
			init = isok ? InitStatus.INIT_OK : InitStatus.INIT_FAILED ; }
		
		IMetadata tmeta = ipc.getTargetMetadata() ;
		String tbid = Util.oneString(tmeta.getName(), tmeta.getLabel()) ;
		
		if(init==InitStatus.INIT_FAILED)
		{	Log.severe("JNDI context initialization failed, cannot bind resource ("+tbid+")") ;
			return null ; }
		
		JNDIResourceBean bean = (JNDIResourceBean)tmeta.getBean() ;
		
		// ensure namespace exists
		String nspc = bean.getNamespace() ;
		
		try
		{ PInitialContext.ensureContextExists(nspc) ; }
		catch(Exception ex)
		{ Log.warning("Cannot create resource ("+tbid+") namespace context ("+nspc+")") ; }
		//TODO: que faire: exception ou on laisse au cas ou ???
		
		// create reference and bind it into context
	    Reference ref = PInitialContext.createReference(bean.getClassName(),
	    												bean.getFactoryClassName(),
	    												bean.getFactoryLocation(),
	    												bean.getAddresses()) ;
	    
	    Context cx = new InitialContext() ;
	    
	    cx = (Context)cx.lookup(nspc) ;
	    cx.bind(tmeta.getLabel(), ref) ;
		
		return new JNDIContextAdvice() ;
	}
	
	final public void stop()
	{
		if(stop) return ;
		
		stop = true ;
		
		System.out.println("Unbinding Azuki JNDI resources...") ;
		
		try
		{
			Context ctxt = new InitialContext() ;
			
			String name = Bean.getMetadata(this).getName() ;
			
			Collection<IPointcut> pcuts = Bean.getPointcuts(name) ;
			if(pcuts.isEmpty()) return ;
			
			for(IPointcut pcut : pcuts)
			{
				JNDIResourceBean bean = (JNDIResourceBean)pcut.getTargetMetadata().getBean() ;
				
				try
				{ ctxt.unbind(bean.getName()) ; }
				catch(Exception ex)
				{ Log.severe(Log.getStack(ex)) ; }
			}
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
		}
	}
}