/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.jndi.server.imp ;

import java.util.*;

import javax.naming.*;

import org.azuki.bean.jndi.shared.ConstJNDI;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.context.Log;

final public class PInitialContext
{
	final static public boolean init()
	{
		try
		{
			Context cx = new InitialContext() ;
			
			try
			{
				cx.lookup(ConstJNDI.JAVA_COMP_ENV_CONTEXT) ;
				return true ;
			}
			catch(Exception ex) {}
			
			try
			{
				cx = (Context)cx.lookup(ConstJNDI.JAVA) ;
			}
			catch(Exception e)
			{
				Log.severe(Log.getStack(e)) ;
				throw e ;
			}
			
			try
			{
				cx = (Context)cx.lookup(ConstJNDI.COMP) ;
			}
			catch(Exception e)
			{
				Log.info("JNDI create context : "+ConstJNDI.COMP) ;
				cx = cx.createSubcontext(ConstJNDI.COMP) ;
			}

			try
			{
				 cx = (Context)cx.lookup(ConstJNDI.ENV) ;
			}
			catch(Exception e)
			{
				Log.info("JNDI create context : "+ConstJNDI.ENV) ;
				cx = cx.createSubcontext(ConstJNDI.ENV) ;
			}
			
			return true ;
		}
		catch(Throwable t)
		{
			Log.severe(Log.getStack(t)) ;
		}
		
		return false ;
	}
	
	final static public void ensureContextExists(String ctxt)
	throws Exception
	{
		if(ctxt==null)
			throw new Exception("Context cannot be null") ;
		
		int idx = ctxt.indexOf(ConstJNDI.ENV) ;
		if(idx!=-1) ctxt = ctxt.substring(idx+ConstJNDI.ENV.length()) ;
		
		ctxt = StringUtil.unslashed(ctxt) ;
		
		Context cx = new InitialContext() ;
		cx = (Context)cx.lookup(ConstJNDI.JAVA_COMP_ENV_CONTEXT) ;
		
		StringTokenizer st = new StringTokenizer(ctxt, "/", false) ;
		
		while(st.hasMoreTokens())
		{
			String tok = st.nextToken() ;
			if(tok.trim().isEmpty()) continue ;
			
			try
			{
				// context exists ?
				cx = (Context)cx.lookup(tok) ; 
			}
			catch(NamingException ne)
			{
				// so create a new one
				Log.info("JNDI create context ("+tok+") parent context ("+cx.getNameInNamespace()+")") ;
				cx = cx.createSubcontext(tok) ;
			}
		}
	}
	
//	@SuppressWarnings("unchecked")
//	final static public Reference createReference(Element lmen)
//	{
//		String clas = lmen.getAttributeValue(ConstJNDI.CLASSNAME) ;
//		String fact = lmen.getAttributeValue(ConstJNDI.FACTORY) ;		
//		String loca = lmen.getAttributeValue(ConstJNDI.FACTORY_LOCATION) ;
//		
//		Map<String,String> addrs = new HashMap<String,String>() ;
//		
//		List<Element> chlds = lmen.getChildren(ConstJNDI.ADDRESS) ; 
//		for(Element chld : chlds)
//		{
//			String type  = chld.getAttributeValue(ConstJNDI.TYPE) ;
//			String valu = chld.getAttributeValue(ConstJNDI.VALUE) ;
//			
//			addrs.put(type, valu) ;			
//		}
//		
//		return createReference(clas, fact, loca, addrs) ;
//	}
	
	final static public Reference createReference(String clas, String fact, String loca, Map<String,String> addrs)
	{
		Reference ref = new Reference(clas, fact, loca) ;
		
		for(Map.Entry<String,String> ety : addrs.entrySet())
		{
			StringRefAddr addr = new StringRefAddr(ety.getKey(), ety.getValue()) ;
			ref.add(addr) ;
		}
		
		return ref ;
	}
}