/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.jpa.server.imp;

import javax.persistence.EntityManager;

import org.azuki.bean.jpa.EntityManagerBean;
import org.azuki.bean.jpa.EntityManagerBean.DTAction;
import org.azuki.common.util.ProcessLifecycle;
import org.azuki.framework.context.*;
import org.azuki.framework.context.IContext.IStub;
import org.azuki.framework.injection.*;
import org.azuki.framework.object.ILifecycle;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

public class EntityManagerAdvice implements IController
{
	final static private ILifecycle ilf = new ProcessLifecycle() ; // PNU stands for all process

	final private EntityManagerBean bean ;

	final private DTAction before  ;
	final private DTAction after   ;
	final private DTAction onerror ;	
	
	public EntityManagerAdvice(EntityManagerBean bean, DTAction before, DTAction after, DTAction onerror)
	{
		this.bean = bean ;
		this.before  = before ;
		this.after   = after ;
		this.onerror = onerror ;		
	}
		
	final public Object intercept(InvocationTarget it)
	throws Exception
	{
		IContext ictx = ThreadContext.get() ;

		// 
		doAction(before, ictx) ;
		
		
		// savedstub is the previous value (saved)
		IStub savedstub = ictx.bind(IContext.OBJECT_PERSISTENCEUNITNAME, bean.getProcessUnitName(), ilf) ;

		
		try
		{
			Object result = it.proceed() ;

			doAction(after, ictx) ;
			
			return result ;
		}
		catch(Exception e)
		{
			doAction(onerror, ictx) ;
			throw e ; 
		}		
		finally
		{
			ictx.unbind(IContext.OBJECT_PERSISTENCEUNITNAME, savedstub) ;		   	
		}
	}
	
	final static private void doAction(DTAction action , IContext ictx)
	throws Exception
	{
		if(action==DTAction.NOTHING) return ;

		EntityManager em = (EntityManager)ictx.getObject(IContext.OBJECT_ENTITYMANAGER) ;
		if(em==null) return ;
		
		if(action==DTAction.COMMIT)
		{
			em.getTransaction().commit() ;
			return ;
		}

		// else,  Rollback
		em.getTransaction().rollback() ;
	}	
}
