/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.logger ;

import java.io.File;
import java.util.*;
import java.util.logging.*;

import org.azuki.bean.logger.server.imp.*;
import org.azuki.bean.logger.shared.*;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.context.Log;
import org.jdom.Element;

final public class FileHandlerBean extends AbstractHandlerBean<FileHandler>
{
	private String path;
	
	private String dire ; 
	public String getPath() { return dire ; }
	
	private String fnam ;
	public String getFileName() { return fnam; }
	
	private boolean dail ; // Un fichier par jour
	public boolean isDaily(){ return dail; }
	
	private boolean rota ; // Rotatif
	public boolean isRotative(){ return rota; }
	
	private int     maxb ; // Taille maxi en bytes
	public int getSizeMax(){ return maxb; }
	
	private String levl ; //FIXME: common to all log handler
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{    	
		try
    	{
			Element logLmen = JDOMUtil.getElement(para, ConstHandler.LOG) ;
			
	        dire = logLmen.getAttributeValue(ConstFileHandler.LOG_DIR) ;
	        if(dire==null) throw new Exception("Path parameter does not exist") ;
	        if(!dire.endsWith("/")) dire = dire+"/" ; // due to ShellOutput pb
	        
	        fnam = logLmen.getAttributeValue(ConstFileHandler.LOG_FILE) ;
	        if(fnam==null) throw new Exception("File name parameter does not exist") ;
	        
	        levl = logLmen.getAttributeValue(ConstHandler.LOG_LEVEL) ;
	        if(levl==null)
	        {
	        	Log.warning("Invalid level("+levl+"), use default value ("+ConstFileHandler.defaultLevel+")") ;
	        	levl = ConstFileHandler.defaultLevel ;
	        }
	        
	        String dailStr = logLmen.getAttributeValue(ConstFileHandler.LOG_DAIL) ;
	        if(dailStr==null) throw new Exception("Daily parameter does not exist") ;
	        dail = Util.isOK(dailStr) ;
	        
	        String rotaStr = logLmen.getAttributeValue(ConstFileHandler.LOG_ROTA) ;
	        if(rotaStr==null) throw new Exception("Rotate parameter does not exist") ;
	        rota = Util.isOK(rotaStr) ;
	        
	        String maxbStr = logLmen.getAttributeValue(ConstFileHandler.LOG_MAXB) ; 
	        try
	        {
	        	maxb = Integer.parseInt(maxbStr)*1024 ;
	        }
	        catch(NumberFormatException nfe)
	        {
		        System.out.println("maxb="+maxbStr+" : not a number !!! use default value (250)") ;
	        	maxb = ConstFileHandler.defaultMaxKBytes.intValue()*1024 ;
	        }
    	}
	    catch(Exception e)
    	{
    		e.printStackTrace();
    		throw e;
    	} // end parameters
	    
	    // create file if not exist
    	path = UtilFileHandler.mkFile(dire, fnam, dail) ;
    	File file = new File(path) ;
    	if(!file.exists())
    	{
    		File parent = file.getParentFile() ;
    		if(!parent.exists()) parent.mkdirs() ;
    	}
    	
    	// purge all lck files if exists. pb with Log API.
    	File[] lck = file.getParentFile().listFiles();
    	if (lck!=null)
    	{
    		for (File f:lck)
    		{
				if (f.getName().startsWith(fnam) && f.getName().endsWith(".lck")) 
					f.delete();
    		}
    	}
    	
		Log.info("\t (*) Create FileHandler '"+path+"'") ;
		Log.info("\t     Level="+levl) ;
		
		FileHandler fl = rota ? new FileHandler(path, maxb, 1, true) : new FileHandler(path, true) ;
		fl.setFormatter(new FileLogFormatter()) ;
		fl.setLevel(Level.parse(levl)) ;
    	
    	setHandler(fl) ;
	}
	
	@Override
	final public String consoleInfo()
	{
		Map<String,Object> map = new HashMap<String,Object>() ;
		map.put("Level", getLevel()) ;
		map.put("Path", getPath()) ;
		map.put("File Name", getFileName()) ;
		map.put("Daily", Boolean.valueOf(isDaily())) ;
		map.put("Rotative", Boolean.valueOf(isRotative())) ;
		map.put("Max. Size", Integer.valueOf(getSizeMax())) ;
		
		return HtmlFactory.propertyList("Properties", map).toString() ;
	}

	@Override
	final public String shellInfo()
	{
		StringBuilder bdr = new StringBuilder() ;
		
		bdr.append("Level     : ").append(getLevel()) ;
		bdr.append('\n') ;
		bdr.append("Path      : ").append(getPath()) ;
		bdr.append('\n') ;
		bdr.append("File Name : ").append(getFileName()) ;
		bdr.append('\n');
		bdr.append("Daily     : ").append(isDaily()) ;
		bdr.append('\n') ;
		bdr.append("Rotative  : ").append(isRotative()) ;
		bdr.append('\n') ;
		bdr.append("Max Size  : ").append(getSizeMax()) ;
		bdr.append('\n') ;
		
		return bdr.toString() ;
	}
}