/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.logger ;

import java.io.StringWriter;
import java.util.logging.*;

import javax.servlet.http.*;

import org.azuki.bean.logger.server.*;
import org.azuki.bean.logger.server.console.*;
import org.azuki.bean.logger.server.imp.ILogger;
import org.azuki.bean.logger.server.shell.*;
import org.azuki.bean.logger.shared.*;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.jdom.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class LoggerBean implements IAspect<LoggerAdvice>, ILogger
{
	IMetadata meta = null ;
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	private Logger logr ;
	final public Logger getLogger() { return logr ; }
	
	@MethodInfo(desc="Returns the logger level")
	final public String getLevel()
	{ return logr.getLevel().getName() ; }
	
	@MethodInfo(desc="Sets the level of the logger")
	final public void setLevel(@ArgInfo(name="levl",desc="the new level name") String levl)
	{ logr.setLevel(Level.parse(levl)) ; }
	
	// ILogger
	final public void addHandler(Handler hdlr)
	{
		logr.addHandler(hdlr) ;
	}
	
	// Initializer
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		Element logLmen = JDOMUtil.getElement(para, ConstLogger.LOG) ;
		
		String slvl = logLmen.getAttributeValue(ConstLogger.LOG_LEVEL) ;
		if(slvl==null || slvl.isEmpty()) 
		{
			Log.warning("Level parameter does not exist, use INFO as default value") ;
			return ;
		}
		
		IMetadata meta = getMetadata() ;
		
		String logrName = Util.oneString(meta.getName(), meta.getLabel()) ;
		
		logr = Logger.getLogger(logrName) ;
		logr.setUseParentHandlers(false) ;
		logr.setLevel(Level.parse(slvl)) ;
	}
	
	// IAspect
	@MethodInfo(desc="Called at the framework initialization to apply a pointcut. Must not be called by others means")
	final public LoggerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		return new LoggerAdvice(this) ;
	}
	
	@MethodInfo(desc="")
	final public void log(String topic, Object msg)
	{
		String labl = toString() ;
		try
		{
			IMetadata meta = getMetadata() ;
			labl = Util.oneString(meta.getName(), meta.getLabel()) ;
		}
		catch(Exception ex)	{}
		
		try
		{
			//TODO: p-e Log.info(...)
			logr.info("Logger ("+labl+") received an event on topic("+topic+")") ;
			
			if(msg instanceof Throwable)
			{
			    Throwable t = (Throwable)msg ;
			    logr.severe(Log.getStack(t)) ;
			}
			else if(msg instanceof Document)
			{
				StringWriter sw = new StringWriter() ;
				JDOMUtil.xmlout.output((Document)msg, sw) ;
				try{ sw.close() ; } catch(Exception ex){}
				
				logr.info(sw.toString()) ;
			}
			else if(msg instanceof Object[])
			{
				logr.info(String.valueOf(msg)) ;
			}
			else if(msg instanceof byte[])
			{
				byte[] bytes = (byte[])msg ;
				logr.info(new String(bytes));
			}
			else
			{
				logr.info(String.valueOf(msg)) ;
			}
		}
  		catch(Throwable t)
  		{
  			// WARNING: no logs !!!
  			t.printStackTrace() ;
  		}
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		LoggerConsole.service(this, srv, req, res) ;
	}
	
	@MethodInfo(desc="Build the list of all LoggerBean advice",
				retu="The advices listing")
	final public String listLoggers()
	throws Exception
	{
		return LoggerShell.listPointcuts(this) ;
	}
	
	@MethodInfo(desc="Enable/disable LoggerBean advices")
	final public String setEnabled(@ArgInfo(name="bean", desc="the bean's name:label") String bean,
								   @ArgInfo(name="enab", desc="the new status") boolean enab)
	throws Exception
	{
		return LoggerShell.setEnabled(this, bean, enab) ;
	}
}