/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.logger;

import java.util.*;
import java.util.logging.Level;

import org.azuki.bean.logger.server.imp.*;
import org.azuki.bean.logger.shared.*;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.context.Log;
import org.jdom.Element;

final public class OutputHandlerBean extends AbstractHandlerBean<OutputHandler>
{
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")	
	final public void postCreate(Element para)
	throws Exception
	{
		Element logLmen = JDOMUtil.getElement(para, ConstHandler.LOG) ;
				
		String level = logLmen.getAttributeValue(ConstHandler.LOG_LEVEL) ;
		if(level==null || level.isEmpty()) 
		{
			Log.warning("Invalid level("+level+"), use default value ("+ConstOutputHandler.defaultLevel+")") ;
			level = ConstOutputHandler.defaultLevel ;
		}
		
		OutputHandler outHandler = new OutputHandler(System.out) ;
		outHandler.setLevel(Level.parse(level)) ;
		
		Log.info("\t (*) Create OutputHandler");
		Log.info("\t     Level="+level);
		
		setHandler(outHandler) ;
	}
	
	@Override
	final public String consoleInfo()
	{
		Map<String,Object> map = new HashMap<String,Object>() ;
		map.put("Level", getHandler().getLevel().getName()) ;
		
		return HtmlFactory.propertyList("Properties", map).toString() ;
	}

	@Override
	final public String shellInfo()
	{
		StringBuilder bdr = new StringBuilder() ;
		
		bdr.append("Level : ").append(getHandler().getLevel().getName()) ;
		bdr.append('\n') ;
		
		return bdr.toString() ;
	}
}
