/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.logger.editor;

import java.awt.BorderLayout;
import java.util.logging.Level;

import javax.swing.BorderFactory;

import org.azuki.bean.logger.shared.*;
import org.azuki.common.editor.basic.AbstractEditor;
import org.azuki.common.editor.jdom.*;
import org.azuki.common.util.JDOMUtil;
import org.azuki.common.widget.JFormPanel;
import org.jdom.*;

@SuppressWarnings("serial")
public class DBHandlerEditor extends AbstractEditor
{
	private JDOMAttributeField dsrcField ;
	private JDOMComboField levlCombo ;
	private JDOMNumberField reteField ;
	
	private JFormPanel form ; 
	
	/*
	 * AbstractEditor
	 */
	@Override
	final public void initParameters(Element para)
	{
		Element logLmen = JDOMUtil.getElement(para, ConstHandler.LOG) ;
		Attribute dsrcAttr = JDOMUtil.getAttribute(logLmen, ConstDBHandler.LOG_DATASOURCE, ConstDBHandler.defaultDataSource) ;
		Attribute levlAttr = JDOMUtil.getAttribute(logLmen, ConstHandler.LOG_LEVEL, ConstDBHandler.defaultLevel) ;
		Attribute daysAttr = JDOMUtil.getAttribute(logLmen, ConstDBHandler.LOG_RETENTION, ConstDBHandler.defaultRetention.toString()) ;
		
		dsrcField = new JDOMAttributeField(dsrcAttr) ;
		
		levlCombo = new JDOMComboField(Level.OFF, Level.FINEST, Level.FINER, Level.FINE, Level.CONFIG, Level.INFO, Level.WARNING, Level.SEVERE, Level.ALL) ;
		levlCombo.setAttribute(levlAttr) ;
		
		reteField = new JDOMNumberField(1, Integer.MAX_VALUE, 1) ;
		reteField.setAttribute(daysAttr) ;
		
		form = new JFormPanel() ;
		form.addField(dsrcField, "JNDI Datasource") ;
		form.addField(levlCombo, "Level") ;
		form.addField(reteField, "Retention") ;

		setDialogTitle("Database logs configuration") ;
		
		setLayout(new BorderLayout()) ;
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		add(form, BorderLayout.NORTH) ;
	}
}
