/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Yoann Perrin (yoann.perrin@gmail.com)
 */

package org.azuki.bean.logger.editor;

import java.awt.BorderLayout;
import java.util.logging.Level;

import javax.swing.BorderFactory;

import org.azuki.bean.logger.shared.*;
import org.azuki.common.consts.Const;
import org.azuki.common.editor.basic.AbstractEditor;
import org.azuki.common.editor.jdom.*;
import org.azuki.common.util.JDOMUtil;
import org.azuki.common.widget.JFormPanel;
import org.jdom.*;

@SuppressWarnings("serial")
public class FileHandlerEditor extends AbstractEditor
{
	private JDOMAttributeField direField ;
	private JDOMAttributeField fileField ;
	private JDOMAttributeField maxbField ;
	
	private JDOMComboField levlCombo ;
	
	private JDOMCheckBox dailCheck ;
	private JDOMCheckBox rotaCheck ;
	
	private JFormPanel form ;
	
	/*
	 * AbstractEditor
	 */
	@Override
	final public void initParameters(Element para)
	{
		Element logLmen = JDOMUtil.getElement(para, ConstHandler.LOG) ;

		Attribute direAttr = JDOMUtil.getAttribute(logLmen, ConstFileHandler.LOG_DIR, ConstFileHandler.defaultDirectory) ;
		Attribute fileAttr = JDOMUtil.getAttribute(logLmen, ConstFileHandler.LOG_FILE, ConstFileHandler.defaultFile) ;
		Attribute dailAttr = JDOMUtil.getAttribute(logLmen, ConstFileHandler.LOG_DAIL, ConstFileHandler.defaultDaily) ;
		Attribute rotaAttr = JDOMUtil.getAttribute(logLmen, ConstFileHandler.LOG_ROTA, ConstFileHandler.defaultRotate) ;
		Attribute levlAttr = JDOMUtil.getAttribute(logLmen, ConstHandler.LOG_LEVEL, ConstFileHandler.defaultLevel) ;
		Attribute maxbAttr = JDOMUtil.getAttribute(logLmen, ConstFileHandler.LOG_MAXB, ConstFileHandler.defaultMaxKBytes.toString()) ; // in Kb
		
		direField = new JDOMAttributeField(direAttr) ;
		fileField = new JDOMAttributeField(fileAttr) ;
		maxbField = new JDOMAttributeField(maxbAttr) ;
		
		levlCombo = new JDOMComboField(Level.OFF, Level.FINEST, Level.FINER, Level.FINE, Level.CONFIG, Level.INFO, Level.WARNING, Level.SEVERE, Level.ALL) ;
		levlCombo.setAttribute(levlAttr) ;
		
		dailCheck = new JDOMCheckBox(Const.YES, Const.NO) ;
		dailCheck.setAttribute(dailAttr) ;
		
		rotaCheck = new JDOMCheckBox(Const.YES, Const.NO);
		rotaCheck.setAttribute(rotaAttr) ;
		
		form = new JFormPanel() ;
		form.addField(direField, "Directory") ;
		form.addField(fileField, "File") ;
		form.addField(levlCombo, "Level") ;
		form.addField(maxbField, "Max. size (Kb)") ;
		form.addField(dailCheck, "Daily") ;
		form.addField(rotaCheck, "Rotate") ;
		
		setDialogTitle("File logs configuration") ;
		
		setLayout(new BorderLayout()) ;
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		add(form, BorderLayout.NORTH) ;
	}
}
