/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.logger.server.console ;

import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.logger.LoggerBean;
import org.azuki.bean.logger.server.LoggerAdvice;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.*;
import org.azuki.common.console.http.HttpResponse;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class LoggerConsole
{
	final static public String[] tableHeaders = {"Bean", "Enabled"} ;
	
	final public static void service(LoggerBean aspt, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		IMetadata meta = aspt.getMetadata() ;
		
		// properties
		Map<String,Object> prps = new HashMap<String,Object>() ;
		prps.put("Level", aspt.getLogger().getLevel().getName()) ;
		
		Table plist = HtmlFactory.propertyList("Properties", prps) ;
		
		// data
		List<List> data = new ArrayList<List>() ;
		
		for(IPointcut pc : Bean.getPointcuts(meta.getID()))
		{
			LoggerAdvice advc = (LoggerAdvice)pc.getAdvice() ;
			if(advc==null) continue ;
			
			IMetadata tmeta = pc.getTargetMetadata() ;
			data.add(ListUtil.toList(Util.oneString(tmeta.getName(), tmeta.getLabel()), Boolean.valueOf(advc.isEnabled()))) ;
		}
		
		// writting response
		Table table = HtmlFactory.sortedTable(req, data, tableHeaders) ;
	
		HtmlBuilder bdr = new HtmlBuilder() ;
		
		bdr.print("<h2>").print(Util.oneString(meta.getName(), meta.getLabel())).print("</h2>") ;
		bdr.println(plist) ;
		bdr.println("<br/>") ;
		bdr.println(table) ;
		
		HttpResponse.print(res, bdr) ;
	}
}