/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.logger.server.imp;

import java.util.Collection;
import java.util.logging.*;

import javax.servlet.http.*;

import org.azuki.bean.logger.server.HandlerAdvice;
import org.azuki.bean.logger.server.console.HandlerConsole;
import org.azuki.bean.logger.server.shell.HandlerShell;
import org.azuki.common.util.Util;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;

/**
 * @author Mathieu Jucker (mathieu.jucker@azuki-framework.org)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

abstract public class AbstractHandlerBean<T extends Handler> implements IAspect
{
	private T handler ;
	public T getHandler()				{ return handler ; }
	public void setHandler(T handler)	{ this.handler = handler ; }
	
	@MethodInfo(desc="Returns the handler level")
	final public String getLevel()
	{ return handler.getLevel().getName() ; }
	
	@MethodInfo(desc="Sets the level of the handler")
	final public void setLevel(@ArgInfo(name="levl",desc="the new level name") String levl)
	{ handler.setLevel(Level.parse(levl)) ; }
	
	private IMetadata meta ;
	public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	// TO OVERRIDE
	public void purge() throws Exception {} // Schedule it appying Scheduler aspect
	
	// Abstract methods
	abstract public String shellInfo() ;
	abstract public String consoleInfo() ;
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public Object createAdvice(IPointcut ipc)
	throws Exception 
	{
		IMetadata meta = getMetadata() ;
		IMetadata tmeta = ipc.getTargetMetadata() ;
		
		Object targ = tmeta.getBean() ;
		if(!ILogger.class.isInstance(targ))
			throw new Exception("Can only be applied on a "+ILogger.class.getName()+" instance: "+targ) ;
		
		if(handler==null)
		{	String bkey = Util.oneString(meta.getName(), meta.getLabel()) ;
			throw new Exception("Handler ("+bkey+") has not been initialized: checks if its run-level is fewer than the logger one") ; }
		
		ILogger ilgr = (ILogger)targ ;
		ilgr.addHandler(handler) ;
		
		return new HandlerAdvice() ;
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		HandlerConsole.service(this, servlet, req, res) ;
	}
	
	@MethodInfo(desc="Build the list of all HandlerBean advice",
				retu="The advices listing")
	final public String listHandlers()
	throws Exception
	{
		return HandlerShell.listPointcuts(this) ;
	}
	
	final public String[] listBeanHandlers()
	throws Exception
	{
		Collection<IPointcut> pcs = Bean.getPointcuts(getMetadata().getID()) ;
		
		String[] ret = new String[pcs.size()] ;
		
		int i = 0 ;
		for(IPointcut pc : pcs)
		{
			IMetadata tmeta = pc.getTargetMetadata() ;
			ret[i++] = Util.oneString(tmeta.getName(), tmeta.getLabel()) ;
		}
		
		return ret ;
	}
}
