/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.logger.shared ;

import java.sql.*;
import java.util.logging.LogRecord;

import javax.naming.*;
import javax.sql.DataSource;

import org.azuki.bean.logger.server.imp.Times;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.context.Log;

final public class UtilDBHandler
{
	// SQL orders
	final static public String SQL_INSERT = "insert into alog(dcre,lvel,msg,src,meth,stack,thid) values(?,?,?,?,?,?,?)" ;
	final static public String SQL_DELETE = "delete from alog where dcre < ?" ;
	
	final static public Connection getConnection(String dsrc)
	throws NamingException, SQLException
	{
		Context ctx = new InitialContext() ;
		DataSource ds = (DataSource)ctx.lookup("java:/comp/env/"+dsrc) ;
		return ds.getConnection() ;
	}
	
	final static public void insertLog(String dsrc, LogRecord lrec)
	{
		Connection cx = null ;
		PreparedStatement ps = null ;
		
		try
		{
			cx = getConnection(dsrc) ;
			ps = cx.prepareStatement(SQL_INSERT) ;

			// dcre
			ps.setTimestamp(1, new Timestamp(lrec.getMillis())) ;
			
			// level
			ps.setString(2, lrec.getLevel().getName()) ;
			
			// msg
			String msg = lrec.getMessage() ;
			ps.setString(3, StringUtil.trunc(msg,ConstDBHandler.SZ_MAX_MSG)) ;
			
			// source
			String src = lrec.getSourceClassName() ;
			ps.setString(4, StringUtil.trunc(src,ConstDBHandler.SZ_MAX_CLASS)) ;
			
			// method
			String meth = lrec.getSourceMethodName() ;
			ps.setString(5, StringUtil.trunc(meth,ConstDBHandler.SZ_MAX_METH)) ;
			
			// exception
			Throwable t = lrec.getThrown() ;
			String stack= t!=null ? null : Log.getStack(t) ;
			ps.setString(6, stack==null ? null : StringUtil.trunc(stack, ConstDBHandler.SZ_MAX_STACK)) ;
			
			// Thread ID
			int thid = lrec.getThreadID() ;
			ps.setInt(7,thid) ;
			
			ps.executeUpdate() ;
			
			cx.commit() ;
		}
		catch(Exception ex)
		{
			System.err.println(Log.getStack(ex)) ;
		}
		finally
		{
			if(ps!=null) try{ ps.close() ; } catch(Exception ex){}
			if(cx!=null) try{ cx.rollback() ; cx.close() ; } catch(Exception ex){}
		}
	}
	
	final static public void deleteLogs(String dsrc, int days)
	throws NamingException, SQLException
	{
		System.out.println("--> START PURGE DB LOGS") ;
  		
		long dmin = System.currentTimeMillis()-(Times.DAY*days) ;
		
		Connection cx = null ;
		PreparedStatement ps = null ;
		
  		try
  		{
  			cx = getConnection(dsrc) ;
  			ps = cx.prepareStatement(SQL_DELETE) ;
			
  			System.out.println(SQL_DELETE) ;
			ps.setTimestamp(1, new Timestamp(dmin)) ;
			
			int rows = ps.executeUpdate() ;
			System.out.println(rows+" records deleted.") ;
			
			cx.commit() ;
  		}
  		catch(NamingException ne)
  		{
  			System.err.println(Log.getStack(ne)) ;
  			throw ne ;
  		}
  		catch(SQLException sqle)
  		{
  			System.err.println(Log.getStack(sqle)) ;
  			throw sqle ;
  		}
  		finally
  		{
  			//FIXME: rollback ??
  			if(ps!=null) try{ ps.close() ; } catch(Exception ex){}
  			if(cx!=null) try{ cx.rollback() ; cx.close() ; } catch(Exception ex){}
  			
  			System.out.println("<-- END PURGE DB LOGS") ;
  		}
	}
}