/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.mail;

import java.util.Map;

import javax.activation.DataHandler;
import javax.mail.*;
import javax.mail.internet.*;
import javax.naming.*;

import org.azuki.bean.mail.server.imp.*;
import org.azuki.bean.mail.shared.ConstMail;
import org.azuki.common.editor.EditorUtil;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

public class MailSMTPBean
{
	private String jndi ;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		Map<String,String> map = EditorUtil.formToMap(para) ;
		
		if(map==null || map.isEmpty())
			throw new Exception("ERROR : "+ getClass().getName() +" parameter are empty !") ; 
		
		jndi = map.get(ConstMail.KEY_JNDI_MAIL_RESOURCE) ;
		
		if(StringUtil.isEmpty(jndi))
			throw new Exception("ERROR : "+getClass().getName()+" JNDI_MAIL_RESOURCE parameter is null") ;
	}
	
	@Export
	@MethodInfo(desc="Send a simple e-mail")
	final public void sendSimpleMail(@ArgInfo(name="from", desc="sender's e-mail address") String from,
									 @ArgInfo(name="dest", desc="recipient's e-mail address") String recp,
									 @ArgInfo(name="subj", desc="subject of the e-mail") String subj,
									 @ArgInfo(name="mess", desc="the message text to send") String mess)
	throws Exception
	{
		sendMail(from, new String[]{recp}, subj , mess, null) ;
	}
	
	@Export
	@MethodInfo(desc="Send a e-mail")
	public void sendMail(String from, String[] dests, String subject, String text, Map<String,byte[]> files)
	throws Exception
	{
		Context ctx = new InitialContext() ;
		Session ses = (Session)ctx.lookup("java:/comp/env/"+jndi) ;
		
		Attachment[] atts = null ;
		
		// create attachment files
		if(files!=null && !files.isEmpty())
		{
			int i = 0 ;
			
			atts = new Attachment[files.size()] ;
			
			for(Map.Entry<String,byte[]> ety : files.entrySet())
				atts[i++] = new Attachment(ety.getKey(), ety.getValue()) ;
		}
		
		sendImpl(ses,from,dests,subject,text,atts);
	}
	
	final private void sendImpl(Session sess, String from, String[] dests, String subject, String text, Attachment[] files)
	throws Exception
	{
		Log.info("sendMail");
		
		sess.setDebug(true);
		//sess.setDebug(false);

		// create a message
		MimeMessage msg = new MimeMessage(sess);
		msg.setHeader("X-Mailer", "AZUKI");
		msg.setFrom(new InternetAddress(from));
		msg.setSentDate(new java.util.Date());

		for(int i=0; i<dests.length; i++)
			msg.addRecipients(Message.RecipientType.TO, dests[i]);

		msg.setSubject(subject);
		msg.setSentDate(new java.util.Date());

		///////////////////////////////
		// Create a multi-part body  //
		///////////////////////////////
		Multipart multipart = new MimeMultipart();

		// Create body part for text
		////////////////////////////
		BodyPart part1 = new MimeBodyPart();

		// Assumption that the string contains only ASCII
		// characters!  Otherwise just pass a charset into this
		// constructor and use it in getBytes()
		byte[] data = text.getBytes("ISO-8859-1");

		//Set text in this part
		if((text.indexOf("<html>") != -1) || (text.indexOf("<HTML>") != -1))
		   part1.setDataHandler(new DataHandler(new ByteDS(data, "text/html")));
		else
		   part1.setDataHandler(new DataHandler(new ByteDS(data, "text/plain")));

		multipart.addBodyPart(part1);

		// Create file parts
		////////////////////
		if(files!=null)
		{
			for(int i=0; i<files.length; i++)
			{
				BodyPart part = new MimeBodyPart();
				part.setDataHandler(new DataHandler(new ByteDS(files[i].getContent(), "application/octet-stream")));
				part.setFileName(files[i].getName()) ;
				multipart.addBodyPart(part) ;
			}
		}

		//put the multipart in the message
		msg.setContent(multipart);

		//send message
		Transport.send(msg);
	}
}