/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.mail.server.imp;

import java.io.*;

import javax.activation.FileTypeMap;

import org.azuki.common.util.*;

public class Attachment
{
	final static private FileTypeMap mimes = FileTypeMap.getDefaultFileTypeMap() ;
	
	final private String name ;
	final private String type ;
	final private byte[] byts ;
	
	public Attachment(File file)
	throws IOException
	{
		this(file.getName(), FileTool.readFile(file)) ;
	}
	
	public Attachment(String name, byte[] byts)
	{
		this(name, byts, mimes.getContentType(name)) ; 
	}
	
	public Attachment(String name, byte[] byts, String type)
	{
		this.name = name ;
		this.byts = byts ; 
		this.type = type ;
	}
	
	// retourne le nom 
	final public String getName()
	{ return name ; }
	
	// retourne le content-type
	final public String getContentType()
	{ return type ; }
	
	// retourne le contenu
	final public byte[] getContent()
	{ return byts ; }
	
	
	/*
	 * STATIC
	 */
	final static public String getMimeType(File file)
	{ return mimes.getContentType(file) ; }
	
	final static public String getMimeType(String file)
	{ return mimes.getContentType(file) ; }
}
