/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.ooo ;

import java.io.*;

import org.azuki.bean.ooo.shared.ConstOOO;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

import com.artofsolving.jodconverter.*;
import com.artofsolving.jodconverter.openoffice.connection.SocketOpenOfficeConnection;
import com.artofsolving.jodconverter.openoffice.converter.StreamOpenOfficeDocumentConverter;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */
public class OOConverterBean
{
	final static private DefaultDocumentFormatRegistry ddfr = new DefaultDocumentFormatRegistry() ;
	
	private String host = null ;
	final public String getHost() 	{ return host ; }
	
	private int port = -1 ;
	final public int getPort()		{ return port ; }
		
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element lmen)
	throws Exception
	{
		Element conv = JDOMUtil.getElement(lmen, ConstOOO.CONV) ;
		
		// host name
		host = conv.getAttributeValue(ConstOOO.CONV_HOST) ;
		if(StringUtil.isEmpty(host)) throw new Exception("Host name must be specified") ;
		
		// port
		try
		{ port = Integer.parseInt(conv.getAttributeValue(ConstOOO.CONV_PORT)) ; }
		catch(Exception ex)
		{ throw new Exception("Invalid port value", ex) ; }
		finally
		{ if(port<0) throw new Exception("Invalid port value: "+port) ; }
	}
	
	@Export
	@MethodInfo(desc="Convert the input bytes from/to specified file extensions",
				retu="The converted document bytes")
	final public byte[] convert(@ArgInfo(name="inpe", desc="Input document file extension")String inpe,
								@ArgInfo(name="inpb", desc="Input document bytes")byte[] inpb,
								@ArgInfo(name="oute", desc="Output document file extension")String oute)
	{
		Log.info("Convert document from("+inpe+") to("+oute+"), connect to ("+host+":"+port+")...") ;
		
		//TODO: test if DocumentFormat are NULL ?
		
		// input
		ByteArrayInputStream inps = new ByteArrayInputStream(inpb) ;
		DocumentFormat inpf = ddfr.getFormatByFileExtension(inpe) ;
		
		// output
		ByteArrayOutputStream outs = new ByteArrayOutputStream() ;
		DocumentFormat outf = ddfr.getFormatByFileExtension(oute) ;
		
		// connecting to OpenOffice server
		SocketOpenOfficeConnection sooc = new SocketOpenOfficeConnection(host, port) ;
		
		// convert input document to PDF
		DocumentConverter dcnv = new StreamOpenOfficeDocumentConverter(sooc) ;
		dcnv.convert(inps, inpf, outs, outf) ;
		
		try{ inps.close() ; outs.close() ; } 
		catch(Exception ex){}
		
		try{ sooc.disconnect() ; }
		catch(Exception ex){}
		
		return outs.toByteArray() ;
	}
}