/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */
package org.azuki.bean.ooo ;

import java.io.*;

import net.sf.jooreports.templates.*;

import org.azuki.bean.ooo.shared.UtilOOO;
import org.azuki.common.util.FileTool;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.xml.sax.InputSource;

import freemarker.ext.dom.NodeModel;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */
public class OOSampleBean
{
	final static public String PKGNAME = OOSampleBean.class.getPackage().getName().replace('.', '/') ;
	
	@DependencyInjectionInfo(desc="To convert document", param={String.class, byte[].class, String.class})
	public IMethodProxy<byte[]> convert ;
	
	@DependencyInjectionInfo(desc="To generate report", param={byte[].class, String.class})
	public IMethodProxy<byte[]> report ;
	
	@DependencyInjectionInfo(desc="To merge PDF files", param={byte[][].class})
	public IMethodProxy<byte[]> merge ;
	
	final public void doConvert()
	{
		try
		{
			// data
			InputStream dtas = OOSampleBean.class.getResourceAsStream("/"+PKGNAME+"/shared/sample/order-data.xml") ;
			InputSource dtai = new InputSource(dtas) ;
			NodeModel data = NodeModel.parse(dtai) ;
			
			// template
			InputStream tpls = OOSampleBean.class.getResourceAsStream("/"+PKGNAME+"/shared/sample/order-template.odt") ;
			DocumentTemplate tmpl = new ZippedDocumentTemplate(tpls) ;
			
			// generate ODT document
			ByteArrayOutputStream baos = new ByteArrayOutputStream() ;
			tmpl.createDocument(data, baos) ;
			try{ baos.close() ; } catch(Exception ex){}
			
			// converting to PDF
			byte[] pdf = convert.call("odt", baos.toByteArray(), "pdf") ;
			
			// write PDF file
			FileTool.writeFile("sample-convert-order"+System.currentTimeMillis()+".pdf", pdf) ;
		}
		catch(Throwable t)
		{
			Log.severe("Error while converting: "+Log.getStack(t)) ;
		}
	}
	
	final public void doReport()
	{
		try
		{
			// data
			InputStream dtas = OOSampleBean.class.getResourceAsStream("/"+PKGNAME+"/shared/sample/order-data.xml") ;
			byte[] byts = UtilOOO.read(dtas) ;
			
			// generate PDF file
			String outf = "sample-report-order"+System.currentTimeMillis()+".pdf" ;
			byte[] outb = report.call(byts, outf) ;
			
			// write PDF file
			FileTool.writeFile(outf, outb) ;
		}
		catch(Throwable t)
		{
			Log.severe("Error while generating: "+Log.getStack(t)) ;
		}
	}
	
	final public void doMerge()
	{
		try
		{
			// data
			InputStream dtas = OOSampleBean.class.getResourceAsStream("/"+PKGNAME+"/shared/sample/order-data.xml") ;
			InputSource dtai = new InputSource(dtas) ;
			NodeModel data = NodeModel.parse(dtai) ;
			
			// template
			InputStream tpls = OOSampleBean.class.getResourceAsStream("/"+PKGNAME+"/shared/sample/order-template.odt") ;
			DocumentTemplate tmpl = new ZippedDocumentTemplate(tpls) ;
			
			// generate ODT document
			ByteArrayOutputStream baos = new ByteArrayOutputStream() ;
			tmpl.createDocument(data, baos) ;
			
			try{ baos.close() ; } catch(Exception ex){}
			
			// converting to PDF
			byte[] pdf = convert.call("odt", baos.toByteArray(), "pdf") ;
			
			// merging with other PDF file
			InputStream ozis = OOSampleBean.class.getResourceAsStream("/"+PKGNAME+"/shared/sample/readme.pdf") ;
			byte[] ozr = UtilOOO.read(ozis) ;
			
			pdf = merge.call(new Object[]{new byte[][]{pdf, ozr}}) ;
			
			// write PDF file
			FileTool.writeFile("sample-merge-order"+System.currentTimeMillis()+".pdf", pdf) ;
		}
		catch(Throwable t)
		{
			Log.severe("Error while merging: "+Log.getStack(t)) ;
		}
	}
}