/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.ooo.editor ;

import java.awt.BorderLayout;
import java.util.List;

import javax.swing.BorderFactory;

import org.azuki.bean.ooo.shared.ConstOOO;
import org.azuki.common.consts.Const;
import org.azuki.common.editor.*;
import org.azuki.common.editor.basic.AbstractEditor;
import org.azuki.common.editor.jdom.*;
import org.azuki.common.util.*;
import org.azuki.common.widget.JFormPanel;
import org.jdom.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */
@SuppressWarnings("serial")
public class OOReportingEditor extends AbstractEditor<IEditorContext>
{
	final static private List<String> tableAttributes = ListUtil.asList(ConstOOO.REPO_IMAG_NAME, ConstOOO.REPO_IMAG_VALU) ;
	final static private List<String> tableHeaders = ListUtil.asList("Name", "Resource") ;
	
	JDOMAttributeField tmplField ;
	JDOMCheckBox publBox ;
	
	JFormPanel form ;
	
	JDOMElementTablePanel imagTable ;
	
	//
	// AbstractEditor
	//
	@Override
	final public void initParameters(Element para)
	{
		Element repo = JDOMUtil.getElement(para, ConstOOO.REPO) ;
		
		Attribute tmpl = JDOMUtil.getAttribute(repo, ConstOOO.REPO_TMPL) ;
		Attribute publ = JDOMUtil.getAttribute(repo, ConstOOO.REPO_PUBL) ;
		
		// properties form
		tmplField = new JDOMAttributeField(tmpl) ;
		
		publBox = new JDOMCheckBox(Const.YES, Const.NO) ;
		publBox.setAttribute(publ) ;
		
		form = new JFormPanel() ;
		form.addField(tmplField, "Template file") ;
		form.addField(publBox, "Publish ?") ;
		
		// images table
		imagTable = new JDOMElementTablePanel(repo, ConstOOO.REPO_IMAG, tableAttributes, tableHeaders) ;
		imagTable.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Image mapping"), BorderFactory.createEmptyBorder(5,5,5,5))) ;
		
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		setLayout(new BorderLayout()) ;
		add(form, BorderLayout.NORTH) ;
		add(imagTable, BorderLayout.CENTER) ;
	}
}