/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.oracle;

import java.sql.*;

import org.azuki.framework.annotation.*;
import org.azuki.framework.context.*;

public class DBMSAlertBean
{
	// Signal
	final static private String SQLSIG  = "{ call dbms_alert.signal('" ;
	final static private String SQLMID  = "', '" ;
	final static private String SQLEND = "')}" ;

	@Export 
	@MethodInfo(desc="Send a Oracle DBMS alert")
	final public void notifyDBMSAlert(@ArgInfo(name="name", desc="The alert name") String name,
									  @ArgInfo(name="mess", desc="The alert message") String mess)
	{

		Connection cx = null;
		try
		{
			if(mess==null) mess = "" ;
			
			String sql = new StringBuffer(SQLSIG).append(name)
			.append(SQLMID).append(mess).append(SQLEND).toString() ;
			
			cx = (Connection)ThreadContext.lookup(IContext.OBJECT_CONNECTION) ;
			
			CallableStatement cs = cx.prepareCall(sql) ;
			cs.execute() ;
			cs.close() ;
			
			Log.info("Send DBMS alert ("+name+")") ;

			cx.commit() ; // Send
		}
		catch(Exception e)
		{
			Log.severe(Log.getStack(e)) ;
		}
		finally
		{
			try
			{
				if (cx!=null)
					cx.close() ;
			}
			catch (SQLException e)
			{
//				e.printStackTrace();
				Log.severe("Error trying to close connection : "+Log.getStack(e));
			}
		}
	}
}