/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.oracle ;

import java.util.Map;

import org.azuki.bean.oracle.server.imp.JobWait;
import org.azuki.bean.oracle.shared.ConstDBMSAlert;
import org.azuki.common.editor.EditorUtil;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.command.Command;
import org.azuki.framework.context.Log;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;
import org.jdom.Element;

public class DBMSWaitoneBean
{
	@DependencyInjectionInfo(desc="publish", param={ Object.class, String.class, Object.class })
	public IMethodProxy<Void> publish ;

	private String eventname ;

	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element lmen)
	throws Exception
	{
		Map<String,String> para = EditorUtil.formToMap(lmen) ;
		if(para==null || para.isEmpty())
			throw new Exception("parameters are not set") ;
		
		eventname = para.get(ConstDBMSAlert.KEY_ALERT_NAME) ;
	}

	private JobWait job = null ;

	@MethodInfo(desc="Retrieve the current job status",
				retu="TRUE if teh job is currently running, FALSE otherwise")	
	final public boolean isRunning()
	{
		return job!=null ;
	}

	@MethodInfo(desc="Start the job and wait for DBMS alert")
	final public void start()
	{
		if(isRunning()) return ; // Already Started

		job = new JobWait(this, eventname) ;

		try
		{
			Command.send(job) ;
		}
		catch(Exception e)
		{
			Log.severe(e) ;
		}
	}

	@MethodInfo(desc="Stop the job")
	final public void stop()
	{
		if(!isRunning()) return ; // Already Stopped

		job.mustStop() ;

		job = null ;
	}
	
	@MethodInfo(desc="Retrieve the job informations",
				retu="The information text")
	public String getInfo()
	{
		return job==null ? "Not currently running" : job.getInfo() ;
	}
}