/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */
package org.azuki.bean.phpbridge.server.imp ;

import java.util.Arrays;

import org.azuki.bean.phpbridge.PhpJavaBridgeBean;
import org.azuki.common.bean.rpc.Service;
import org.azuki.common.util.Util;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class PhpJavaInvoker
{
	final static private String ASPECT_NAME = "PhpJavaBridge" ;
	
	static private int count = 0 ;
	
	final private String labl ;
	
	private PhpJavaBridgeBean phpb = null ;
	
	public PhpJavaInvoker(String labl)
	{
		this.labl = labl ; 
	}
	
	final public Object invoke(String name, Object... args)
	throws Exception
	{
		Log.info("PhpJavaInvoker try to call service ("+name+") ...") ;
		
		if(phpb==null)
		{
			IMetadata meta = Bean.searchFirst(ASPECT_NAME, labl) ;
			if(meta==null) throw new Exception("Bean not found: "+Util.oneString(ASPECT_NAME, labl)) ;
			
			phpb = (PhpJavaBridgeBean)meta.getBean() ;
			
			Log.info("PhpJavaInvoker found bridge ("+Util.oneString(ASPECT_NAME, labl)+")") ;
		}
		
		Service serv = phpb.getService(name) ;
		if(serv==null) throw new Exception("Service not found: "+serv) ;
		
		try
		{ 
			Log.info("--> START PHP Invoke ID="+count+" "+serv+" with arguments("+(args==null?null:Arrays.toString(args))+")") ;
			
			count++ ; 
			
			Object ores = serv.call(args) ;
						
			Log.info("<-- END PHP Invoke ID="+count) ;		

			return ores ;
		}
		catch(Exception ex)
		{ 
			Log.severe(Log.getStack(ex)) ; 
			throw ex ;
		}
	}
}