/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.queuing.server.shell ;

import org.azuki.bean.queuing.QueuingBean;
import org.azuki.bean.queuing.server.QueuingAdvice;
import org.azuki.bean.queuing.shared.ConstQueuing;
import org.azuki.common.util.*;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.*;

final public class QueuingShell
{
	final static public String listPointcuts(QueuingBean aspt)
	throws Exception
	{
		StringBuilder bdr = new StringBuilder() ;
		
		// headers
		StringBuilder tmp = new StringBuilder() ;
		tmp.append(ShellUtil.rpad("Bean", ConstQueuing.SZ_BEAN)) ;
		tmp.append(ShellUtil.rpad("Queue Size", ConstQueuing.SZ_QSIZE)) ;
		tmp.append(ShellUtil.rpad("Max Thread", ConstQueuing.SZ_THREAD)) ;
		tmp.append("\n") ;
		tmp.append(ShellUtil.rpad("", tmp.length(), '-')) ;
		tmp.append("\n") ;
		
		bdr.append(tmp) ;
		
		// lines
		for(IPointcut pc : Bean.getPointcuts(aspt.getMetadata().getID()))
		{
			QueuingAdvice qadv = (QueuingAdvice)pc.getAdvice() ;
			if(qadv==null) continue ;
			
			IMetadata tmeta = pc.getTargetMetadata() ;
			
			String qsiz = String.valueOf(qadv.getQueueSize()) ;
			String maxt = String.valueOf(qadv.getMaxThread()) ;
			
			bdr.append(ShellUtil.rpad(Util.oneString(tmeta.getName(), tmeta.getLabel()), ConstQueuing.SZ_BEAN)) ;
			bdr.append(ShellUtil.rpad(qsiz, ConstQueuing.SZ_QSIZE)) ;
			bdr.append(ShellUtil.rpad(maxt, ConstQueuing.SZ_THREAD)) ;
			bdr.append("\n") ;
		}
		
		return bdr.toString() ;
	}

	final static public String getAdviceInfo(QueuingAdvice advc)
	throws Exception
	{
		StringBuilder bdr = new StringBuilder() ;
		
		String maxt = String.valueOf(advc.getMaxThread()) ;
		String qsiz = String.valueOf(advc.getQueueSize()) ;
		
		bdr.append("Max Thread : ").append(maxt).append("\n") ;
		bdr.append("Queue Size : ").append(qsiz).append("\n") ;
		
		return bdr.toString() ;
	}
} 