/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.request ;

import java.lang.reflect.Method;
import java.util.*;

import org.azuki.bean.request.server.Base64AuthenticatorAdvice;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.DependencyInjectionInfo;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
final public class Base64AuthenticatorBean implements IAspect
{
	private IMetadata meta = null ; // cached
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	}
	
	@DependencyInjectionInfo(desc="Check user/password. Return true/false following authentification status, throws exception if an error occurs.",
							param={String.class, String.class})
	public IMethodProxy<Boolean> check ;
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	final public Object createAdvice(IPointcut ipc)
	throws Exception
	{
		IMetadata tmeta = ipc.getTargetMetadata() ;
		
		getMetadata().getContract().checkBindable(tmeta.getBean().getClass()) ;

		// List of selected methods
		Method[] methods = ipc.getSelectedMethods() ;
		List<String> lms = new ArrayList<String>() ;
		for(Method m : methods) lms.add(m.getName()) ;
		
		return new Base64AuthenticatorAdvice(this, lms) ;
	}
	
	final public boolean check(String user, String pass)
	throws Exception
	{
		if(check==null) throw new Exception("Undefined authentication method in base64 authenticator") ;
		return check.call(user, pass).booleanValue() ;
	}
}