/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.request.server;

import java.util.List;

import javax.servlet.http.*;

import org.azuki.bean.request.Base64AuthenticatorBean;
import org.azuki.bean.request.server.imp.Base64Access;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.*;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 */
public class Base64AuthenticatorAdvice implements IController
{
	final private Base64AuthenticatorBean auth ;
	final private List<String> meths ;
	
	public Base64AuthenticatorAdvice(Base64AuthenticatorBean auth, List<String> meths)
	{
		this.auth = auth ;
		this.meths = meths ;
	}
	
	final public Object intercept(InvocationTarget it)
	throws Exception
	{
		String targetmethod = it.getMethodName() ;
		if(!meths.contains(targetmethod))
			return it.proceed() ;
		
		// {HttpServlet.class, HttpServletRequest.class, HttpServletResponse.class} ;
		HttpServletRequest req = (HttpServletRequest)it.getParameters()[1];
		
		HttpSession sess = req.getSession(true) ;
		
		// improvement: check if not already registered
		Object ousr = sess.getAttribute(IContext.ATTR_USER) ;
		if(ousr!=null) return it.proceed() ;
		
		try
		{
			String[] up = Base64Access.getBasicAuth(req) ;
			if(up!=null) // null if not defined
			{
				String user = up[0] ;
				String pass = up[1] ;

				if(auth.check(user, pass))
				{
					sess.setAttribute(IContext.ATTR_USER, user) ;
					return it.proceed() ; // End
				}
			}
		}
		catch(Exception e)
		{
			Log.severe(e.getMessage()) ;
		}
		
		// Invalid authorization
		HttpServletResponse res = (HttpServletResponse)it.getParameters()[2];
		Base64Access.askForBasicAuth(res, "Azuki") ;
		
		return null ;
	}
}