/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.request.server ;

import java.lang.reflect.Method;

import org.azuki.bean.request.RequestHandlerBean;
import org.azuki.bean.request.shared.ConstRequestHandler;
import org.azuki.framework.aop.IPointcut;
import org.azuki.framework.bean.IMetadata;
import org.azuki.framework.context.Log;
import org.jdom.Element;

public class RequestHandlerAdvice
{
	final private String uri ;
	final public String getURI() { return uri; } 
	
	private RequestHandlerAdvice(String uri)
	{
		this.uri = uri ;
	}
	
	final static public RequestHandlerAdvice create(RequestHandlerBean rqd, IPointcut ipc)
	throws Exception
	{
		IMetadata tmeta = ipc.getTargetMetadata() ;
		
		rqd.getMetadata().getContract().checkBindable(tmeta.getBean().getClass()) ;

		Method[] methods = ipc.getSelectedMethods() ;
		
		if(methods==null || methods.length<1)
			throw new Exception("No method selected") ;

		if(methods.length>1)
			throw new Exception("This aspect can be applied to one method only") ;
		
		Method sel = methods[0] ;
		
		
		Element lm = ipc.getParameter().getChild(ConstRequestHandler.URI) ;
		if(lm==null) 
			throw new Exception("Undefine URI parameter") ;
		
		String uri = lm.getAttributeValue(ConstRequestHandler.NAME) ;
		
		String bnd = lm.getAttributeValue(ConstRequestHandler.BIND) ;
		boolean ex = ConstRequestHandler.EXACT.equals(bnd) ;
		boolean st = ConstRequestHandler.STARTWITH.equals(bnd) ;
		boolean en = ConstRequestHandler.ENDWITH.equals(bnd) ;
		boolean ma = ConstRequestHandler.MATCH.equals(bnd) ;
		
		RequestHandlerAdvice advice = null ;
		
		try
		{
			Integer targetid = tmeta.getID() ;
			
			String furi = "";
						
			if(ex) furi = rqd.bindExact(uri, targetid, sel) ;  
			if(st) furi = rqd.bindStart(uri, targetid, sel) ; 
			if(en)
			{
				furi = uri ;
				rqd.bindEnd  (uri, targetid, sel) ;
			}
			
			if(ma)
			{
				furi = uri ;
				rqd.bindMatch(uri, targetid, sel) ;  
			}
			
			advice = new RequestHandlerAdvice(uri) ;
			Log.info("\t\tBean ["+targetid+"] bind to ("+furi+")") ;
		}		
		catch(Exception e)
		{
			Log.severe(Log.getStack(e));
		}
		
		return advice ; // null if errors		
	}
}