/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.request.server.console ;

import java.lang.reflect.Method;
import java.util.*;

import javax.servlet.http.*;

import org.azuki.bean.request.RequestHandlerBean;
import org.azuki.bean.request.shared.ConstRequestHandler;
import org.azuki.common.bean.rpc.Service;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.Table;
import org.azuki.common.console.html.renderer.*;
import org.azuki.common.console.http.HttpResponse;
import org.azuki.common.console.renderer.HourRenderer;
import org.azuki.common.reflect.*;
import org.azuki.common.util.*;

public class RequestHandlerConsole
{
	final static public String[] tableHeaders = {"URI", "Binding", "Bean", "Method", "Description", "Nb call", "Min", "Max", "Average"} ;
	
	final static public TableCellRenderer signatureCellRenderer = new TooltipCellRenderer(50, ',') ;
	final static public TableCellRenderer descriptionCellRenderer = new TooltipCellRenderer(100) ;
	final static public TableCellRenderer centerAlignCellRenderer = new HorizontalAlignCellRenderer(HorizontalAlignCellRenderer.ALIGN_CENTER) ;
	
	final static public void service(RequestHandlerBean rha, HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{	
		List<List> data = new ArrayList<List>() ;
		
		addTo(ConstRequestHandler.EXACT, rha.getExactBindings(), data) ;
		addTo(ConstRequestHandler.STARTWITH, rha.getStartBindings(), data) ;
		addTo(ConstRequestHandler.ENDWITH, rha.getEndBindings(), data) ;
		addTo(ConstRequestHandler.MATCH, rha.getMatchBindings(), data) ;
	
		Table table = HtmlFactory.sortedTable(req, data, tableHeaders) ;
		table.setColumnRenderer(3, signatureCellRenderer) ;
		table.setColumnRenderer(4, descriptionCellRenderer) ;
		table.setColumnRenderer(5, centerAlignCellRenderer) ;
		table.setColumnRenderer(6, centerAlignCellRenderer) ;
		table.setColumnRenderer(7, centerAlignCellRenderer) ;
		table.setColumnRenderer(8, centerAlignCellRenderer) ;
		
		HttpResponse.print(res, table) ;
	}
	
	@SuppressWarnings("boxing")
	final static private void addTo(String name, Map<String, Service> map, List<List> dest)
	{
		for(Service serv : map.values())
		{
			Method meth = serv.meth ;
			
			String    tnam = Util.oneString(serv.name, serv.labl) ;
			Signature stur = new Signature(meth) ;
			String    desc = Annotations.getMethodDescription(meth) ;
				 
			if(serv.nbc==0)
			{
				dest.add(ListUtil.toList(serv.serv, name, tnam, stur, desc, 0, " - ", " - ", " - ")) ;
			}
			else 
			{
				String min = HourRenderer.get(serv.min) ;
				String max = HourRenderer.get(serv.max) ;
				String moy = HourRenderer.get(serv.tot/serv.nbc) ;
				
				dest.add(ListUtil.toList(serv.serv, name, tnam, stur, desc, serv.nbc, min, max, moy)) ;
			}
		}
	}
}