/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.scheduler.editor;

import java.awt.BorderLayout;
import java.util.List;

import javax.swing.*;

import org.azuki.bean.scheduler.editor.imp.*;
import org.azuki.bean.scheduler.shared.ConstSched;
import org.azuki.common.consts.Const;
import org.azuki.common.editor.jdom.JDOMCheckBoxPanel;
import org.azuki.common.editor.pointcut.AbstractPointcutEditor;
import org.jdom.Element;

@SuppressWarnings("serial")
public class SchedulerEditor extends AbstractPointcutEditor
{
	private TypeSelectorPanel typeSel ;
	private TimeSelectorPanel timeSel ;
	private DaySelectorPanel wDaySel, mDaySel ;

	private JPanel form ;
	
	private JDOMCheckBoxPanel week, month ;
	
	/*
	 * AbstractPointcutEditor
	 */
	@Override
	final public void initParameters(Element para)
	{
		Element typeLmen = para.getChild(ConstSched.TYPE) ;
		Element weekLmen = para.getChild(ConstSched.WEEK) ;
		Element mnthLmen = para.getChild(ConstSched.MONTH) ;
		
		typeSel = new TypeSelectorPanel() ;
		timeSel = new TimeSelectorPanel() ;
		
		JPanel timePane = new JPanel(new BorderLayout(10, 10)) ;
		timePane.add(typeSel, BorderLayout.WEST) ;
		timePane.add(timeSel, BorderLayout.CENTER) ;
		
		wDaySel = new DaySelectorPanel(DaySelectorPanel.WEEK) ;
	
		week = new JDOMCheckBoxPanel("All days of week", Const.YES, Const.NO) ;
//		week.setEnabledWhenCheck(false) ;
		week.add(wDaySel,BorderLayout.CENTER);
		
		mDaySel = new DaySelectorPanel(DaySelectorPanel.MONTH) ;
		
		month = new JDOMCheckBoxPanel("All days of month", Const.YES, Const.NO) ;
//		month.setEnabledWhenCheck(false) ;
		month.add(mDaySel,BorderLayout.CENTER);
		
		JPanel temp = new JPanel(new BorderLayout()) ;
		temp.add(week, BorderLayout.NORTH) ;
		temp.add(month, BorderLayout.SOUTH) ;
		
		form = new JPanel(new BorderLayout()) ;
		form.setBorder(BorderFactory.createEmptyBorder(10,10,10,10)) ;
		form.add(timePane, BorderLayout.NORTH) ;
		form.add(temp, BorderLayout.SOUTH) ;
		
		// chargement des donnees
		if(typeLmen!=null && weekLmen!=null && mnthLmen!=null)
		{
			typeSel.load(typeLmen.getAttribute(ConstSched.VALUE)) ;
			timeSel.load(typeLmen.getAttribute(ConstSched.HOUR), typeLmen.getAttribute(ConstSched.MINU), typeLmen.getAttribute(ConstSched.SECO)) ;
			
			wDaySel.load(weekLmen.getAttribute(ConstSched.DAYS)) ;
			mDaySel.load(mnthLmen.getAttribute(ConstSched.DAYS)) ;
			
			// gestion de l'affichage
			week.setAttribute(weekLmen.getAttribute(ConstSched.ALL)) ;
			month.setAttribute(mnthLmen.getAttribute(ConstSched.ALL)) ;
		}
		
		setDialogTitle("Scheduler") ;
		
		setLayout(new BorderLayout()) ;
		add(form, BorderLayout.NORTH) ;
	}

	@Override
	final public void initFields(List<Element> flds)
	{}

	@Override
	final public void initMethods(List<Element> mths)
	{}
}
