/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Mathieu Jucker
 */

package org.azuki.bean.scheduler.editor.imp;

import javax.swing.*;
import javax.swing.event.*;

import org.jdom.Attribute;

@SuppressWarnings("serial")
public class TimeSelectorPanel  extends JPanel implements ChangeListener
{
	private boolean editable = true ;
	
	private Attribute h, m, s ;
	
	private JSpinner.NumberEditor hedi, medi, sedi ;
	
	private JSpinner hour, minu, seco ;
	private SpinnerNumberModel hmdl, mmdl, smdl ;
	
	// CONSTRUCTEUR
	public TimeSelectorPanel()
	{
		super() ;
		
		hmdl = new SpinnerNumberModel(0, 0, 23, 1) ;
		mmdl = new SpinnerNumberModel(0, 0, 59, 1) ;
		smdl = new SpinnerNumberModel(0, 0, 59, 1) ;
		
		hour = new JSpinner(hmdl) ;
		hedi = new JSpinner.NumberEditor(hour, "00") ;
		hour.setEditor(hedi) ;
		hour.setMaximumSize(hour.getPreferredSize()) ;
		
		minu = new JSpinner(mmdl) ;
		medi = new JSpinner.NumberEditor(minu, "00") ;
		minu.setEditor(medi) ;
		minu.setMaximumSize(minu.getPreferredSize()) ;
		
		seco = new JSpinner(smdl) ;
		sedi = new JSpinner.NumberEditor(seco, "00") ;
		seco.setEditor(sedi) ;
		seco.setMaximumSize(seco.getPreferredSize()) ;
		
		JPanel pane = new JPanel() ;
		pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS)) ;
		
		pane.add(Box.createGlue()) ;
		pane.add(hour) ;
		pane.add(new JLabel("h ")) ;
		pane.add(minu) ;
		pane.add(new JLabel("m ")) ;
		pane.add(seco) ;
		pane.add(new JLabel("s ")) ;
		pane.add(Box.createGlue()) ;
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS)) ;
		add(Box.createGlue()) ;
		add(pane) ;
		add(Box.createGlue()) ;
	}
	
	final public void load(Attribute h, Attribute m, Attribute s)
	{
		this.h = h ;
		this.m = m ;
		this.s = s ;
		
		hour.removeChangeListener(this) ;
		minu.removeChangeListener(this) ;
		seco.removeChangeListener(this) ;

		hour.setValue(integerValue(h.getValue())) ;
		minu.setValue(integerValue(m.getValue())) ;
		seco.setValue(integerValue(s.getValue())) ;
		
		hour.addChangeListener(this) ;
		minu.addChangeListener(this) ;
		seco.addChangeListener(this) ;
	}
	
	final public boolean isEditable()
	{ return editable ; }
	
	final public void setEditable(boolean b)
	{
		hour.setEnabled(b) ;
		minu.setEnabled(b) ;
		seco.setEnabled(b) ;
		editable = b ;
	}
	
	final static private Integer integerValue(String s)
	{
		try					{ return Integer.valueOf(s) ; }
		catch(Exception e)	{ return new Integer(0) ; }
	}
	
	// ChangeListener
	final public void stateChanged(ChangeEvent e)
	{
		Object src = e.getSource() ;
		
		if(src==hour)
			h.setValue(String.valueOf(hour.getValue())) ;
		else if(src==minu)
			m.setValue(String.valueOf(minu.getValue())) ;
		else if(src==seco)
			s.setValue(String.valueOf(seco.getValue())) ;
	}
}
