/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.scheduler.server.imp ;

import java.util.*;

import org.azuki.framework.command.ICommand;
import org.azuki.framework.context.Log;


//Wrapper utilise pour l'execution d'un job schedulé

final public class JTT implements ICommand<Object>
{
	// Map of currently running job (to avoid execute twice)
	final static public Map<Integer, JTT> tasks = new HashMap<Integer, JTT>();
	
	// if  job is idle : it's scheduled but process() is never call until end of suspend mod
	private boolean isIdle = false;
	public boolean isSuspended(){ return isIdle; }
	public void suspendJob(boolean suspend){ isIdle = suspend; }
	
	static public int count = 0;	
	
	private Integer ID ;
	private ICommand<?> jc ;

	private ATimer timer ;

	private String nfo ;
	private String jobName;
	
	public String getJobName() { return jobName; }
	public ATimer getTimer() { return timer ;}
	
	final private SchedulerImp isched ;
	
	public JTT(ICommand<?> jc, String jobName, ATimer timer, SchedulerImp isched)
	{
		this(jc, jobName, isched) ;
		this.timer = timer ;
	}

	public JTT(ICommand<?> jc, String jobName, SchedulerImp isched) // Schedule once
	{
		this.jc = jc ;
		this.jobName = jobName ;
		this.isched = isched ;
		ID = new Integer(++count) ;
		nfo = new StringBuilder("SCHEDULER[").append(jobName).append(']').toString() ;
	}


	final public String getName() { return nfo; }

	final public Integer getID()
	{
		return ID ;
	}

	final public Object process()
	throws Exception
	{
		try
		{
		
			// Check for day
			if(timer!=null)
			{
				Calendar dar = Calendar.getInstance() ;
				int today = dar.get(Calendar.DAY_OF_WEEK);
	
				if(!timer.isDayOfWeek(today)) return null;
	
				today = dar.get(Calendar.DAY_OF_MONTH) ;
	
				if(!timer.isDayOfMonth(today)) return null;
			}
	
			synchronized(tasks)
			{
				if(tasks.containsKey(ID)) return null; // skip this execution
				tasks.put(ID, this) ;
			}
			
			if (!isIdle) jc.process();  // Can throught exception
		}
		catch(Throwable t)
		{
			Log.severe("Schedule error : " + t.getMessage()) ;
		}
		
		synchronized(tasks)
		{
			tasks.remove(ID) ;
		}
		 
		// FIX BUG MJ : can't schedule a task to be execute juste one time at a fixed time : timer is never null, but if period is 0, no schedule anymore , juste one ine.
		if(timer!=null && timer.getPeriod()==0)  // Scheduled once only.
			isched.cancel(ID) ; // To remove from map of running job.

		return null ;
	}
}