/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.shell ;

import java.util.Map;

import org.azuki.bean.shell.server.imp.TcpListener;
import org.azuki.bean.shell.server.shell.Shell;
import org.azuki.bean.shell.shared.AttrShell;
import org.azuki.common.bean.rpc.AbstractRpcBean;
import org.azuki.common.editor.EditorUtil;
import org.azuki.framework.annotation.*;
import org.azuki.framework.bean.*;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class ShellBean extends AbstractRpcBean
{
	private int port ;
	final public int getPort(){ return port; }
	
	private String charSet ;
	final public String getCharSet() { return charSet ; }
	
	private TcpListener lsnr ;
	
	private IMetadata meta = null ; 
	final public IMetadata getMetadata()
	throws Exception
	{
		if(meta==null) meta = Bean.getMetadata(this) ;
		return meta ;
	} 
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
    final public void postCreate(Element para)
    throws Exception
    {
		// Parameters
		Map<String, String> map = EditorUtil.formToMap(para) ;
    	if (map==null)
    		throw new Exception("ERROR : No parameter found");

		String sport = map.get(AttrShell.PORT) ;
		if (sport==null || sport.isEmpty())
    		throw new Exception("ERROR : Port parameter is not set") ;

		try
    	{
    		port = Integer.parseInt(sport) ;
    	}
    	catch(NumberFormatException nfe)
    	{
    		throw new Exception("ERROR : Port parameter is not a valid Number", nfe);
    	}
    		
	    charSet = map.get(AttrShell.CHAR) ;
	    	
	    if (charSet==null || charSet.isEmpty())
	    	throw new Exception("ERROR : charSet parameter is not set") ;
	    
	    lsnr = new TcpListener(this) ;
    }
	
    // methods available to start and stop the shell server via JMX
    final public boolean isRunning() 	{ return lsnr.isRunning() ; }
    final public void start()			{ lsnr.start() ; }
	final public void stop()			{ lsnr.stop() ; }
	
	@Export
	@MethodInfo(desc="Build the usage text of the specified service", retu="The service's usage text")
	final public String usage(@ArgInfo(name="serviceName", desc="The name of the service") String serviceName)
	throws Exception
	{
		return Shell.usage(this, serviceName) ;
	}
	
	@Export
	@MethodInfo(desc="Build the bean documentation of the specified service", retu="The service's documentation")
	final public String help(@ArgInfo(name="serviceName", desc="The name of the service") String serviceName)
	throws Exception
	{
		return Shell.help(this, serviceName) ;
	}
	
	@Export
	@MethodInfo(desc="List all services registered for the ShellCmd bean's instance", retu="The services listing")
	final public String list()
	throws Exception
	{
		return Shell.list(this) ;
	}
}