/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.shell.server.imp ;

import java.io.*;
import java.net.Socket;
import java.util.Arrays;

import org.azuki.bean.shell.ShellBean;
import org.azuki.bean.shell.server.shell.Shell;
import org.azuki.bean.shell.shared.ConstShell;
import org.azuki.common.bean.rpc.Service;
import org.azuki.common.util.StringUtil;
import org.azuki.framework.command.ICommand;
import org.azuki.framework.context.Log;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class TcpHandler<T> implements ICommand<T>
{
	final private ShellBean shel ;
	final private Socket sock ;
	
	public TcpHandler(ShellBean shel, Socket sock)
	{
		this.shel = shel ;
		this.sock = sock ;

		try
		{ sock.setSoTimeout(ConstShell.TIMEOUT) ; }
		catch(Exception e)
		{ Log.severe(Log.getStack(e)) ; }
	}
	
	//
	// ICommand
	//
	final public T process()
	throws Exception
	{
		/*
		 * Command-line
		 * - no arguments: display the services list
		 * - serviceName [arguments]
		 */
		
		BufferedReader sin = new BufferedReader(new InputStreamReader(sock.getInputStream())) ;
		BufferedOutputStream sou = new BufferedOutputStream(sock.getOutputStream()) ;
		
		// Read from socket
		String line = sin.readLine() ;
		
		line.trim() ;
		Log.info("ShellCmd receive ("+line+")") ;			
		
		int retc = 0 ;	// 0:OK  1:KO
		
		StringBuilder bdr = new StringBuilder() ; // message content
		
		String[] cmdl = StringUtil.splitQuote(line) ;
		if(cmdl.length==0)
		{
			bdr.append(Shell.list(shel)) ;
		}
		else
		{
			String snam = cmdl[0] ;
			
			Service serv = shel.getService(snam) ;
			if(serv==null)
			{
				String errm = "Service not found: "+snam ;
				bdr.append(errm) ;
				Log.severe(errm) ;
			}
			else
			{
				String[] args = new String[cmdl.length-1] ;
				System.arraycopy(cmdl, 1, args, 0, args.length) ;
				Log.info("Call "+serv+" with arguments ("+Arrays.asList(args)+")") ;
				
				Class rtyp = serv.meth.getReturnType() ;
				
				Object outp = null ;
				
				try
				{
					outp = serv.call((Object[])args) ;
					
					if(Void.class.isAssignableFrom(rtyp))
						outp = "Service ("+serv.name+") called successfully" ;
				}
				catch(Throwable t)
				{
					retc = 1 ;
					outp = Log.getStack(t)+"\n"+Shell.usage(shel, serv.serv) ;
					
					Log.severe(t) ;
				}
				
				if(outp!=null) bdr.append(outp) ;
			}
		}
		
		if(bdr.length()>0) bdr.append('\n') ;
		
		// Write data to socket
		String charSet = shel.getCharSet() ;
						
		sou.write(String.valueOf(retc).getBytes(charSet)) ;
		sou.write("\n".getBytes(charSet)) ;
		sou.write(bdr.toString().getBytes(charSet)) ;
		sou.flush() ;
		
		// close streams
		try
		{ sin.close() ; sou.close() ; sock.close() ; }
		catch(Exception ex)
		{ Log.severe(Log.getStack(ex)) ; }
		
		return null ;
	}
}