/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.sql;

import java.util.*;

import org.azuki.common.util.SQL;


/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 */


// mkv : Master key-value 

public class DatasetBean
{
	
	final static public List<Object[]> selectDataset(String table, String[] fields, String where, String orderby, Integer limit)
	throws Exception
	{
		StringBuilder sb = new StringBuilder("select ") ;
		
		for(int i=0; i<fields.length; i++)
		{   
			if(i!=0) sb.append(", ") ;	
			sb.append(fields[i]) ;
		}
		
		sb.append(" from ").append(table) ;

		if(where!=null && !where.isEmpty())
			sb.append(" where ").append(where) ;
		
		if(orderby!=null && !orderby.isEmpty())
			sb.append(" order by ").append(orderby) ;
		
		if(limit.intValue()> -1)
			sb.append(" LIMIT ").append(limit) ;
			
		sb.append(';') ;
				
		return SQL.query(sb.toString()) ;
	}

	final static public Integer[] majDataset(String table, String[] fields, String[] keys, Object[] delval, Object[] updval, Object[] addval, boolean bseq)
	throws Exception
	{
		if(delval!=null && delval.length>0)
			deleteDataset(table, keys, delval) ;
	
		if(updval!=null && updval.length>0)
			updateDataset(table, fields, keys, updval) ;

		Integer[] res = null ;
		
		if(addval!=null && addval.length>0)
			res = insertDataset(table, fields, addval, bseq) ;			
		
		return res ;
	}
	
	
	final static public Integer[] insertDataset(String table, String[] fields, Object[] val, boolean bseq)
	throws Exception
	{
		StringBuilder sb = new StringBuilder("insert into ") ;
		sb.append(table).append(" (") ;
		
		for(int i=0; i<fields.length; i++)
		{   if(i!=0) sb.append(", ") ;
			sb.append(fields[i]) ; }

		sb.append(") values (") ;

		for(int i=0; i<fields.length; i++)
		{   if(i!=0) sb.append(',') ;
			sb.append('?') ; }
		
		sb.append(')') ;
		
		Integer[] res = null ;
		if(bseq) res = SQL.updateArraySequence(sb.toString(), table, val) ;
		else	 SQL.updateArray(sb.toString(), val) ; 
		
		return res ;
	}

	// fields contains all columns (with the key)
	// val contains all data, also key values
	
	final static public void updateDataset(String table, String[] fields, String[] keys, Object[] val)
	throws Exception
	{
		// val must be sorted
		List<Object[]> setlist = new ArrayList<Object[]>() ;
		List<Object[]> keylist = new ArrayList<Object[]>() ;
		
		boolean foundwc = false; 
		StringBuilder wc = new StringBuilder(" where ") ;
		
		boolean foundset = false ;
		StringBuilder sb = new StringBuilder("update ") ;
		sb.append(table).append(" set ") ;
				
		for(int i=0; i<fields.length; i++)
		{
			String field = fields[i] ;
			
			// where ?
			int idx = find(keys, field) ;
			
			if(idx==-1) // set
			{
				setlist.add((Object[])val[i]) ;
				
				if(foundset) sb.append(", ");
				else		 foundset = true;
				sb.append(field).append(" =?" );
				continue ;
			}
		
			// else : where
			keylist.add((Object[])val[i]);
			
			if(foundwc) wc.append("and ");
			else        foundwc = true;		
			
			wc.append(field).append(" =? ");
		}

		if(foundwc)	sb.append(wc.toString());
		
		Object[] oos = new Object[val.length] ;
		
		int i=0 ;
		
		for(Object[] oo : setlist) oos[i++] = oo ;
		for(Object[] oo : keylist) oos[i++] = oo ;
		
		SQL.updateArray(sb.toString(), oos) ; 
	}
	
	final static private int find(String[] tab, String val)
	{
		for(int i=0; i<tab.length; i++)
			if(val.equalsIgnoreCase(tab[i])) return i ;
			
		return -1 ;
	}

	final static public void deleteDataset(String table, String[] keys, Object[] val)
	throws Exception
	{
		boolean found = false ;

		StringBuilder sb = new StringBuilder("delete from ") ;
		sb.append(table) ;
		
		if(keys!=null && keys.length > 0)
		{
			sb.append(" where ") ;

			for(int i=0; i<keys.length; i++)
			{
				if(found) sb.append("and ") ;
				else	  found = true ;
				
				sb.append(keys[i]) ;
				sb.append("=? ") ;
			}			
		}
		
		SQL.updateArray(sb.toString(), val) ; 
	}
}