/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.bean.tomcat ;

import javax.servlet.http.*;

import org.azuki.bean.tomcat.server.console.TomcatConnectorConsole;
import org.azuki.bean.tomcat.shared.ConstTomcat;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.MethodInfo;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.injection.IMethodProxy.*;
import org.jdom.Element;

/**
 *   @author Yoann Perrin
 */

final public class TomcatConnectorBean
{
	// Name of the bean, must be equal to the name in xml definition file
	final static public String NAME = "TomcatConnector" ;
	
	// Defaults values	
	private int port ; // = 8081 ;
	private String scheme ; // = HttpConstants.SCHEME_HTTP ;
	
	private String keystoreFile ; // = "" ;
	private String keystorePass ; // = "" ;
	
	private int maxThreads ; // = 200 ;
	private int minSpareThreads ; // = 4 ;
	private int maxSpareThreads ; // = 50 ;
	private int acceptCount ; // = 10 ;
	private int threadPriority ; // = PConnector.THREAD_NORMAL ;
	
	// Getter
	final public int getPort()			  { return port ; }
	final public String getScheme()		  { return scheme ; }
	final public String getKeystoreFile() { return keystoreFile ; }
	final public String getKeystorePass() { return keystorePass ; }
	final public int getMaxThreads()	  { return maxThreads; }
	final public int getMinSpareThreads() { return minSpareThreads; }
	final public int getMaxSpareThreads() { return maxSpareThreads; }
	final public int getAcceptCount() 	  { return acceptCount; }
	final public int getThreadPriority()  { return threadPriority; }

	@DependencyInjectionInfo(desc="To declare this Connector bean to a Tomcat Engine", 
							param={TomcatConnectorBean.class})
	public IMethodProxy<Void> addConnector ;
	
	@MethodInfo(desc="Initializer contract implementation. Called at the framework initialization. Must not be called by others means")
	final public void postCreate(Element para)
	throws Exception
	{
		try
		{
			Element connLmen = JDOMUtil.getElement(para, ConstTomcat.CONNECTOR) ;
			String portStr = connLmen.getAttributeValue(ConstTomcat.CONNECTOR_PORT, ConstTomcat.defaultPort) ;
			
			Element secuLmen = JDOMUtil.getElement(connLmen, ConstTomcat.SECURITY) ;
			
			Element poolLmen = JDOMUtil.getElement(connLmen, ConstTomcat.POOL) ;
			String maxtStr = poolLmen.getAttributeValue(ConstTomcat.POOL_MAXT, ConstTomcat.defaultMaxThreads) ;
			String minsStr = poolLmen.getAttributeValue(ConstTomcat.POOL_MINS, ConstTomcat.defaultMinSpareThreads) ;
			String maxsStr = poolLmen.getAttributeValue(ConstTomcat.POOL_MAXS, ConstTomcat.defaultMaxSpareThreads) ;
			String accpStr = poolLmen.getAttributeValue(ConstTomcat.POOL_ACCP, ConstTomcat.defaultAcceptCount) ;
			String prioStr = poolLmen.getAttributeValue(ConstTomcat.POOL_PRIO, ConstTomcat.defaultThreadPriority) ;
			
	        // connector
			scheme = connLmen.getAttributeValue(ConstTomcat.CONNECTOR_SCHM, ConstTomcat.defaultScheme) ;
			port   = Integer.parseInt(portStr) ;
	        
	        // security
	        keystoreFile = secuLmen.getAttributeValue(ConstTomcat.SECURITY_KFIL, "") ;
	        keystorePass = secuLmen.getAttributeValue(ConstTomcat.SECURITY_KPAS, "") ;
	        try
	        { keystorePass = Crypto.decypher(keystorePass) ; }
	        catch(Exception e)
	        { /* on ne fait rien */ }
	        
	        // pool
	        maxThreads		= Integer.parseInt(maxtStr) ; 
	        minSpareThreads	= Integer.parseInt(minsStr) ;
	        maxSpareThreads	= Integer.parseInt(maxsStr) ;
	        acceptCount 	= Integer.parseInt(accpStr) ;
	        threadPriority  = Integer.parseInt(prioStr) ;
	        
	        addConnector.call(this) ;
		}
		catch(Exception e)
		{
			e.printStackTrace() ;
			throw e ;
		}
	}
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		TomcatConnectorConsole.service(this, srv, req, res) ; 
	}
}