/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Yoann Perrin
 */

package org.azuki.bean.tomcat.server.imp ;

import java.util.*;

import org.apache.catalina.connector.Connector;
import org.apache.catalina.startup.Embedded;
import org.azuki.bean.tomcat.TomcatConnectorBean;
import org.azuki.framework.context.Log;

/**
 * Tomcat connector container
 */
public class PConnector
{	
	final static public void attach(List<TomcatConnectorBean> ltc, Embedded embe)
	{
		for(TomcatConnectorBean tcb : ltc)
			add(embe, tcb) ;
	}
	
	// configure connector threading properties
	final static private void add(Embedded embe, TomcatConnectorBean tc)
	{
		int port = tc.getPort() ;
		String scheme = tc.getScheme() ;
		
		Connector tmp = findConnector(embe, port) ;
		if(tmp!=null)
		{	Log.severe("Connector already bind on port "+port) ;
			return ; }
			
		Log.info("Tomcat connector listen: "+scheme+"-"+port) ;
		
		Connector conn = embe.createConnector((String)null, port, scheme) ;
		
		if(scheme.equals(HttpConstants.SCHEME_HTTPS))
		{
			conn.setProperty("sslProtocol", "TLS") ;
			conn.setProperty("keystoreFile", tc.getKeystoreFile()) ;
			conn.setProperty("keystorePass", tc.getKeystorePass()) ;
		}
		
		conn.setProperty("maxThreads", String.valueOf(tc.getMaxThreads())) ;
    	conn.setProperty("acceptCount", String.valueOf(tc.getAcceptCount())) ;
    	conn.setProperty("threadPrioriy", String.valueOf(tc.getThreadPriority())) ;
    	conn.setProperty("minSpareThreads", String.valueOf(tc.getMinSpareThreads())) ;
    	conn.setProperty("maxSpareThreads", String.valueOf(tc.getMaxSpareThreads())) ;
    	
    	embe.addConnector(conn) ;
	}
	
	// return a connector instance if found, else NULL
	final static private Connector findConnector(Embedded em, int port)
	{
		for(Connector c : em.findConnectors())
			if(c.getPort()==port) return c ;
		
		return null ;
	}
}
	