/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.transaction ;

import org.azuki.bean.transaction.server.*;
import org.azuki.bean.transaction.shared.ConstTransaction;
import org.azuki.common.util.JDOMUtil;
import org.azuki.framework.aop.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

final public class TransactionBean extends TransactionFactory implements IAspect<TransactionManagerAdvice>
{
	public enum DTAction { NOTHING, COMMIT, ROLLBACK }

	final public TransactionManagerAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		Element paraLmen = ipc.getParameter() ;
		
		Element tranLmen = JDOMUtil.getElement(paraLmen, ConstTransaction.TRAN) ;
		
		String bforStr = tranLmen.getAttributeValue(ConstTransaction.TRAN_BFOR, DTAction.NOTHING.name()) ;
		String aftrStr = tranLmen.getAttributeValue(ConstTransaction.TRAN_AFTR, DTAction.NOTHING.name()) ;
		String erroStr = tranLmen.getAttributeValue(ConstTransaction.TRAN_ERRO, DTAction.NOTHING.name()) ;
		
		DTAction bfor = DTAction.NOTHING ;
		try
		{ bfor = DTAction.valueOf(bforStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid data transaction before action: "+bforStr+" using default.") ; }
		
		DTAction aftr = DTAction.NOTHING ;
		try
		{ aftr = DTAction.valueOf(aftrStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid data transaction after action: "+aftrStr+" using default.") ; }
		
		DTAction erro = DTAction.NOTHING ;
		try
		{ erro = DTAction.valueOf(erroStr) ; }
		catch(Exception ex)
		{ Log.warning("Invalid data transaction on error action: "+erroStr+" using default.") ; }
		 
		return new TransactionManagerAdvice(bfor, aftr, erro) ;
	}
}