/**
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2007 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 *
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

package org.azuki.bean.transaction.server;

import java.sql.Connection;

import org.azuki.bean.transaction.TransactionBean.DTAction;
import org.azuki.framework.context.*;
import org.azuki.framework.injection.*;

public class TransactionManagerAdvice implements IController
{
	final private DTAction before  ;
	final private DTAction after   ;
	final private DTAction onerror ;
	
	public TransactionManagerAdvice(DTAction before, DTAction after, DTAction onerror)
	{
		this.before  = before ;
		this.after   = after ;
		this.onerror = onerror ;
	}
	
	final public Object intercept(InvocationTarget it)
	throws Exception
	{
		IContext ictx = ThreadContext.get() ;
				
		doAction(before, ictx) ;
		
		try
		{ 
			Object result = it.proceed() ;

			doAction(after, ictx) ;
			
			return result ;
		}
		catch(Exception e)
		{
			doAction(onerror, ictx) ;
			throw e ; 
		}
	}	
	
	final static private void doAction(DTAction action , IContext ictx)
	throws Exception
	{
		if(action==DTAction.NOTHING) return ;

		Connection cx  = (Connection)ictx.getObject(IContext.OBJECT_CONNECTION) ;
		if(cx==null) return ;
		
		if(cx.isClosed())
		{	Log.warning("Connection already closed while trying to perform commit/rollback") ;
			return ; }
		
		if(action==DTAction.COMMIT)
		{
			cx.commit() ;
			return ;
		}

		// Rollback
		cx.rollback() ;
	}
}