/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.bean.rpc ;

import java.lang.reflect.Method;
import java.util.*;

import javax.servlet.http.*;

import org.azuki.common.bean.rpc.console.RpcServiceConsole;
import org.azuki.common.bean.rpc.server.RpcServiceAdvice;
import org.azuki.common.bean.rpc.shared.ConstRpc;
import org.azuki.common.consts.Tag;
import org.azuki.common.util.*;
import org.azuki.framework.annotation.*;
import org.azuki.framework.aop.*;
import org.azuki.framework.bean.*;
import org.azuki.framework.context.Log;
import org.jdom.Element;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

abstract public class AbstractRpcBean implements IAspect<RpcServiceAdvice>
{
	final private Map<String,Service> srvs = new HashMap<String,Service>() ;
	final private List<Service> errs = new ArrayList<Service>() ;			
	
	@Export
	final public Map getServiceMap()
	{
		return Collections.unmodifiableMap(srvs) ;
	}
	
	final public Service getService(String name)
	{
		return srvs.get(name) ;
	}
	
	final public Collection<Service> getServices()
	{
		return srvs.values() ;
	}
	
	final public Collection<Service> getServicesInErrors()
	{
		return Collections.unmodifiableCollection(errs) ;
	}
	
	@MethodInfo(desc="Called at the framework initialisation to apply a pointcut. Must not be called by others means")
	public RpcServiceAdvice createAdvice(IPointcut ipc)
	throws Exception
	{
		IMetadata meta = ipc.getAspectMetadata() ;
		IMetadata tmeta = ipc.getTargetMetadata() ;

		IContract ctrt = meta.getContract() ;
		if(ctrt!=null) ctrt.checkBindable(tmeta.getBeanClass()) ;
		
		for(Method m : ipc.getSelectedMethods())
		{
			String snam = m.getName() ;
			
			Element mpar = ipc.getElement(m) ;
			
			Element para = mpar.getChild(Tag.PARA) ;
			if(para!=null)
			{
				Element lmen = para.getChild(ConstRpc.SERV) ;
			
				String temp = lmen==null ? null : lmen.getAttributeValue(ConstRpc.SERV_NAME) ;
				if(!StringUtil.isEmpty(temp)) snam = temp ;
			}
			
			Service serv = new Service(snam, tmeta.getName(), tmeta.getLabel(), m) ;
			
			Service othr = srvs.get(snam) ;
			if(othr!=null)
			{	Log.severe("Service("+snam+") already registered: "+othr) ;
				errs.add(serv) ;
				continue ; }
			
			srvs.put(snam, serv) ;
			
			Log.info(serv.toString()) ;
		}		
		
		return new RpcServiceAdvice() ;
	}	
	
	@MethodInfo(desc="The contract implementation of the application portal aspect (AppPortal) for the Console application")
	final public void console(HttpServlet srv, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		RpcServiceConsole.p(this, srv, req, res) ;
	}	
}