/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.bean.rpc ;

import java.lang.reflect.Method;

import org.azuki.common.util.Util;
import org.azuki.framework.injection.IMethodProxy;
import org.azuki.framework.object.Locator;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class Service
{
	final public String serv ; // Nom du service (egalement utilisé pour stocké une URI)
	final public String name ;
	final public String labl ;
	final public Method meth ;
	
	// Stat
	public int  nbc = 0 ; // nb d'appel
	public long min = Long.MAX_VALUE ; 
	public long max = 0 ; 
	public long tot = 0 ;
	
	public Service(String serv, String name, String labl, Method meth)
	{
		this.serv = serv ;
		this.name = name ;
		this.labl = labl ;
		this.meth = meth ;
	}
	
	private IMethodProxy mp = null ; // Cached, to speed up proxy creation
	final public Object call(Object... arg)
	throws Exception
	{
		if(mp==null)  mp = (IMethodProxy)Locator.create(Locator.OBJECT_METHOD_PROXY, name, labl, meth.getName());
		
		long deb = System.currentTimeMillis() ;

		Object o = mp.call(arg);    
		
		long fin = System.currentTimeMillis() ;
		
		// If exception, pas comptabilisé....
		addTime(fin-deb) ;			

		return o ;
	}
	
	final private synchronized void addTime(long time)
	{
		nbc++;
		tot+=time;
		if(min>time) min=time;
		if(max<time) max=time;
	}
	
	private String toStr = null; // Lazy
	@Override
	final public String toString()
	{
		if(toStr==null)
			toStr = new StringBuilder("Service(").append(serv).append(") --> (")
			.append(Util.oneString(name, labl)).append(".").append(meth.getName())
			.append(")").toString() ;
		return toStr ;
	}
}