/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.bean.rpc.console ;

import java.lang.reflect.Method;
import java.util.*;

import javax.servlet.http.*;

import org.azuki.common.bean.rpc.*;
import org.azuki.common.console.HtmlFactory;
import org.azuki.common.console.html.*;
import org.azuki.common.console.html.renderer.*;
import org.azuki.common.console.http.HttpResponse;
import org.azuki.common.console.renderer.HourRenderer;
import org.azuki.common.reflect.*;
import org.azuki.common.util.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class RpcServiceConsole
{
	final static public String[] tableHeaders = {"Service", "Bean", "Method", "Description", "Nb call", "Min", "Max", "Average"} ;
	final static public String[] errorHeaders = {"Service", "Bean", "Method", "Description", "Cause"} ;
	
	final static public TableCellRenderer signatureCellRenderer = new TooltipCellRenderer(50, ',') ;
	final static public TableCellRenderer descriptionCellRenderer = new TooltipCellRenderer(100) ;
	final static public TableCellRenderer centerAlignCellRenderer = new HorizontalAlignCellRenderer(HorizontalAlignCellRenderer.ALIGN_CENTER) ; 
	
	@SuppressWarnings("boxing")
	final public static void p(AbstractRpcBean pa, HttpServlet servlet, HttpServletRequest req, HttpServletResponse res)
	throws Exception
	{
		// services with errors
		List<List> errs = new ArrayList<List>() ;
		for(Service serv : pa.getServicesInErrors())
		{
			Method meth = serv.meth ;
			
			String    tnam = Util.oneString(serv.name, serv.labl) ;
			Signature stur = new Signature(meth) ;
			String    desc = Annotations.getMethodDescription(meth) ;
			
			Service caus = pa.getService(serv.serv) ;
			
			errs.add(ListUtil.toList(serv.serv, tnam, stur, desc, "Already registered: "+caus)) ;
		}
		
		// services
		List<List> data = new ArrayList<List>() ;
		
		for(Service serv : pa.getServices())
		{
			Method meth = serv.meth ;
			
			String    tnam = Util.oneString(serv.name, serv.labl) ;
			Signature stur = new Signature(meth) ;
			String    desc = Annotations.getMethodDescription(meth) ;
				 
			if(serv.nbc==0)
			{
				data.add(ListUtil.toList(serv.serv, tnam, stur, desc, 0, " - ", " - ", " - ")) ;
			}
			else 
			{
				String min = HourRenderer.get(serv.min) ;
				String max = HourRenderer.get(serv.max) ;
				String moy = HourRenderer.get(serv.tot/serv.nbc) ;
				
				data.add(ListUtil.toList(serv.serv, tnam, stur, desc, serv.nbc, min, max, moy)) ;
			}
		}
		
		Table table = HtmlFactory.sortedTable(req, data, tableHeaders) ;
		table.setColumnRenderer(2, signatureCellRenderer) ;
		table.setColumnRenderer(3, descriptionCellRenderer) ;
		table.setColumnRenderer(4, centerAlignCellRenderer) ;
		table.setColumnRenderer(5, centerAlignCellRenderer) ;
		table.setColumnRenderer(6, centerAlignCellRenderer) ;
		table.setColumnRenderer(7, centerAlignCellRenderer) ;
		
		HtmlBuilder bdr = new HtmlBuilder() ;
		
		if(!errs.isEmpty())
		{
			Table temp = HtmlFactory.sortedTable(req, errs, errorHeaders) ;
			temp.setColumnRenderer(2, signatureCellRenderer) ;
			temp.setColumnRenderer(3, descriptionCellRenderer) ;
			
			bdr.print("<h2>Errors</h2>") ;
			bdr.println(temp) ;
			bdr.println("<br/>") ;	
		}
		
		bdr.print("<h2>Registered services</h2>") ;
		bdr.println(table) ;
		
		HttpResponse.print(res, bdr) ;
	}
}