<?xml version="1.0" encoding="UTF-8"?>

<!-- 
	Created the 2 April 2006
	
	Azuki Framework
	Copyright 2006 the original author or authors.
	
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.
	
	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
	
	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
	
	Contact : support@azuki-framework.org
	
	@author Yoann Perrin (yoann.perrin@gmail.com)
-->

<!--
	pour &nbsp; -> &#xA0;
	pour &amp;  -> &#x26;
 -->
 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" encoding="UTF-8"/>
	<xsl:template match="/">
		<html>
			<head>
				<style>
					body {
						margin: 0px;
						color: black;
						background: none;
						font: sans-serif;
					}
					h1, h2, h3, h4, h5, h6 {
						width: 100%;
						color: black;
						background: none;
						font-weight: normal; 
						margin: 0;
						padding-top: 0.5em;
						padding-bottom: .17em; 
					}
				
					h1 {
						height : 1.3em;
						padding : 0.3em 0.5em 0 ; 
						color: #FFFFFF;
						background: #B10B1E; 
						font-size: 120%; 
						font-weight: bold;
					}
					h2 {
						margin : 0.5em; 
						color: #B10B1E;
						background: white;
						font-size: 115%;
						font-weight: bold;
						border-bottom: 1px solid #B10B1E;
					}
					h3, h4, h5, h6 {
						padding-left: 1.5em; 
						color: black;
						background: white; 
						font-weight: bold;
						border-bottom: none; 
					}
					h3 { font-size: 110%; }
					h4 { font-size: 105%; } 
					h5 { font-size: 100%; }
					h6 { font-size: 80%; }

					.label {
						text-align: right ;
						font-weight: bold;
						white-space: nowrap;
					}
					
					.bean {
						font-weight: bold;
					}
					.signature {
						font-weight: bold;
					}
					
					.definition-content {
						padding: 5px 0px 0px 20px;
					}
					.definition-subcontent {
						padding: 5px 0px 0px 20px;
					}
					
					.cell-entry {
						padding: 5px 0px 5px 20px;
					}
					
					.p {
						padding-left: 20px;
					}
				</style>
			</head>
			<body>
				<xsl:apply-templates select="/beandoc/definition"/>
				<br/>
				<xsl:apply-templates select="/beandoc/documentation"/>
				<br/>
			</body>
		</html>
	</xsl:template>
	
	<xsl:template match="definition">
		<h1>Definition</h1>
		<div class="definition-content">
			<table cellspacing="0">
				<tr>
					<td class="label">Name</td>
					<td>:</td>
					<td><xsl:value-of select="@name"/></td>
				</tr>
				<tr>
					<td class="label">JAR file</td>
					<td>:</td>
					<td><xsl:value-of select="@jarfile"/></td>
				</tr>
				<tr>
					<td class="label">Class</td>
					<td>:</td>
					<td><xsl:value-of select="@class"/></td>
				</tr>
				<tr>
					<td class="label">Version</td>
					<td>:</td>
					<td><xsl:value-of select="@version"/></td>
				</tr>
				<tr>
					<td class="label">Icon</td>
					<td>:</td>
					<td><xsl:value-of select="@icon"/></td>
				</tr>
				<tr>
					<td class="label">Description</td>
					<td>:</td>
					<td>
						<xsl:value-of select="@description"/>
						<xsl:if test="@description=''">
							No description
						</xsl:if>
					</td>
				</tr>
			</table>
		</div>
		<div class="definition-subcontent">
			<xsl:apply-templates select="node()"/>
		</div>
	</xsl:template>
	
	<xsl:template match="definition/aspect">
		<br/>
		<h2>Aspect</h2>
		<table cellspacing="0">
			<tr>
				<td class="label">Method selection</td>
				<td>:</td>
				<td><xsl:value-of select="@methodselection"/></td>
			</tr>
			<tr>
				<td class="label">Field selection</td>
				<td>:</td>
				<td><xsl:value-of select="@fieldselection"/>
				</td>
			</tr>
		</table>
	</xsl:template>

	<xsl:template match="definition/exportmethod">
		<xsl:if test="node()">
			<br/>
			<h2>Exported method</h2>
			<table>
				<xsl:for-each select="method">
					<tr>
						<td class="cell-entry">
							<div class="signature"><xsl:value-of select="@signature"/></div>
							<xsl:value-of select="@description"/>
						</td>
					</tr>
				</xsl:for-each>
			</table>
		</xsl:if>
	</xsl:template>

	<xsl:template match="definition/methodproxy">
		<xsl:if test="node()">
			<br/>
			<h2>Method proxies</h2>
			<table>
				<xsl:for-each select="field">
					<tr>
						<td class="cell-entry">
							<div class="signature"><xsl:value-of select="@signature"/></div>
							<xsl:if test="@mandatory='true'">[MANDATORY]<xsl:text> </xsl:text></xsl:if>
							<xsl:value-of select="@description"/>
						</td>
					</tr>
				</xsl:for-each>
			</table>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="definition/defaultpointcut">
		<xsl:if test="node()">
			<br/>
			<h2>Default pointcuts</h2>
			<table cellspacing="0">
				<xsl:for-each select="bean">
					<tr>
						<td class="bean"><xsl:value-of select="@name"/></td>
						<td><xsl:value-of select="@description"/></td>
					</tr>
				</xsl:for-each>
			</table>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="documentation">
		<xsl:for-each select="paragraph">
			<xsl:if test="@title">
				<h1><xsl:value-of select="@title"/></h1>
			</xsl:if>
			<p>
				<xsl:text><xsl:value-of select="text()"/></xsl:text>
			</p>
			<br/>
		</xsl:for-each>
	</xsl:template>

</xsl:stylesheet>
