/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console ;

import java.util.*;
import javax.servlet.http.HttpServletRequest;

import org.azuki.common.console.html.Table;
import org.azuki.common.console.html.Table.Row;
import org.azuki.common.console.http.HttpRequest;
import org.azuki.common.util.ListUtil;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class HtmlFactory
{
	final static public Table propertyList(String name, Map<?,?> propertyMap)
	{
		List<List> data = new ArrayList<List>() ;
		
		for(Map.Entry<?,?> ety : propertyMap.entrySet())
			data.add(ListUtil.toList(ety.getKey(), ":", ety.getValue())) ;
		
		Table.Header header = new Table.Header(name) ;
		header.setAttribute("colspan", "3") ;
		header.setAttribute("class", "propertyList") ;
		
		Table table = new Table(data) ;
		table.setHeaders(ListUtil.asList(header)) ;
		
		applyPropertyListCSS(table) ;
				
		return table ;
	}
	
	final static public Table sortedTable(HttpServletRequest req, List<List> data, String[] head)
	{
		return sortedTable(req, null, data, head) ;
	}
	
	final static public Table sortedTable(HttpServletRequest req, String tbid, List<List> data, String[] head)
	{
		return sortedTable(req, tbid, data, ListUtil.asList(head)) ;
	}
	
	final static public Table sortedTable(HttpServletRequest req, List<List> data, List<String> head)
	{
		return sortedTable(req, null, data, head) ;
	}
	
	final static public Table sortedTable(HttpServletRequest req, String tbid, List<List> data, List<String> head)
	{
		TableSortInfo tsfo = TableSortInfo.parse(req, tbid) ;
		
		Comparator<List> cmpr = new RowListComparator<List>(tsfo.idx) ;
		if(!tsfo.asc) cmpr = Collections.reverseOrder(cmpr) ;
		
		Collections.sort(data, cmpr) ;
		
		Table table = new Table(data) ;
		table.setHeaders(getTableHeaders(req, head, tsfo)) ;
		
		applyTableCSS(table) ;
		
		return table ;
	}
	
	final static public void applyPropertyListCSS(Table table)
	{
		table.setAttribute("class", "propertyList") ;
		
		boolean b = true ;
		for(Row row : table.getRows())
		{
			row.setAttribute("class", b?"blank":"grey") ;
			b = !b ;
			
			row.getCell(0).setAttribute("class", "propertyList") ;
			row.getCell(1).setAttribute("class", "propertyList") ;
		}
	}
	
	final static public void applyTableCSS(Table table)
	{
		table.setAttribute("class", "thin") ;
		
		boolean b = true ;
		for(Row row : table.getRows())
		{	row.setAttribute("class", b?"blank":"grey") ;
			b = !b ; }
	}
	
	final static private List<Table.Header> getTableHeaders(HttpServletRequest req, List<String> head, TableSortInfo tsfo)
	{
		List<Table.Header> list = new ArrayList<Table.Header>() ;
		
		for(int i=0 ; i<head.size() ; i++)
		{	
			Map<String,String> para = new HashMap<String,String>() ;
			para.putAll(HttpRequest.getParameterMap(req)) ;
			para.putAll(tsfo.asParameterMap(i)) ;
			
			StringBuilder bdr = new StringBuilder() ;
			//TODO: attention au '/' de l'URI !
			bdr.append("<a class=\"th\"") ;
			bdr.append(" href=\"").append(HttpRequest.mkURL(req, para, true)).append('"') ;
			bdr.append('>') ;
			bdr.append(head.get(i)) ;
			bdr.append("</a>") ;
			
			list.add(new Table.Header(bdr.toString())) ;
		}
		
		return list ;
	}
	
	final static class TableSortInfo
	{
		final static private String PARAM_TABLE_SORT_INFO = "aztsnfo" ;
		
		final public String key ;
		final public int idx ;
		final public boolean asc ;
		
		private TableSortInfo(String key, int idx, boolean asc)
		{
			this.key = key ;
			this.idx = idx ;
			this.asc = asc ;
		}
		
		final private int manage(int idx)
		{
			return idx==this.idx && asc ? -(idx+1) : idx+1 ;
		}
		
		final public Map<String,String> asParameterMap(int idx)
		{
			int nidx = manage(idx) ;
			return Collections.singletonMap(key, String.valueOf(nidx)) ;
		}
		
		@Override
		final public String toString()
		{
			return "TableSortInfo key("+key+") index("+idx+") asc("+asc+")" ;
		}
		
		final static public TableSortInfo parse(HttpServletRequest req, String tbid)
		{
			StringBuilder bdr = new StringBuilder(PARAM_TABLE_SORT_INFO) ;
			if(tbid!=null) bdr.append('.').append(tbid) ;
			
			String key = bdr.toString() ;
			String val = HttpRequest.get(req, key) ;
			
			if(val==null) return new TableSortInfo(key, 0, true) ;
			
			int itg = 0 ;
			try { itg = Integer.parseInt(val) ; }
			catch(Exception ex){}
			
			return new TableSortInfo(key, Math.abs(itg)-1, itg>=0) ;
		}
	}
	
	final static class RowListComparator<T extends List> implements Comparator<T>
	{
		final private int index ;
		
		public RowListComparator(int index)
		{
			this.index = index ;
		}
		
		@SuppressWarnings("unchecked")
		final public int compare(T l1, T l2)
		{
			Object o1 = index<l1.size() ? l1.get(index) : null ;
			Object o2 = index<l2.size() ? l2.get(index) : null ;
			
			if(o1==null) return o2==null ? 0 : -1 ;
			if(o2==null) return 1 ;
			
			return o1 instanceof Comparable
					? ((Comparable)o1).compareTo(o2)
					: o1.toString().compareTo(o2.toString()) ;
		}
	}
}