/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console ;

import java.io.*;
import java.util.logging.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class Resource
{
	final static public int defaultReadBufferSize = 4096 ; //4Ko
	
	final static private Logger logger = Logger.getLogger("org.azuki.common") ;
	
	/*
	 * Return the absolute resource name 
	 */
	final static public String absolute(Class cla, String res)
	{
		if(res.startsWith("/")) return res.substring(1) ;
		
		StringBuilder bdr = new StringBuilder() ;
		bdr.append(cla.getPackage().getName().replace('.', '/')) ;
		bdr.append('/').append(res) ;
		
		return bdr.toString() ;
	}
	
	/*
	 * Return the resource as InputStream
	 */ 
	final static public InputStream getInputStream(Class cla, String res)
	{
		return cla.getResourceAsStream(res) ;
	}	
	
	final static public InputStream getInputStream(ClassLoader cl, String res)
	{
		return cl.getResourceAsStream(res) ;
	}
	
	/*
	 * Return the content of the resource in a byte array 
	 */
	final static public byte[] getBytes(Class cla, String res)
	{
		String rnam = absolute(cla, res) ;
		return getBytes(cla.getClassLoader(), rnam) ;
	}
	
	final static public byte[] getBytes(ClassLoader cl, String res)
	{
		InputStream is = getInputStream(cl, res) ;
		if(is==null)
		{	logger.severe("Resource not found ("+res+")") ;
			return null ; }
		
		byte[] byts = null ;
		
		try
		{ byts = read(is) ; }
		catch(IOException ex)
		{ logger.log(Level.SEVERE, "Cannot read resource ("+res+")", ex) ; }
		finally
		{ try{ is.close() ; } catch(Exception ex){} }
		
		return byts ;
	}
	
	/*
	 * Read the stream content
	 */
	final static public byte[] read(InputStream is)
	throws IOException
	{
		return read(is, defaultReadBufferSize) ;
	}
	
	final static public byte[] read(InputStream is, int sz)
	throws IOException
	{
		if(sz<1) sz = defaultReadBufferSize ;
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream() ;
		byte[] buf = new byte[sz] ;
		
		while(true)
		{
			int read = is.read(buf) ;
			if(read==-1) break ;
			baos.write(buf, 0, read) ;
		}
		
		baos.flush() ;
		baos.close() ;
		
		return baos.toByteArray() ;
	}
}