/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console.html ;

import java.util.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class Element implements Content
{
	final private Set<Attribute> attrs = new HashSet<Attribute>() ;
	final private List<Content> contents = new ArrayList<Content>() ;
	
	final protected String name ;
	
	protected Object content ;
	
	public Element(String name)
	{
		this.name = name ;
	}
	
	/*
	 * Attributes management
	 */
	final public void setAttribute(Attribute attr)
	{
		attrs.add(attr) ;
	}
	
	final public void setAttribute(String name, String valu)
	{
		setAttribute(new Attribute(name, valu)) ;
	}
	
	final public Attribute getAttribute(String name)
	{
		for(Attribute attr : attrs)
			if(attr.getName().equals(name)) return attr ;
		
		return null ; 
	}
	
	final public String getAttributeValue(String name)
	{
		Attribute attr = getAttribute(name) ;
		return attr==null ? null : attr.getValue() ;
	}
	
	final public Collection<Attribute> getAttributes()
	{
		return Collections.unmodifiableCollection(attrs) ;
	}
	
	/*
	 * Contents management
	 */
	final public <T extends Content> void addContent(int index, T t)
	{
		contents.add(index, t) ;
	}
	
	final public <T extends Content> void addContent(T t)
	{
		contents.add(t) ;
	}
	
	final public void addContents(List<? extends Content> list)
	{
		contents.addAll(list) ;
	}
	
	final public void removeContent(int index)
	{
		contents.remove(index) ;
	}
	
	final public <T extends Content> void removeContent(T t)
	{
		contents.remove(t) ;
	}

	final public <T extends Content> void setContent(T t)
	{
		contents.clear() ;
		contents.add(t) ;
	}
	
	final public void removeContents(List<? extends Content> list)
	{
		contents.removeAll(list) ;
	}
	
	final public void clearContents()
	{
		contents.clear() ;
	}
	
	final public List<Content> getContents()
	{
		return Collections.unmodifiableList(contents) ;
	}
	
	final public <T extends Content> List<T> getContents(Class<T> t)
	{
		List<T> list = new ArrayList<T>() ;
		for(Content c : contents)
			if(t.isInstance(c)) list.add(t.cast(c)) ;
		return list ;
	}
	
	final public void setContents(List<? extends Content> list)
	{
		contents.clear() ;
		contents.addAll(list) ;
	}
	
	@Override
	public String toString()
	{
		StringBuilder bdr = new StringBuilder() ;
		
		bdr.append('<').append(name).append(' ') ;
		for(Attribute attr : attrs)
			bdr.append(attr).append(' ') ;
		
		if(contents.isEmpty())
		{	bdr.append("/>").append('\n') ;
			return bdr.toString() ; }
		
		bdr.append('>').append('\n') ;
		
		for(Content c : contents)
			bdr.append(c) ;
		
		bdr.append("</").append(name).append('>').append('\n') ;
		
		return bdr.toString() ;
	}
}