/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.console.html ;

import org.azuki.common.util.StringUtil;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class HtmlBuilder 
{
	final StringBuilder bdr  ;
	
	public HtmlBuilder()
	{
		this.bdr = new StringBuilder() ;
	}
	public HtmlBuilder(String s)
	{
		this.bdr = new StringBuilder(s) ;
	}	
	
	final public int length()
	{
		return bdr.length() ;
	}
	
	final public HtmlBuilder print(boolean b)
	{ bdr.append(b) ; return this ;}
	final public HtmlBuilder print(char c)
	{ bdr.append(c) ; return this ;}
	final public HtmlBuilder print(int i)
	{ bdr.append(i) ; return this ;}
	final public HtmlBuilder print(long d)
	{ bdr.append(d) ; return this ;}
	final public HtmlBuilder print(float f)
	{ bdr.append(f) ; return this ;}
	final public HtmlBuilder print(double d)
	{ bdr.append(d) ; return this ;}	
	final public HtmlBuilder print(char[] c)
	{ bdr.append(c) ; return this ;}
	
	final public HtmlBuilder print(Object o)
	{ bdr.append(o) ; return this ;}
	final public HtmlBuilder println(Object o)
	{ bdr.append(o).append('\n') ; return this ; }
	final public HtmlBuilder println()
	{ bdr.append('\n') ; return this ; }
	
	// remplace le caractere dans le buffer
	final public void replace(char old, char c)
	{
		int[] idxs = StringUtil.indexesOf(bdr.toString(), old) ;
		
		for(int idx : idxs)
			bdr.setCharAt(idx, c) ;
	}
	
	// remplace la chaine dans le buffer
	final public void replace(String old, String s)
	{
		int len = old.length() ;
		
		int[] idxs = StringUtil.indexesOf(bdr.toString(), old) ;
		
		for(int i=idxs.length-1 ; i>=0 ; i--)
		{	int idx = idxs[i] ;
			bdr.replace(idx, idx+len, s) ; }
	}
	
	final public void reset()
	{
		bdr.delete(0, bdr.length()) ;
	}
	
	/*
	 * Object
	 */
	@Override
	final public String toString()
	{
		return bdr.toString() ;
	}	
}