/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor ;

import java.util.*;

import org.jdom.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class EditorUtil
{
	final static public Map<String,String> formToMap(Element parent)
	{
		return childrenToMap(parent, TagCommon.ROW, AttrCommon.NAME, AttrCommon.VALU) ;
	}
	
	@SuppressWarnings("unchecked")
	final static public Map<String,String> childrenToMap(Element parent, String childName, String attrKey, String attrVal)
	{
		if(parent==null) return Collections.emptyMap() ;
		
		Map<String,String> map = new HashMap<String,String>() ;
		
		List<Element> lmens = parent.getChildren(childName) ;
		for(Element lmen : lmens)
		{
			String key = lmen.getAttributeValue(attrKey) ;
			if(key==null || key.isEmpty()) continue ;
			
			String val = lmen.getAttributeValue(attrVal) ;
			
			map.put(key, val) ;
		}
		
		return map ;
	}
	
	@SuppressWarnings("unchecked")
	final static public List<Map<String,String>> tableToList(Element parent, String childName)
	{
		if(parent==null) return Collections.emptyList() ;
		
		List<Map<String,String>> list = new ArrayList<Map<String,String>>() ;
		
		List<Element> lmens = parent.getChildren(childName) ;
		for(Element lmen : lmens)
			list.add(attrToMap(lmen)) ;
		
		return list ;
	}
	
	@SuppressWarnings("unchecked")
	final static public Map<String,String> attrToMap(Element lmen)
	{
		if(lmen==null) return Collections.emptyMap() ;
		
		Map<String,String> map = new HashMap<String,String>() ;
		
		List<Attribute> attrs = lmen.getAttributes() ; 
		for(Attribute attr : attrs)
			map.put(attr.getName(), attr.getValue()) ;
		
		return map ;
	}
}