/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.basic;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.azuki.common.editor.*;
import org.azuki.common.icon.CommonIcons;
import org.azuki.common.reflect.BeanInfo;
import org.azuki.common.util.Util;
import org.azuki.common.widget.*;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
abstract public class AbstractEditor<T extends IEditorContext> extends JPanel implements IEditor<T>
{
	final private ActionListener dialogActionAdapter = new DialogActionAdapter() ;
	
	private Element lmen ;
	
	protected T context ;
	
	protected Component parent ;
	
	protected String 	headerText = null ;
	protected String 	headerDesc = null ;
	protected Icon		headerIcon = null ;
	
	protected String dialogTitle = "Editor" ;
	
	protected boolean validated = false ;
	
	protected JHeaderPanel banner ;
	
	protected JButton okButton ;
	protected JButton cancelButton ;
	protected JPanel actions ;
	
	protected JDialog dialog ;
	
	public AbstractEditor()
	{
		super(new BorderLayout()) ;
		
		okButton = new JButton("OK") ;
		okButton.addActionListener(dialogActionAdapter) ;
		okButton.setIcon(new ImageIcon(getClass().getResource("/"+CommonIcons.APPLY_16)));
		
		cancelButton = new JButton("Cancel") ;
		cancelButton.addActionListener(dialogActionAdapter) ;
		cancelButton.setIcon(new ImageIcon(getClass().getResource("/"+CommonIcons.CANCEL_16)));
		
		actions = new JPanel(new FlowLayout()) ;
		actions.add(okButton) ;
		actions.add(cancelButton) ;
	}
	
	final public T getContext()
	{
		return context ;
	}
	
	final public Component getParentComponent()
	{
		return parent ;
	}
	
	final public void setHeaderText(String headerText)
	{
		this.headerText = headerText ;
	}
	
	final public void setHeaderDescription(String headerDesc)
	{
		this.headerDesc = headerDesc ;
	}
	
	final public void setHeaderIcon(Icon headerIcon)
	{
		this.headerIcon = headerIcon ;
	}
	
	final public void setDialogTitle(String dialogTitle)
	{
		this.dialogTitle = dialogTitle ;
	}
	
	// OVERRIDE
	protected void init()
	{
		this.lmen = context.getParameters() ;
		
		// initialize editor contents
		initParameters(lmen) ;
	}
	
	/*
	 * Abstract methods
	 */
	abstract public void initParameters(Element para) ;
	
	/*
	 * IBeanEditor
	 */
	final public Element getParameters()
	{
		return lmen ;
	}
	
	public boolean edit(Component parent, T context)
	{
		try
		{
			this.parent = parent ;
			this.context = context ;
			
			BeanInfo bnfo = context.getInfo() ;
			
			// default header values
			headerText = Util.oneString(bnfo.getName(), context.getLabel()) ;
			headerDesc = getClass().getName() ;
			headerIcon = bnfo.getIcon() ;
			
			// initialize editor
			init() ;
			
			// assembling banner
			banner = new JHeaderPanel(headerText, headerDesc, headerIcon) ;
		
			// building dialog content pane
			JPanel dialogContentPane = new JPanel(new BorderLayout()) ;
			dialogContentPane.add(actions, BorderLayout.SOUTH) ;
			dialogContentPane.add(this, BorderLayout.CENTER) ;
			dialogContentPane.add(banner, BorderLayout.NORTH) ;
			
			// show dialog
			dialog = new JDialog(JOptionPane.getFrameForComponent(parent)) ;
			dialog.setTitle(dialogTitle) ;
			dialog.setContentPane(dialogContentPane) ;
			dialog.setModal(true) ;
			
			dialog.getRootPane().setDefaultButton(okButton) ;
			
			dialog.pack() ;
	
			dialog.setLocationRelativeTo(parent) ;
			dialog.setVisible(true) ;
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
		
		return validated ;
	}
	
	/*
	 * Inner classes
	 */
	final class DialogActionAdapter implements ActionListener
	{
		final public void actionPerformed(ActionEvent e)
		{
			validated = e.getSource()==okButton ;
			if(validated) SwingToolkit.ensureEditingStopped(AbstractEditor.this) ;
			
			if(dialog!=null) dialog.setVisible(false) ;	
		}
	}
}