/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.util.*;
import javax.swing.ListModel;
import javax.swing.event.*;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JDOMAttributeListModel implements ListModel
{
	final private EventListenerList listenerList = new EventListenerList() ;
	
	final protected String lmenName ;
	final protected String attrName ;
	
	protected Element parent ;
	
	public JDOMAttributeListModel(Element parent, String lmenName, String attrName)
	{
		this.lmenName = lmenName ;
		this.attrName = attrName ;
		
		this.parent = parent ;
	}
	
	final public void setElement(Element parent)
	{
		this.parent = parent ;
		contentChanged() ;
	}
	
	@SuppressWarnings("unchecked")
	final public List<String> getNames()
	{
		ArrayList<String> list = new ArrayList<String>() ; 
		
		List<Element> topics = parent.getChildren(lmenName) ;
		for(Element topic : topics)
			list.add(topic.getAttributeValue(attrName,"")) ;
		
		return list ;
	}
	
	@SuppressWarnings("unchecked")
	final public boolean contains(String itemName)
	{
		List<Element> topics = parent.getChildren(lmenName) ;
		for(Element e : topics)
		{
			if(e.getAttributeValue(attrName,"").equals(itemName))
				return true ;
		}
		return false ;
	}
	
	final public void addItem(String itemName)
	{
		if(itemName==null) return ;
		if(contains(itemName)) return ;
			
		Element topic = new Element(lmenName) ;
		topic.setAttribute(attrName, itemName) ;
		
		parent.addContent(topic) ;
		
		itemAdded(parent.getChildren(lmenName).indexOf(topic)) ;
	}
	
	final public void removeItem(int index)
	{
		Element topic = (Element)parent.getChildren(lmenName).get(index) ;
		
		parent.removeContent(topic) ;
		
		itemRemoved(index) ;
	}
	
	final public Element getItem(int index)
	{
		return (Element)parent.getChildren(lmenName).get(index) ;
	}
	
	@SuppressWarnings("unchecked")
	final public Element getItem(String itemName)
	{
		List<Element> lmens = parent.getChildren(lmenName) ;
		
		for(Element lmen : lmens)
		{	if(lmen.getAttributeValue(attrName).equals(itemName))
			return lmen ; }
		
		return null ;
	}
	
	/*
	 * Events
	 */
	final protected void itemAdded(int index)
	{
		ListDataEvent event = new ListDataEvent(this, ListDataEvent.INTERVAL_ADDED, index, index) ;
		fireContentChanged(event) ;
	}
	
	final protected void itemRemoved(int index)
	{
		ListDataEvent event = new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, index, index) ;
		fireContentChanged(event) ;
	}
	
	final protected void contentChanged()
	{
		ListDataEvent event = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, -1, -1) ;
		fireContentChanged(event) ;
	}
	
	final private void fireContentChanged(ListDataEvent event)
	{
		for(ListDataListener l : listenerList.getListeners(ListDataListener.class))
			l.contentsChanged(event) ;
	}
	
	/*
	 * ListModel
	 */
	final public int getSize()
	{
		return parent.getChildren(lmenName).size() ;
	}
	
	final public String getElementAt(int index)
	{
		Element topc = (Element)parent.getChildren(lmenName).get(index) ;
		return topc.getAttributeValue(attrName) ;
	}	

	final public void addListDataListener(ListDataListener l)
	{
		listenerList.add(ListDataListener.class, l) ;
	}
	
	final public void removeListDataListener(ListDataListener l)
	{
		listenerList.remove(ListDataListener.class, l) ;
	}		
}