/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.util.List;

import javax.swing.event.*;
import javax.swing.table.TableModel;

import org.jdom.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JDOMAttributeTableModel implements TableModel
{
	final protected EventListenerList listenerList = new EventListenerList() ;
	
	protected Element lmen ;
	
	private List<Attribute> attributeList ;
	
	@SuppressWarnings("unchecked")
	public JDOMAttributeTableModel(Element lmen)
	{
		this.lmen = lmen ;
		this.attributeList = lmen.getAttributes() ;
	}
	
	@SuppressWarnings("unchecked")
	final public void setElement(Element lmen)
	{
		this.lmen = lmen ;
		this.attributeList = lmen.getAttributes() ;
		
		modelChanged() ;
	}
	
	@SuppressWarnings("unchecked")
	final public void addRow(String attrName, String attrValue)
	{
		if(lmen.getAttribute(attrName)!=null)
		{	modelChanged() ;
			return ; }
		
		lmen.setAttribute(attrName, attrValue) ;
		attributeList = lmen.getAttributes() ;
		
		rowInserted(attributeList.size()) ;
	}
	
	final public void removeRow(int rowIndex)
	{
		Attribute attr = attributeList.remove(rowIndex) ;
		lmen.removeAttribute(attr) ;
		
		rowRemoved(rowIndex) ;
	}
	
	/*
	 * Event handling
	 */
	final protected void cellChanged(int rowIndex, int columnIndex)
	{
		TableModelEvent event = new TableModelEvent(this, rowIndex, rowIndex, columnIndex) ;
		fireModelChanged(event) ;
	}
	
	final protected void rowChanged(int rowIndex)
	{
		TableModelEvent event = new TableModelEvent(this, rowIndex) ;
		fireModelChanged(event) ;
	}
	
	final protected void rowInserted(int rowIndex)
	{
		TableModelEvent event = new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT) ;
		fireModelChanged(event) ;
	}
	
	final protected void rowRemoved(int rowIndex)
	{
		TableModelEvent event = new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE) ;
		fireModelChanged(event) ;
	}
	
	final protected void modelChanged()
	{
		TableModelEvent event = new TableModelEvent(this) ;
		fireModelChanged(event) ;
	}
	
	final private void fireModelChanged(TableModelEvent event)
	{
		TableModelListener[] listeners = listenerList.getListeners(TableModelListener.class) ;
		for(TableModelListener tml : listeners)
			tml.tableChanged(event) ;
	}
	
	/*
	 * TableModel
	 */
	final public int getColumnCount()
	{
		return 2 ;
	}
	
	final public String getColumnName(int columnIndex)
	{
		return columnIndex==0 ? "Attribute" : "Value" ;
	}

	final public Class<?> getColumnClass(int columnIndex)
	{
		return String.class ;
	}
	
	final public int getRowCount()
	{
		return attributeList.size() ;
	}

	final public Object getValueAt(int rowIndex, int columnIndex)
	{
		Attribute attr = attributeList.get(rowIndex) ;
		return columnIndex==0 ? attr.getName() : attr.getValue() ;
	}

	final public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return true;
	}
	
	final public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		Attribute attr = attributeList.get(rowIndex) ;
		if(columnIndex==0)	attr.setName(String.valueOf(aValue)) ;
		else				attr.setValue(String.valueOf(aValue)) ;
		
		cellChanged(rowIndex, columnIndex) ;
	}

	final public void addTableModelListener(TableModelListener l)
	{
		listenerList.add(TableModelListener.class, l) ;
	}
	
	final public void removeTableModelListener(TableModelListener l)
	{
		listenerList.remove(TableModelListener.class, l) ;
	}
}