/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.azuki.common.widget.field.JComboField;
import org.jdom.Attribute;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMComboField extends JComboField
{
	final protected ItemListener itemAdapter = new ItemAdapter() ; 
	
	protected Attribute attr ;
	
	public JDOMComboField()
	{
		this(new DefaultComboBoxModel()) ;
	}
	
	public JDOMComboField(Object... items)
	{
		this(new DefaultComboBoxModel(items)) ;
	}
	
	public JDOMComboField(Vector<?> items)
	{
		this(new DefaultComboBoxModel(items)) ;
	}
	
	public JDOMComboField(Collection<?> items)
	{
		this(new DefaultComboBoxModel(items.toArray())) ;
	}
	
	public JDOMComboField(ComboBoxModel model)
	{
		super(model) ;
		
		addItemListener(itemAdapter) ;
	}
	
	@Override
	final public void setSelectedItem(Object obj)
	{
//		int idx = modelIndexOf(obj) ;
//		super.setSelectedItem(idx==-1 && !isEditable ? null : dataModel.getElementAt(idx)) ;		
		dataModel.setSelectedItem(obj) ;
	}
	
//	final private int modelIndexOf(Object obj)
//	{
//		for(int i=0 ; i<dataModel.getSize() ; i++)
//		{
//			String s1 = String.valueOf(obj) ;
//			String s2 = String.valueOf(dataModel.getElementAt(i)) ;
//			if(s1.equals(s2)) return i ;
//		}
//		
//		return -1 ;
//	}
	
	final public void setAttribute(Attribute attr)
	{
		this.attr = attr ;
		
		removeItemListener(itemAdapter) ;
		setSelectedItem(attr.getValue()) ;
		addItemListener(itemAdapter) ;
	}
	
	/*
	 * Default ItemListener
	 */
	final class ItemAdapter implements ItemListener
	{
		final public void itemStateChanged(ItemEvent e)
		{
			if(e.getStateChange()==ItemEvent.SELECTED)
				attr.setValue(String.valueOf(e.getItem())) ;
		}
	} 
}