/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.util.*;
import javax.swing.*;

import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMElementList extends JList
{
	final public List<String> childAttrs ;
	
	public JDOMElementList(Element lmen, String childName, List<String> childAttrs)
	{
		this(new JDOMElementListModel(lmen, childName), childAttrs) ;
	}
	
	public JDOMElementList(JDOMElementListModel model, List<String> childAttrs)
	{
		super(model) ;
		
		this.childAttrs = childAttrs ;
	}
	
	final public List<String> getChildAttributes()
	{
		return Collections.unmodifiableList(childAttrs) ;
	}
	
	final public void addElement()
	{
		JDOMElementListModel model = getModel() ;
		
		Element lmen = new Element(model.getChildName()) ;
		for(String attr : childAttrs)
			lmen.setAttribute(attr, "") ;
		
		getModel().addElement(lmen) ;
	}
	
	final public void removeElement(int index)
	{
		getModel().removeElement(index) ;
	}
	
	final public void setModel(JDOMElementListModel model)
	{
		super.setModel(model) ;
	}
	
	@Override
	final public JDOMElementListModel getModel()
	{
		return (JDOMElementListModel)super.getModel() ;
	}
	
	@Override
	final public void setModel(ListModel model)
	{
		if(!JDOMElementListModel.class.isInstance(model))
			throw new IllegalArgumentException("Not a valid model.") ;
		
		super.setModel(model) ;
	}
}