/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

import org.azuki.common.icon.CommonIcons;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMElementListPanel extends JPanel
{
	final private ActionAdapter defaultActionAdapter = new ActionAdapter() ;
	
	final protected JButton addButton ;
	final protected JButton delButton ;
	final protected JToolBar listActions ;
	
	final protected JDOMElementList list ;
	final protected JScrollPane listScroll ;
	
	public JDOMElementListPanel(Element lmen, String childName, List<String> childAttrs)
	{
		// toolbar
		addButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_ADD_16))) ;
		addButton.addActionListener(defaultActionAdapter) ;
		
		delButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_REMOVE_16))) ;
		delButton.addActionListener(defaultActionAdapter) ;
		
		listActions = new JToolBar(SwingConstants.VERTICAL) ;
		listActions.setFloatable(false) ;
		listActions.add(addButton) ;
		listActions.add(delButton) ;
		
		// list
		list = new JDOMElementList(lmen, childName, childAttrs) ;
		listScroll = new JScrollPane(list) ;
		
		// main
		setLayout(new BorderLayout()) ;
		add(listScroll, BorderLayout.CENTER) ;
		add(listActions, BorderLayout.EAST) ;
	}
	
	final public JDOMElementList getList()
	{
		return list ;
	}
	
	final class ActionAdapter implements ActionListener
	{
		final public void actionPerformed(ActionEvent e)
		{
			Object src = e.getSource() ; 
			if(src==addButton)
			{
				list.addElement() ;
			}
			else if(src==delButton)
			{
				int[] idxs = list.getSelectedIndices() ;
				Arrays.sort(idxs) ;
				
				for(int i=idxs.length-1 ; i>=0 ; i--)
					list.removeElement(idxs[i]) ;
				
				//FIXME: la selection ne se mets pas a jour ... bizarre !!
				list.getSelectionModel().clearSelection() ;
			}
		}
	}
}