/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.util.*;

import javax.swing.JTable;
import javax.swing.table.TableModel;

import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMElementTable extends JTable
{
	protected List<String> tableHeaderValues = Collections.emptyList() ;
	
	public JDOMElementTable(Element lmen, String childName, List<String> childAttrs)
	{
		this(lmen, childName, childAttrs, childAttrs) ;
	}
	
	public JDOMElementTable(Element lmen, String childName, List<String> childAttrs, List<String> childHeads)
	{
		this(new JDOMElementTableModel(lmen, childName, childAttrs), childHeads) ;
	}
	
	public JDOMElementTable(JDOMElementTableModel model)
	{
		this(model, model.getChildAttributeNames()) ;
	}
	
	public JDOMElementTable(JDOMElementTableModel model, List<String> childHeads)
	{
		super(model) ;
		
		installTableHeaders(childHeads) ;
	}
	
	final public void installTableHeaders(List<String> tableHeaderValues)
	{
		this.tableHeaderValues = tableHeaderValues ;
		
		int ccnt = columnModel.getColumnCount() ;
		for(int i=0 ; i<tableHeaderValues.size() ; i++)
		{
			if(i>=ccnt) break ;
			columnModel.getColumn(i).setHeaderValue(tableHeaderValues.get(i)) ;
		}
	}
	
	final public void addRow()
	{
		getModel().newElement() ;
	}
	
	final public void removeRow(int index)
	{
		JDOMElementTableModel modl = getModel() ;
		modl.removeElement(index) ;
		if(modl.getRowCount()>index) addRowSelectionInterval(index, index) ;
	}
	
	final public void removeSelectedRow()
	{
		int selRow = getSelectedRow() ;
		if(selRow==-1) return ;
		
		removeRow(selRow) ; 
	}
	
	final public void removeSelectedRows()
	{
		int[] idxs = getSelectedRows() ;
		Arrays.sort(idxs) ;
		
		for(int i=idxs.length-1 ; i>=0 ; i--) // inverting order !!
			removeRow(idxs[i]) ;
	}
	
	final public void setModel(JDOMElementTableModel model)
	{
		super.setModel(model) ;
	}
	
	@Override
	final public JDOMElementTableModel getModel()
	{
		return (JDOMElementTableModel)super.getModel() ;
	}
	
	@Override
	final public void setModel(TableModel model)
	{
		if(!JDOMElementTableModel.class.isInstance(model))
			throw new IllegalArgumentException("Not a valid model: "+model) ;
		
		super.setModel(model) ;
	}
	
	@Override
	final protected TableModel createDefaultDataModel()
	{
		return new JDOMElementTableModel() ;
	}
}