/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom ;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;

import javax.swing.*;

import org.azuki.common.icon.CommonIcons;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMElementTablePanel extends JPanel
{
	final private ActionAdapter defaultActionAdapter = new ActionAdapter() ;
	
	final protected JButton addButton ;
	final protected JButton delButton ;
	final protected JToolBar tableActions ;
	
	final protected JDOMElementTable table ;
	final protected JScrollPane tableScroll ;
	
	public JDOMElementTablePanel(Element lmen, JDOMElementTableBinding tableBinding)
	{
		this(lmen, tableBinding.getChildName(), tableBinding.getAttributeNames(), tableBinding.getAttributeHeaders(), tableBinding.getDefaultValues()) ;
	}
	
	public JDOMElementTablePanel(Element lmen, String childName, List<String> childAttrs, List<String> childHeads)
	{
		this(lmen, childName, childAttrs, childHeads, null) ;
	}
	
	public JDOMElementTablePanel(Element lmen, String childName, List<String> childAttrs, List<String> childHeads, Map<String,String> defValus)
	{
		// toolbar
		addButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_ADD_16))) ;
		addButton.addActionListener(defaultActionAdapter) ;
		
		delButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_REMOVE_16))) ;
		delButton.addActionListener(defaultActionAdapter) ;
		
		tableActions = new JToolBar(SwingConstants.VERTICAL) ;
		tableActions.setFloatable(false) ;
		tableActions.add(addButton) ;
		tableActions.add(delButton) ;
		
		// table
		table = new JDOMElementTable(new JDOMElementTableModel(lmen, childName, childAttrs, defValus), childHeads) ;
		
		tableScroll = new JScrollPane(table) ;
		tableScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS) ;
	
		// main
		setLayout(new BorderLayout()) ;
		add(tableScroll, BorderLayout.CENTER) ;
		add(tableActions, BorderLayout.EAST) ;
	}
	
	final public JDOMElementTable getTable()
	{
		return table ;
	}
	
	final class ActionAdapter implements ActionListener
	{
		final public void actionPerformed(ActionEvent e)
		{
			Object src = e.getSource() ; 
			if(src.equals(addButton)) 		table.addRow() ;
			else if(src.equals(delButton))	table.removeSelectedRows() ;
		}
	}
}