/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom;

import javax.swing.*;
import javax.swing.event.*;

import org.jdom.Attribute;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMNumberField extends JSpinner
{
	final protected ChangeListener defaultChangeAdapter = new ChangeAdapter() ;
	
	final private int def ;
	
	protected Attribute attr ;
	
	// CONSTRUCTEUR
	public JDOMNumberField(int min, int max, int step)
	{
		this(min, max, step, min) ; 
	}
	public JDOMNumberField(int min, int max, int step, int def)
	{
		super(new SpinnerNumberModel(min, min, max, step)) ;
		
		this.def = def ;
	}

	final public int getDefaultValue()
	{
		return def ;
	}
	
	final public void setNumberFormat(String decimalFormat)
	{
		setEditor(new JSpinner.NumberEditor(this, decimalFormat)) ;
	} 
	
	final public void setAttribute(Attribute attr)
	{
		this.attr = attr ;
		
		removeChangeListener(defaultChangeAdapter) ;
		
		String attrValue = attr.getValue() ; 
		
		try
		{ setValue(Integer.valueOf(attrValue)) ; }
		catch(Exception e)
		{ setValue(new Integer(def)) ; }
		
		addChangeListener(defaultChangeAdapter) ;
	}
	
	@Override
	final public void setValue(Object value)
	{
		attr.setValue(value==null?String.valueOf(def):value.toString()) ;
		super.setValue(value) ;
	}
		
	/*
	 * Default ChangeListener
	 */
	final class ChangeAdapter implements ChangeListener
	{
		final public void stateChanged(ChangeEvent e)
		{
			attr.setValue(getValue().toString()) ;
		}
	}
}