/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.jdom;

import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.jdom.Attribute;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDOMRadioGroup extends JComponent
{
	final protected ActionListener defaultActionAdapter = new ActionAdapter() ;
	
	final protected String def ;
	
	final private Map<String,ButtonModel> buttonModelMap = new HashMap<String,ButtonModel>() ; 
	final private ButtonGroup buttonGroup ;
	
	protected Attribute attr ;
	
	// Contructors parameters
	//--> 	String[][] { {RADIO_VALUE,RADIO_LABEL}, ... }
	//--> 	Map<RADIO_VALUE,RADIO_LABEL>
	
	// CONSTRUCTEUR
	public JDOMRadioGroup(Map<String,String> items)
	{
		this(items, null) ;
	}
	
	public JDOMRadioGroup(Map<String,String> items, String def)
	{
		this(items, def, true) ;
	}
	
	public JDOMRadioGroup(Map<String,String> items, String def, boolean horizontal)
	{
		this(mapToArray(items), def, horizontal) ;
	}
	
	public JDOMRadioGroup(String[][] items)
	{
		this(items, null) ;
	}
	
	public JDOMRadioGroup(String[][] items, String def)
	{
		this(items, def, true) ;
	}
	
	public JDOMRadioGroup(String[][] items, String def, boolean horizontal)
	{
		this.def = def ;
		
		buttonGroup = new ButtonGroup() ;
		
		setLayout(new BoxLayout(this, horizontal ? BoxLayout.LINE_AXIS : BoxLayout.PAGE_AXIS)) ;
		
		for(String[]row : items)
		{
			if(row==null || row.length==0) continue ;
			
			String val = row[0] ;
			String lbl = row.length>=2 ? row[1] : val 
					;
			JRadioButton rad = new JRadioButton(lbl) ;
			rad.setActionCommand(val) ;
			
			rad.addActionListener(defaultActionAdapter) ;
			
			buttonModelMap.put(val, rad.getModel()) ;
			buttonGroup.add(rad) ;
			
			add(rad) ;
		}
	}
	
	final public void setAttribute(Attribute attr)
	{
		this.attr = attr ;
		
		ButtonModel buttonModel = buttonModelMap.get(attr.getValue()) ;
		if(buttonModel!=null) buttonGroup.setSelected(buttonModel, true) ;
	}
	
	/*
	 * JComponent
	 */
	@Override
	final public void setEnabled(boolean b)
	{
		super.setEnabled(b) ;
		
		Enumeration<AbstractButton> enu = buttonGroup.getElements() ;
		while(enu.hasMoreElements()) enu.nextElement().setEnabled(b) ;
	}

	/*
	 * Default ActionListener
	 */
	final class ActionAdapter implements ActionListener
	{
		final public void actionPerformed(ActionEvent e)
		{
			attr.setValue(e.getActionCommand()) ;
		}
	}
	
	/**
	 * STATIC
	 */
	final static public String[][] mapToArray(Map<String,String> map)
	{
		String[][] arr = new String[map.size()][2] ;
		
		int idx = 0 ;
		
		for(Map.Entry<String,String> ety : map.entrySet())
		{
			arr[idx][0] = ety.getKey() ;
			arr[idx][1] = ety.getValue() ;
			
			idx++ ;
		}
		
		return arr ;
	}
}