/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.page ;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.azuki.common.editor.IEditorContext;
import org.azuki.common.editor.jdom.*;
import org.azuki.common.icon.CommonIcons;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class ElementPage extends Page
{
	final private ActionListener defaultActionAdapter = new ActionAdapter() ;
	final private TreeSelectionListener defaultTreeSelectionAdapter = new TreeSelectionAdapter() ;
	
	JDOMElementTree jdomTree ;
	JScrollPane jdomTreeScroll ;
	JPanel jdomTreePane ;
	
	JButton insertNodeButton ;
	JButton removeNodeButton ;
	JToolBar jdomTreeActions ;
	
	JDOMAttributeTable jdomTable ;
	JScrollPane jdomTableScroll ;
	JPanel jdomTablePane ;
	
	JButton insertRowButton ;
	JButton removeRowButton ;
	JToolBar jdomTableActions ;
	
	JSplitPane splitPane ;
	
	/*
	 * Page
	 */
	@Override
	public void init(IEditorContext ectx, Element lmen)
	{
		// Tree
		insertNodeButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_ADD_16))) ;
		insertNodeButton.addActionListener(defaultActionAdapter) ;
		
		removeNodeButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_REMOVE_16))) ;
		removeNodeButton.addActionListener(defaultActionAdapter) ;
		
		jdomTreeActions = new JToolBar(SwingConstants.HORIZONTAL) ;
		jdomTreeActions.setFloatable(false) ;
		jdomTreeActions.add(insertNodeButton) ;
		jdomTreeActions.add(removeNodeButton) ;
		
		jdomTree = new JDOMElementTree(lmen) ;
		jdomTree.setEditable(true) ;
		jdomTree.addTreeSelectionListener(defaultTreeSelectionAdapter) ;
		
		jdomTreeScroll = new JScrollPane(jdomTree) ;
		jdomTreeScroll.setPreferredSize(new Dimension(200, 200)) ;
		
		jdomTreePane = new JPanel(new BorderLayout()) ;
		jdomTreePane.add(jdomTreeScroll, BorderLayout.CENTER) ;
		jdomTreePane.add(jdomTreeActions, BorderLayout.SOUTH) ;
		
		// Table
		insertRowButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_ADD_16))) ;
		insertRowButton.addActionListener(defaultActionAdapter) ;
		
		insertRowButton.setEnabled(false) ;
		
		removeRowButton = new JButton(new ImageIcon(getClass().getResource("/"+CommonIcons.LIST_REMOVE_16))) ;
		removeRowButton.addActionListener(defaultActionAdapter) ;
		
		removeRowButton.setEnabled(false) ;
		
		jdomTableActions = new JToolBar(SwingConstants.VERTICAL) ;
		jdomTableActions.setFloatable(false) ;
		jdomTableActions.add(insertRowButton) ;
		jdomTableActions.add(removeRowButton) ;
		
		jdomTable = new JDOMAttributeTable() ;
		
		jdomTableScroll = new JScrollPane(jdomTable) ;
		jdomTableScroll.setPreferredSize(new Dimension(300, 200)) ;
		
		jdomTablePane = new JPanel(new BorderLayout()) ;
		jdomTablePane.add(jdomTableScroll, BorderLayout.CENTER) ;
		jdomTablePane.add(jdomTableActions, BorderLayout.EAST) ;
		
		// General
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT) ;
		splitPane.setBorder(BorderFactory.createEmptyBorder()) ;
		splitPane.setLeftComponent(jdomTreePane) ;
		splitPane.setRightComponent(jdomTablePane) ;
		
		setLayout(new BorderLayout()) ;
		add(splitPane, BorderLayout.CENTER) ;
	}	
	

	/*
	 * ActionListener
	 */
	final private class ActionAdapter implements ActionListener
	{
		final public void actionPerformed(ActionEvent e)
		{
			Object source = e.getSource() ;
			
			if(source==insertNodeButton)
			{
				TreePath treePath = jdomTree.getSelectionPath() ;
				if(treePath==null) return ;
				
				String name = JOptionPane.showInputDialog("Enter new element name: ") ;
				if(name!=null)
				{
					JDOMElementTreeNode parentNode = (JDOMElementTreeNode)treePath.getLastPathComponent() ;
					JDOMElementTreeNode newNode = new JDOMElementTreeNode(parentNode, new Element(name)) ;
					jdomTree.getModel().insertNodeInto(newNode, parentNode, parentNode.getChildCount()) ;
				}
			}
			else if(source==removeNodeButton)
			{
				TreePath[] treePaths = jdomTree.getSelectionPaths() ;
				if(treePaths.length==0) return ;
				
				for(TreePath treePath : treePaths)
				{
					MutableTreeNode mtn = (MutableTreeNode)treePath.getLastPathComponent() ;
					if(mtn.getParent()!=null) // is ROOT ?
						jdomTree.getModel().removeNodeFromParent(mtn) ;
				}
			}
			else if(source==insertRowButton)
			{
				jdomTable.addRow() ;
			}
			else if(source==removeRowButton)
			{
				jdomTable.removeSelectedRows() ;
			}
		}
	}
	
	/*
	 * TreeSelectionListener
	 */
	final private class TreeSelectionAdapter implements TreeSelectionListener
	{
		final public void valueChanged(TreeSelectionEvent e)
		{
			insertRowButton.setEnabled(false) ;
			removeRowButton.setEnabled(false) ;
			
			if(!e.isAddedPath())
			{	jdomTableScroll.setViewportView(null) ;
				return ; }
			
			JDOMElementTreeNode jtn = (JDOMElementTreeNode)e.getPath().getLastPathComponent() ;
			boolean root = jtn.getParent()==null ;
			
			removeNodeButton.setEnabled(!root) ;
			
			if(root)
			{	jdomTableScroll.setViewportView(null) ;
				return ; }
			
			insertRowButton.setEnabled(true) ;
			removeRowButton.setEnabled(true) ;
			
			jdomTable.setElement(jtn.getElement()) ;
			jdomTableScroll.setViewportView(jdomTable) ;
		}
	}
}