/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.pointcut ;

import java.awt.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;

import org.azuki.common.consts.*;
import org.azuki.common.editor.page.*;
import org.azuki.common.editor.pointcut.imp.*;
import org.azuki.common.editor.pointcut.imp.renderer.*;
import org.azuki.common.reflect.ReflectUtil;
import org.azuki.common.util.*;
import org.azuki.common.widget.JView;
import org.azuki.common.widget.jdom.*;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class DefaultPointcutEditor extends AbstractPointcutEditor
{
	protected Page paraPage = new ElementPage() ;
	
	protected Class<? extends Page> fildPageClass = ElementPage.class ;
	protected Class<? extends Page> methPageClass = ElementPage.class ;
	
	final public void setParameterPage(Page paraPage)
	{
		this.paraPage = paraPage ;
	}
	
	final public void setFieldPageClass(Class<? extends Page> fildPageClass)
	{
		this.fildPageClass = fildPageClass ;
	}
	
	final public void setMethodPageClass(Class<? extends Page> methPageClass)
	{
		this.methPageClass = methPageClass ;
	}
	
	JTabbedPane tabs ;
	
	protected FieldChooser  fildChooser ;
	protected MethodChooser methChooser ;
	
	JView fildView ;
	JView methView ;
	
	final private JDOMElementFilter listFilter = new JDOMElementFilter(Attr.NAME) ;
	final private ListCellRenderer listFilterRenderer = new JDOMElementListCellRenderer(Attr.NAME, "(undefined)") ;
	
	public DefaultPointcutEditor()
	{
		tabs = new JTabbedPane() ;
		
		setLayout(new BorderLayout()) ;
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		add(tabs, BorderLayout.CENTER) ;
		
		setPreferredSize(new Dimension(720, 384)) ;
	}
	
	final private boolean isSelectionValid()
	{
		boolean fildOK = fildChooser==null ? true : fildChooser.isSelectionValid() ;
		boolean methOK = methChooser==null ? true : methChooser.isSelectionValid() ;
		return fildOK && methOK ;
	}
	
	/*
	 * AbstractPointcutEditor
	 */
	@Override
	public void initParameters(Element lmen)
	{
		if(paraPage==null) return ;
		
		paraPage.init(context, lmen) ;
			
		tabs.addTab("Parameters", paraPage) ;
	}
	
	@Override
	public void initFields(List<Element> flds)
	{
		Collection<Element> fsel = context.getSelectedFields() ;
		Collection<String> curr = JDOMUtil.listAttributeValues(fsel, Attr.NAME, "(none)") ;
		
		Set<String> names = new TreeSet<String>() ;
		names.addAll(ReflectUtil.stringList(context.getFields())) ;
		names.removeAll(curr) ;
		
		Set<Element> lmens = new TreeSet<Element>(new JDOMElementComparator(Attr.NAME)) ;
		lmens.addAll(fsel) ;
		for(String name : names)
		{
			Element temp = new Element(Tag.FIELD) ;
			temp.setAttribute(Attr.NAME, name) ;
			
			lmens.add(temp) ;
		}
		
		fildChooser = new FieldChooser(lmens, context.getFields()) ;
		fildChooser.setFieldCellRenderer(new FieldDeprecatedCellRenderer(context.getFields())) ;
		fildChooser.setSelected(fsel, Boolean.TRUE) ;
		fildChooser.setItemSelectionMode(SelectionUtil.toItemSelection(context.getFieldSelection())) ;
		
		if(fildPageClass==null)
		{
			tabs.addTab("Fields", fildChooser) ;
		}
		else
		{
			FieldSelectionAdapter fieldSelectionAdapter = new FieldSelectionAdapter() ;
			
			fildChooser.setFilter(listFilter) ;
			fildChooser.setFilterCellRenderer(listFilterRenderer) ;
			fildChooser.addItemSelectionListener(fieldSelectionAdapter) ;
			fildChooser.addListSelectionListener(fieldSelectionAdapter) ;
			
			fildView = new JView() ;
			
			JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, fildChooser, fildView) ; 
			split.setBorder(BorderFactory.createEmptyBorder()) ;
			
			tabs.addTab("Fields", split) ;
		}
	}
	
	@Override
	public void initMethods(List<Element> mths)
	{
		Collection<Element> msel = context.getSelectedMethods() ;
		List<String> curr = JDOMUtil.listAttributeValues(msel, Attr.NAME, "(none)") ;
		
		Set<String> names = new TreeSet<String>() ;
		names.addAll(ReflectUtil.stringList(context.getMethods())) ;
		names.removeAll(curr) ;
		
		Set<Element> lmens = new TreeSet<Element>(new JDOMElementComparator(Attr.NAME)) ;
		lmens.addAll(msel) ;
		for(String name : names)
		{
			Element temp = new Element(Tag.METHOD) ;
			temp.setAttribute(Attr.NAME, name) ;
			
			lmens.add(temp) ;
		}
		
		methChooser = new MethodChooser(lmens, context.getMethods()) ;
		methChooser.setMethodCellRenderer(new MethodDeprecatedCellRenderer(context.getMethods())) ;
		methChooser.setSelected(msel, Boolean.TRUE) ;
		methChooser.setItemSelectionMode(SelectionUtil.toItemSelection(context.getMethodSelection())) ;
		
		if(methPageClass==null)
		{
			tabs.addTab("Methods", methChooser) ;
		}
		else
		{
			MethodSelectionAdapter methodSelectionAdapter = new MethodSelectionAdapter() ;
			
			methChooser.setFilter(listFilter) ;
			methChooser.setFilterCellRenderer(listFilterRenderer) ;
			methChooser.addItemSelectionListener(methodSelectionAdapter) ;
			methChooser.addListSelectionListener(methodSelectionAdapter) ;
			
			methView = new JView() ;
			
			JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, methChooser, methView) ;
			split.setBorder(BorderFactory.createEmptyBorder()) ;
			
			tabs.addTab("Methods", split) ;
		}
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public Collection<Element> getFields()
	{
		return fildChooser!=null ? fildChooser.getSelectedItems() : Collections.EMPTY_LIST ;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public Collection<Element> getMethods()
	{
		return methChooser!=null ? methChooser.getSelectedItems() : Collections.EMPTY_LIST ;
	}
	
	/*
	 * Field selection adapter
	 */
	final class FieldSelectionAdapter implements TableModelListener, ListSelectionListener
	{
		final private void manage(int index)
		{
			if(!fildChooser.isSelected(index))
			{	fildView.setComponent(null) ;
				return ; }
			
			if(fildPageClass==null)
			{	fildView.setComponent(null) ;
				return ; }
			
			Element lmen = fildChooser.getItem(index) ;
			Element para = JDOMUtil.getElement(lmen, Tag.PARA) ;
			
			try
			{
				Page page = fildPageClass.newInstance() ;
				page.init(getContext(), para) ;
				
				fildView.setComponent(page) ;
			}
			catch(Exception ex)
			{
				ex.printStackTrace() ; //FIXME
			}
		}
		
		// TableModelListener
		public void tableChanged(TableModelEvent e)
		{
			manage(e.getLastRow()) ;
			
			if(okButton!=null) okButton.setEnabled(isSelectionValid()) ;
		}
		
		// ListSelectionListener
		public void valueChanged(ListSelectionEvent e)
		{
			if(e.getValueIsAdjusting()) return ;
			manage(fildChooser.getTable().getSelectedRow()) ;
		}
	}
	
	/*
	 * Method selection adapter
	 */
	final class MethodSelectionAdapter implements TableModelListener, ListSelectionListener
	{
		final private void manage(int index)
		{
			if(!methChooser.isSelected(index))
			{	methView.setComponent(null) ;
				return ; }
			
			if(methPageClass==null)
			{	methView.setComponent(null) ;
				return ; }
			
			Element lmen = methChooser.getItem(index) ;
			Element para = JDOMUtil.getElement(lmen, Tag.PARA) ;
			
			try
			{
				Page page = methPageClass.newInstance() ;
				page.init(getContext(), para) ;
				
				methView.setComponent(page) ;
			}
			catch(Exception ex)
			{
				ex.printStackTrace() ; //FIXME
			}
		}
		
		// TableModelListener
		public void tableChanged(TableModelEvent e)
		{
			manage(e.getLastRow()) ;
			
			if(okButton!=null) okButton.setEnabled(isSelectionValid()) ;
		}
		
		// ListSelectionListener
		public void valueChanged(ListSelectionEvent e)
		{
			if(e.getValueIsAdjusting()) return ;
			manage(methChooser.getTable().getSelectedRow()) ;
		}
	}
}