/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.editor.pointcut.imp ;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.lang.reflect.Method;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.TableCellRenderer;

import org.azuki.common.consts.Attr;
import org.azuki.common.reflect.Annotations;
import org.azuki.common.widget.JItemChooser;
import org.azuki.common.widget.jdom.JDOMElementTableCellRenderer;
import org.azuki.common.widget.model.ItemChooserTableModel;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class MethodChooser extends JItemChooser<Element> implements ActionListener
{
	final static private String UM = "Undefined method" ;
	
	final protected TableCellRenderer defaultCellRenderer = new JDOMElementTableCellRenderer(Attr.NAME, "(undefined)") ;
	final protected ListSelectionListener defaultSelectionAdapter = new ListSelectionAdapter() ;
	final protected TableModelListener itemSelectionAdapter = new ItemSelectionAdapter() ;
	
	final protected Method[] methods ;
	
	protected JTextArea methodDesc ;
	protected JScrollPane methodDescScroll ;
	protected JLabel methodDescLabel ;
	protected JPanel methodDescPane ;
	
	public MethodChooser(Collection<Element> lmens, Method[] methods)
	{
		super(new ItemChooserTableModel<Element>("Method", new ArrayList<Element>(lmens), Boolean.FALSE)) ;
		
		this.methods = methods ;
		
		methodDesc = new JTextArea() ;
		methodDesc.setRows(4) ;
		methodDesc.setEditable(false) ;
		methodDesc.setLineWrap(true) ;
		
		methodDescScroll = new JScrollPane(methodDesc) ;
		
		methodDescLabel = new JLabel("Description :") ;
		methodDescLabel.setLabelFor(methodDesc) ;
		
		methodDescPane = new JPanel(new BorderLayout()) ;
		methodDescPane.add(methodDescLabel, BorderLayout.NORTH) ;
		methodDescPane.add(methodDescScroll, BorderLayout.SOUTH) ;
		
		setMethodCellRenderer(defaultCellRenderer) ;
		
		addItemSelectionListener(itemSelectionAdapter) ;
		addListSelectionListener(defaultSelectionAdapter) ;
		
		add(methodDescPane, BorderLayout.SOUTH) ; //TODO: faire autrement
	}
	
	final public void setMethodCellRenderer(TableCellRenderer cellRenderer)
	{
		table.getColumnModel().getColumn(1).setCellRenderer(cellRenderer) ;
	}
	
	final public Method getMethod(String name)
	{
		if(methods==null) return null ;
		
		for(Method method : methods)
			if(method.getName().equals(name)) return method ;
		
		return null ;
	}
	
	/*
	 * Default table selection adapter 
	 */
	final public class ListSelectionAdapter implements ListSelectionListener
	{
		final public void valueChanged(ListSelectionEvent e)
		{
			if(e.getValueIsAdjusting()) return ;
			
			int idx = table.getSelectedRow() ;
			
			if(idx==-1)
			{	methodDesc.setText("") ;
				return ; }
			
			String methodName = ((Element)table.getValueAt(idx, 1)).getAttributeValue(Attr.NAME) ;
			Method method = getMethod(methodName) ;
			
			methodDesc.setText(method==null ? UM : Annotations.getMethodDescription(method)) ;
			methodDesc.setCaretPosition(0) ;
		}
	}
	
	/*
	 * Item selection adapter
	 */
	final public class ItemSelectionAdapter implements TableModelListener
	{
		final public void tableChanged(TableModelEvent e)
		{
			if(dialogActions!=null)
				dialogActions.getAction(OK_ACTION).setEnabled(tableModel.isSelectionValid()) ;
		}
	}
}