/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.framework ;

import org.azuki.common.consts.Attr;
import org.jdom.Element;

//This attributes are immutable, does not depends on the 
//bean instance definition.

//Static metadata

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com
 */

final public class BeanDefinition
{
	final private BeanRepository repo ;
	
	final private String   name ;
	final private Class    clas ;
	final private String   jarf ;
	final private String   vers ;
	
	final private Element lmen ;
	
	private AspectDefinition adef ;
	
	protected BeanDefinition(BeanRepository repo, String name, Class clas, String jarf, String vers, Element lmen) 
	{
		this.repo = repo ;
		
		this.name = name ;
		this.clas = clas ;
		this.jarf = jarf ;
		this.vers = vers ;
		
		this.lmen = lmen ;
	}
	
	final protected void setAspectDefinition(AspectDefinition adef)
	{
		this.adef = adef ;
	}
	
	final public BeanRepository getRepository() 		{ return repo; }
		
	final public String getName()						{ return name; } 
	
	final public Class getBeanClass()					{ return clas; }
	
	final public String getJarFile()					{ return jarf; }

	final public String getVersion()					{ return vers; }
	
	final public Element getElement()					{ return (Element)lmen.clone(); }
	
	final public String getDescription()				{ return lmen.getAttributeValue(Attr.DESC); }
	
	final public String getIcon()						{ return lmen.getAttributeValue(Attr.ICON); }
	
	final public String getGroup()						{ return lmen.getAttributeValue(Attr.GRUP); }
	
	final public String getDocumentation()				{ return lmen.getAttributeValue(Attr.BDOC); }
	
	final public boolean isAspect()						{ return adef!=null; }
	
	final public AspectDefinition getAspectDefinition()	{ return adef; }
}