/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.reflect ;

import java.lang.reflect.Method;
import java.util.*;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class Signature implements Comparable<Signature>
{
	final static public String VARARGS = "?" ;
	
	final static private String[] NO_ARGS = {} ;
	
	// Remarque : les types  sont exprimés avec leur nom court
	// ex : String au lieu de java.lang.String
	// Ce qui implique que l'on ne peut deduire l'object class
	// a partir de ce type qui n'est pas assez pecis.
	
	final private String stur ; // ex : "getToto(int, String):String"
	final public  String getSignature() { return stur; } 

	final private String meth ;
	final public String getMethod() { return meth; }

	final private String[] para ; // if method as no parameters, para=NO_ARGS (String[0])
	final public  String[] getParametersType() { return para; }

	final private String rtyp ; 
	final public  String getReturnType() { return rtyp; }
	
	public Signature(String stur)
	throws Exception
	{
		if(stur==null) 
			throw new IllegalArgumentException("signature string cannot be null") ;
		
		this.stur = stur ;
		
		String tmp ;
		
		// return type
		int idx = stur.indexOf(':') ;
		if(idx==-1)
		{
			tmp  = stur   ;
			rtyp = "void" ;
		}
		else
		{
			tmp  = stur.substring(0, idx) ;
			rtyp = stur.substring(idx+1) ;
		}
		
		// arguments
		idx = tmp.indexOf('(') ;
		if(idx==-1)
		{
			// no args, same as ()
			para = NO_ARGS ; 
		}
		else
		{
			int idx2 = tmp.indexOf(')', idx) ;
			if (idx2==-1)
				throw new Exception(stur) ;
			String args = tmp.substring(idx+1, idx2) ;
			args.trim() ;
			
			ArrayList<String> list = new ArrayList<String>() ;
			StringTokenizer stk = new StringTokenizer(args, ",") ;
			while(stk.hasMoreTokens())
				list.add(stk.nextToken().trim()) ;
			para = new String[list.size()] ;
			list.toArray(para) ;
		
			tmp = tmp.substring(0, idx) ;
		}
		
		meth = tmp ;
	}

	public Signature(Method method)
	{
		if(method==null)
			throw new IllegalArgumentException("method cannot be null") ;
	
		meth = method.getName() ;
		
//		Class c = method.getReturnType() ;
//		String cname = c.getName() ;
//		rtyp = cname.substring(cname.lastIndexOf('.')+1) ;
		rtyp = method.getReturnType().getSimpleName() ;
		
		Class[] cs = method.getParameterTypes() ;
		para = new String[cs.length] ;
		for(int i=0; i<para.length; i++)
		{
			para[i] = cs[i].getSimpleName() ;
		}
		
		StringBuilder sb = new StringBuilder(meth).append("(") ;
		boolean first = true ; 
		for(String p : para)
		{
			if(first) first=false ;
			else	  sb.append(',') ;
			sb.append(p) ;
		}
		sb.append("):") ;
		sb.append(rtyp) ;
		
		stur = sb.toString() ;
	}

	@Override
	final public boolean equals(Object obj)
	{
		if(this==obj) return true ;
		if(!(obj instanceof Signature)) return false ;
		return stur.equals(((Signature)obj).stur) ;
	}
	
	@Override
	final public String toString()
	{
		return stur ;
	}
	
	final public int compareTo(Signature o)
	{
		if(o==null) return -1 ;
		if(this==o) return 0 ;
		return stur.compareTo(o.stur) ;
	}
}