/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util ;

import java.io.*;

import javax.crypto.*;

import org.apache.commons.codec.binary.Base64;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

final public class Crypto
{
	final private java.security.Key key ;
	
	public Crypto(String passphrase)
	throws Exception
	{
		if(passphrase==null || passphrase.isEmpty())
			throw new Exception("Passphrase cannot be null or empty") ;
		
		javax.crypto.SecretKeyFactory kfact = javax.crypto.SecretKeyFactory.getInstance("DES") ;
		javax.crypto.spec.DESKeySpec kspec = new javax.crypto.spec.DESKeySpec(passphrase.getBytes()) ;
		this.key = kfact.generateSecret(kspec) ;
	}
	
	final private Cipher getCipher(int mode)
	throws Exception
	{
		Cipher cipher = Cipher.getInstance("DES/ECB/PKCS5Padding") ;
		cipher.init(mode, key) ;
		return cipher ;
	}    
			
	// Decypher the 'encoded' text to a readable text using a bean static internal key
	// Returns a readable text	
	final public String decrypt(String encoded)
	throws Exception
	{
		if(encoded==null) return null ;
		if("".equals(encoded)) return "" ;

		// Decode base64 to get bytes
		byte[] bin = Base64.decodeBase64(encoded.getBytes()) ;
		
		// Decryptage
        ByteArrayInputStream bis = new ByteArrayInputStream(bin) ;
        Cipher cipher = getCipher(Cipher.DECRYPT_MODE) ;
        CipherInputStream cis = new CipherInputStream(bis, cipher) ;
 
        byte[] buffer = new byte[256]; 
        ByteArrayOutputStream baos = new ByteArrayOutputStream() ;

        // Lecture
        boolean hasWrite = false ;
        while(true)
        {
        	int nb = cis.read(buffer) ;
        	if (nb==-1) break; // EOF
        	if (nb==0) continue ;
        	baos.write(buffer, 0, nb) ;
        	hasWrite = true ;
        }
        if(!hasWrite) return null ;
        
        cis.close();
        bis.close();
        
        // Recuperation du String
        return new String(baos.toByteArray(), "UTF8") ;
	}

	// Cypher the 'readable' text to an encoded text using a bean static internal key
	// Returns an encoded text	
	final public String encrypt(String readable)
	throws Exception
	{
		if(readable==null) return null ;
		if("".equals(readable)) return "" ;
		
		// Encode the string into bytes using utf-8
        byte[] utf8 = readable.getBytes("UTF8");
		
        // Pour recuperer la chaine cryptee
		ByteArrayOutputStream baos = new ByteArrayOutputStream() ; 

		// Cryptage
		Cipher cipher = getCipher(Cipher.ENCRYPT_MODE) ;
        CipherOutputStream cos = new CipherOutputStream(baos, cipher) ;
        cos.write(utf8) ;
        cos.close();
        
        // Encoder en base64 pour pouvoir la mettre dans le fichier XML
        byte[] encoded = Base64.encodeBase64(baos.toByteArray()) ;
		return new String(encoded) ;
	}
	
	//
	// STATIC
	//
	final static private String defp = "Demain se construit aujourd'hui" ;
	static private Crypto defc ;
	
	final static public String decypher(String encoded)
	throws Exception
	{
		if(defc==null) defc = new Crypto(defp) ;
		return defc.decrypt(encoded) ;
	}
	
	final static public String cypher(String readable)
	throws Exception
	{
		if(defc==null) defc = new Crypto(defp) ;
		return defc.encrypt(readable) ;
	}
}