/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */
package org.azuki.common.util ;

import java.sql.*;
import java.util.*;

import org.azuki.framework.context.*;

public class SQL
{
	final static public String MESSAGE_CANNOT_CONNECT_TO_DATABASE = "Not connected to a database" ;
	
	final static public String MAX_ROWS = "SQL_MAX_ROWS" ; 
	
	@SuppressWarnings("unchecked")
	final static public int getMaxTotalRecords()
	{
		Map<String,String> glob = null ;
		
		try
		{ glob = (Map<String,String>)ThreadContext.lookup(IContext.OBJECT_GLOBAL) ; }
		catch(Exception ex)
		{ Log.finest("Cannot retrieve ("+IContext.OBJECT_GLOBAL+") resource, using default") ; }
		
		if(glob==null) return -1 ;
	
		String valu = glob.get(MAX_ROWS) ;
		
		int maxr = -1 ;
		try
		{ maxr = Integer.parseInt(valu) ; }
		catch(Exception ex)
		{ Log.finest("No a valid integer value: "+valu+" , using default") ; }
		
		return maxr ;
	}
	
	
	final static public Connection getCurrentConnection()
	throws Exception
	{
		Connection conn = (Connection)ThreadContext.lookup(IContext.OBJECT_CONNECTION) ;
	
		if(conn==null)
		{	Log.severe(MESSAGE_CANNOT_CONNECT_TO_DATABASE) ;
			throw new SQLException(MESSAGE_CANNOT_CONNECT_TO_DATABASE) ; }
		
		conn.setAutoCommit(false) ; // ?
		
		return conn ;
	}
	
	final static public void release()
	{
		try
		{
			ThreadContext.release(IContext.OBJECT_CONNECTION) ; // Call the finalizer (close the connection)	
		}
		catch(Exception e)
		{
			// Don't care.
		}
	}
	
	
	final static public List<Object[]> query(String sql)
	throws Exception
	{
		return query(-1, sql) ;
	}
	
	final static public List<Object[]> query(int maxr, String sql)
	throws Exception
	{
		return SQLCx.query(getCurrentConnection(), maxr, sql) ;
	}
	
	final static public List<Object[]> query(Connection c, String sql, Object...args) 
	throws Exception
	{
		return SQLCx.query(c,-1, sql, args) ;
	}
	
	final static public List<Object[]> query(String sql, Object... args)
	throws Exception
	{
		return query(-1, sql, args) ;
	}
	
	final static public List<Object[]> query(int maxr, String sql, Object... args)
	throws Exception
	{
		return SQLCx.query(getCurrentConnection(), maxr, sql, args) ;
	}
	
	final static public int update(String sql, Object... arrs)
	throws Exception
	{
		return SQLCx.update(getCurrentConnection(), sql, arrs) ;
	}
	
	final static public int update(Connection c, String sql, Object... arrs)
	throws Exception
	{
		return SQLCx.update(c, sql, arrs) ;
	}
	
	final static public void updateWithTypes(String sql,  Object[] args, int[] sqlTypes)
	throws Exception
	{
		SQLCx.updateWithTypes(getCurrentConnection(), sql,  args, sqlTypes) ;
	}
	
	final static public void updateArray(String sql, Object[]... arrs)
	throws Exception
	{
		SQLCx.updateArray(getCurrentConnection(), sql, arrs) ;
	}
	
	final static public Integer[] updateArraySequence(String sql, String table, Object[]... arrs)
	throws Exception
	{
		return SQLCx.updateArraySequence(getCurrentConnection(), sql, table, arrs) ;
	}
	
	final static public void updateArray(Connection conn, String sql, Object[]... arrs)
	throws Exception
	{
		SQLCx.updateArray(conn, sql,  arrs) ;
	}
	
	final static public void commit()
	throws Exception
	{
		SQLCx.commit(getCurrentConnection()) ;
	}
	
	final static public void rollback()
	throws Exception
	{
		SQLCx.rollback(getCurrentConnection()) ;
	}
	

	@Deprecated
	final static public void close()
	throws Exception
	{
		try
		{
			commit() ; 
		}
		catch(Exception ex)
		{
			try { SQL.rollback() ; } catch(Exception e){}
			throw ex ;
		}
	}
}