/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util ;

import java.sql.*;
import java.util.*;

import org.azuki.framework.context.Log;

public class SQLCx
{
	final static public List<Object[]> query(Connection conn, String sql)
	throws Exception
	{
		return query(conn, -1, sql) ;
	}
	
	final static public List<Object[]> query(Connection conn, int maxr, String sql)
	throws Exception
	{
		return query(conn, maxr, sql, (Object[])null) ;
	}
	
	final static public List<Object[]> query(Connection conn, String sql, Object... args)
	throws Exception
	{
		
		return query(conn, -1, sql, args) ;
	}
	
	final static public List<Object[]> query(Connection conn, int maxr, String sql, Object... args)
	throws Exception
	{
		PreparedStatement st = null ;
		ResultSet rs = null ;
		
		Log.info("--> START SQL QUERY: "+sql) ;
		
		try
		{
			st = conn.prepareStatement(sql) ;
			
			if(args!=null)
			{
				for(int i=0 ; i<args.length ; i++)
				{
					Object arg = args[i] ;
					Log.info("ARG["+i+"] = "+arg+" ("+(arg==null?"NULL":arg.getClass().getName())+")") ;
					st.setObject(i+1, args[i]) ;
				}
			}
			
			rs = st.executeQuery() ;
			ResultSetMetaData rmta = rs.getMetaData() ;
			
			if(maxr<1) maxr = SQL.getMaxTotalRecords() ;
			
			int rows = 0 ;
			int cols = rmta.getColumnCount() ;
			
			List<Object[]> ret = new ArrayList<Object[]>() ;
			while(rs.next())
			{
				rows++ ;
				
				if(maxr>0 && rows>maxr)
					throw new SQLException("Too many rows (row count > "+maxr+")") ;
				
				Object[] row = new Object[cols] ;
				for(int i=0 ; i<cols ; i++)
					row[i] = rs.getObject(i+1) ;
				
				ret.add(row) ;
			}
			
			Log.info(ret.size()+" records found") ;
			Log.info("<-- END SQL QUERY") ;
			
			return ret ;
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
			throw ex ;
		}
		finally
		{
			if(rs!=null) try{ rs.close() ; } catch(Exception ex){}
			if(st!=null) try{ st.close() ; } catch(Exception ex){}
		}
	}
	
	final static public int update(Connection conn, String sql, Object[]  arrs)
	throws Exception
	{
		PreparedStatement ps = null ;
		
		Log.info("--> START SQL UPDATE: "+sql) ;
		
		try
		{
			ps = conn.prepareStatement(sql) ; 
			
			for(int i=0 ; i<arrs.length ; i++)
			{
				Log.finest("param no "+i);
				if (arrs[i]!=null)	
					Log.finest("Java type="+arrs[i].getClass().getName());
				else
					Log.finest("value is null");
				ps.setObject(i+1, arrs[i]) ;
			}
			int r = ps.executeUpdate() ;
			Log.info("<-- END SQL UPDATE  ("+r+" records)") ;
			return r;
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
			throw ex ;
		}
		finally
		{
			if(ps!=null) try{ ps.close() ; } catch(Exception ex){}
		}
	}
	
	final static public int updateWithTypes(Connection conn, String sql,  Object[] arrs, int[] sqlTypes)
	throws Exception
	{
		PreparedStatement ps = null ;
		
		Log.info("--> START SQL UPDATE: "+sql) ;
		
		try
		{
			ps = conn.prepareStatement(sql) ; 
			
			for(int i=0 ; i<arrs.length ; i++)
			{
				
				if (arrs[i]!=null)
				{
					Log.finest("param no "+i+"  type="+sqlTypes[i] +" "+arrs[i].toString());
					ps.setObject(i+1, arrs[i]) ;
				}
				else
				{
					Log.finest("value is null");
					ps.setNull(i+1,sqlTypes[i]);
				}
			}
			int r = ps.executeUpdate() ;
			Log.info("<-- END SQL UPDATE ("+r+" records)") ;
			return r;
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
			throw ex ;
		}
		finally
		{
			if(ps!=null) try{ ps.close() ; } catch(Exception ex){}
		}

	}
	
	
	final static public int updateArray(Connection conn, String sql, Object[]...arrs)
	throws Exception
	{
		PreparedStatement ps = null ;

		Log.info("--> START SQL UPDATEARRAY: "+sql) ;
		if (arrs==null  || arrs.length==0 || arrs[0]==null || arrs[0].length==0 )
		{
			Log.info("<-- END SQL UPDATEARRAY (0 records)") ;
			return 0;
		}

		try
		{
			ps = conn.prepareStatement(sql) ; 
			
			int count = arrs[0].length ;
			
			int totalrec =0;
			for(int i=0 ; i<count ; i++)
			{
				for(int j=0 ; j<arrs.length ; j++)
					ps.setObject(j+1, arrs[j][i]) ;

				totalrec = totalrec + ps.executeUpdate() ;
			}
			Log.info("<-- END SQL UPDATEARRAY ("+totalrec+" records)") ;
			return totalrec;
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
			throw ex ;
		}
		finally
		{
			if(ps!=null) try{ ps.close() ; } catch(Exception ex){}
		}
	}
	
	final static public Integer[] updateArraySequence(Connection conn, String sql, String table, Object[]... arrs)
	throws Exception
	{
		String sel = new StringBuilder("SELECT LAST_INSERT_ID() FROM ").append(table).toString() ;

		PreparedStatement ps = null ;
//		PreparedStatement sl = null ;
	
		int idx = 0 ;

		
		Log.info("--> START SQL UPDATEARRAY SEQUENCE : "+sql) ;
		
		
		Integer[] seqs = null ;
		
		try
		{
			
			int count = arrs[0].length ;
		
			seqs = new Integer[count] ;
			
			Log.info("COUNT : "+count) ;
			
			
			for(int i=0 ; i<count ; i++)
			{
				ps = conn.prepareStatement(sql) ; 
				for(int j=0 ; j<arrs.length ; j++)
					ps.setObject(j+1, arrs[j][i]) ;

				ps.executeUpdate() ;

				
				{
					ps.executeQuery(sel) ;
					
//					sl = conn.prepareStatement(sel) ;
					ResultSet rs = ps.executeQuery(sel) ;
					rs.next();
			
					int seq = rs.getInt(1) ;
					rs.close() ;
	//				sl.close() ;
					
					Log.info("NEW SEQUENCE : "+seq) ;
					seqs[idx++] = new Integer(seq);		
				}
			}
			
			ps.close() ;
			
			ps = null ; // For finally
		//	sl = null ; // For finally
			
			Log.info("<-- END SQL UPDATEARRAY SEQUENCE") ;
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
			throw ex ;
		}
		finally
		{
			if(ps!=null) try{ ps.close() ; } catch(Exception ex){}
		//	if(sl!=null) try{ sl.close() ; } catch(Exception ex){}
		}
		
		return seqs ;
	}
	
	
	final static public void commit(Connection conn)
	throws Exception
	{
		Log.info("--> START SQL COMMIT") ;
		
		conn.commit() ;
		
		Log.info("<-- END SQL COMMIT") ;
	}
	
	final static public void rollback(Connection conn)
	throws Exception
	{
		Log.info("--> START SQL ROLLBACK") ;
		
		conn.rollback() ;
		
		Log.info("<-- END SQL ROLLBACK") ;
	}
}