/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util ;

import java.sql.*;

import org.azuki.framework.context.Log;
import org.jdom.*;



/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

public class SQLXMLCx
{
	
	
	final static public Document query(Connection conn, int maxr, String sql, Object... args)
	throws Exception
	{
		PreparedStatement st = null ;
		ResultSet rs = null ;
		
		Log.info("--> START SQL QUERY: "+sql) ;
		
		try
		{
			st = conn.prepareStatement(sql) ;
			
			if(args!=null)
			{
				for(int i=0 ; i<args.length ; i++)
				{
					Object arg = args[i] ;
					Log.info("ARG["+i+"] = "+arg+" ("+(arg==null?"NULL":arg.getClass().getName())+")") ;
					st.setObject(i+1, args[i]) ;
				}
			}
			
			rs = st.executeQuery() ;
			
			if(maxr<1) maxr = SQL.getMaxTotalRecords() ;
			
			Document doc = toDocument(rs, maxr) ;

			Log.info("<-- END SQL QUERY") ;
			
			return doc ;
		}
		catch(Exception ex)
		{
			Log.severe(Log.getStack(ex)) ;
			throw ex ;
		}
		finally
		{
			if(rs!=null) try{ rs.close() ; } catch(Exception ex){}
			if(st!=null) try{ st.close() ; } catch(Exception ex){}
		}
	}
	
	/*
	 * <query rows="23">
	 * 		<row field1="" field2=""  field3="" />
	 * </query>
	 */	
	final static public Document toDocument(ResultSet rs, int maxr)
	throws Exception
	{
		ResultSetMetaData rmta = rs.getMetaData();
		int cols = rmta.getColumnCount();
		
		String[] colnames = new String[cols];						
		for(int i=1; i<=cols;i++)
			colnames[i] = rmta.getColumnLabel(i) ;
		
		Document doc = new Document() ;
		Element query = new Element("query") ;
		
		int rows = 0 ;	
		
		while(rs.next())
		{
			rows++ ;
			
			if(maxr>0 && rows>maxr)
				throw new SQLException("Too many rows (row count > "+maxr+")") ;
		
			Element row = new Element("row") ;

			for(int i=1 ; i<=cols ; i++)
				row.setAttribute(colnames[i], String.valueOf(rs.getObject(i))) ;

			query.addContent(row) ;
		}

		Log.info(rows+" records found") ;
		query.setAttribute("rows", String.valueOf(rows)) ;
		doc.addContent(query) ;
		return doc ;
	}
}