/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util ;

import java.util.*;

import org.azuki.common.widget.model.ItemChooserTableModel.ItemSelection;
import org.azuki.framework.aop.IContract.Selection;
import org.jdom.Element;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class SelectionUtil
{
	final static public ItemSelection toItemSelection(Selection s)
	{
		if(s==Selection.ONLY_ONE)		return ItemSelection.ONLY_ONE ;
		if(s==Selection.AT_LEAST_ONE)	return ItemSelection.AT_LEAST_ONE ;
		return ItemSelection.UNCHECKED ;
	}
	
	final static public boolean isSelectionValid(Selection slct, Element lmen, String chld, String attr, Collection<String> validValues)
	{
		// check for invalid values
		List<String> invd = JDOMUtil.listChildrenAttributes(lmen, chld, attr) ;
		invd.removeAll(validValues) ;
		if(!invd.isEmpty()) return false ;
		
		if(slct==Selection.UNCHECKED) return true ;
		
		// checks valid values against selection type
		List<String> list = JDOMUtil.listChildrenAttributes(lmen, chld, attr) ;
		list.retainAll(validValues) ;
		
		int count = list.size() ; 
		
		if(slct==Selection.NONE) 				return count==0 ;
		else if(slct==Selection.ONLY_ONE) 		return count==1 ;
		else if(slct==Selection.AT_LEAST_ONE)	return count>=1 ;
		
		// cannot be reached !
		return false ;
	}
	
	final static public String asString(Selection s)
	{
		return StringUtil.replace(s.name(), "_", " ").toLowerCase() ;
	}
}