/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.util ;

import java.io.File;
import java.net.*;
import java.util.*;

import org.jdom.*;

/**
 * @author Robert Bakic (robert.bakic@gmail.com)
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

final public class Util
{
	final static public char beanSeparatorChar = ':' ;
	final static public char methodSeparatorChar = '.' ;
	
	final static public String oneString(String name, String labl)
	{
		StringBuilder sb = new StringBuilder(name) ;
		if(labl!=null && !labl.isEmpty())
			sb.append(beanSeparatorChar).append(labl) ;
		return sb.toString() ;
	}
	
	final static public boolean isOK(String value)
	{
		if(value==null) return false ;
		if(value.equalsIgnoreCase("YES")) return true ;
		if(value.equalsIgnoreCase("ON"))  return true ;
		if(value.equalsIgnoreCase("TRUE"))  return true ;
		return false ;
	}	
	
	final static public URL[] dirToURLs(String libpath)
    {
		return dirsToURLs(Collections.singletonList(libpath)) ;
    }
	
	final static public URL[] dirsToURLs(List<String> libpaths)
    {
		ArrayList<URL> aurl = new ArrayList<URL>() ;
		
		for(String libpath : libpaths)
		{
	    	File dir = new File(libpath) ;
	
	    	if(!dir.exists())
	    	{
	    		System.out.println("Directory "+libpath+" does not exist !") ;
	    		continue ;
	    	}

	    	// for azuki eclipse plugin, we need to add jars project in the classloader
	    	// so url is a jar file , not a directory.
//	    	if(!dir.isDirectory())
//	    	{
//	    		System.out.println("File "+libpath+" is not a directory !") ;
//	    		continue ;
//	    	}	    		
	    	
	    	System.out.println("Scanning : ("+libpath+")") ;
	    	
	    	File[] fs = dir.listFiles() ;
	    	// directory is empty ?
	    	if (fs!=null)
	    	{
	    		for(int i=0; i<fs.length; i++)
				{
			    	File f = fs[i] ;
				    //if(f.isDirectory()) continue ;
				    
				    String name = f.getName() ;
			    	//if(!name.endsWith(".jar")) continue ;
			    	//System.out.println("\tfound : ("+name+")") ;
			    	System.out.println("\tadd : ("+name+")") ;
			    	
				    try
				    {
				    	aurl.add(f.toURI().toURL()) ;
				    }
				    catch(Exception e)
				    {
				    	e.printStackTrace() ;
				    }
				}
	    	}
			try
			{
				// add parent directory (for eclipse plugin)
				aurl.add(dir.toURI().toURL());
				System.out.println("\tadd : ("+dir+")") ;
		    	
			}
			catch (MalformedURLException e)
			{
				e.printStackTrace();
			}
		}
		
		URL[] urls = new URL[aurl.size()] ;
		aurl.toArray(urls) ;
		return urls ;
    }
	
	final static public Collection<URL> listURLs(File file, boolean deep, String... excl)
	{
		return listURLs(file, deep, Arrays.asList(excl)) ;
	}
	
	final static public Collection<URL> listURLs(File file, boolean deep, Collection<String> excl)
	{
		String filepath = file.getAbsolutePath() ;
		
		if(!file.isDirectory())
		{	System.out.println("Not a directory : ("+filepath+")") ;
			return Collections.emptyList() ; }
		
		if(!file.exists())
		{	System.out.println("Directory doesn't exists : ("+filepath+")") ;
			return  Collections.emptyList() ; }
		
		System.out.println("Scanning directory ("+filepath+")") ;

		List<URL> urls = new ArrayList<URL>() ;
		List<File> dirs = new ArrayList<File>() ; // for deep listing ordering !
		
		try
		{ urls.add(file.toURI().toURL()) ; }
		catch(Exception ex)
		{ ex.printStackTrace() ; }
		
		File[] farr = file.listFiles() ;
		for(File f : farr)
		{
			String fnam = f.getName() ;
			if(excl.contains(fnam)) continue ;
			
			if(f.isDirectory()) dirs.add(f) ;
			if(!fnam.endsWith(".jar") && !fnam.endsWith(".zip")) continue ;
			
			System.out.println("\tadding ("+fnam+")") ;
			try
			{ urls.add(f.toURI().toURL()) ; }
			catch(Exception ex)
			{ ex.printStackTrace() ; }
		}

		if(deep)
		{
			for(File dir : dirs)
				urls.addAll(listURLs(dir, deep, excl)) ;
		}
		
		return urls ;
	}
	
    @SuppressWarnings("unchecked")
	final static public Map<String, String> attrToMap(Element lmen)
    {
    	HashMap<String, String> map = new HashMap<String, String>() ;
    	if(lmen==null) return map ;
    	
    	List<Attribute> list = lmen.getAttributes() ; 
    	for(Attribute attr : list)
    		map.put(attr.getName(), attr.getValue()) ;
    	
    	return map ;
    } 	
    
    final static public Element mapToAttr(String elemName, Map<String, String> m)
    {
        Element e = new Element(elemName);
        
        if(m==null) return e ;
        Iterator it = m.keySet().iterator();
        while (it.hasNext())
        {
            String key = (String)it.next();
            e.setAttribute(key,m.get(key));
        }
        return e;
    }
    
    final static public class Key
    {
    	final private String name ;
    	final public String getName()  { return name ; }
    	
    	final private String labl ;
    	final public String getLabel() { return labl ; }
    	
    	@Override
		final public String toString() { return Util.oneString(name, labl) ; }
    	
    	private Key(String name, String labl)
    	{
    		this.name = name ;
    		this.labl = labl ;
    	}
    	
    	final static public Key valueOf(String str)
    	{
    		if(str==null || str.isEmpty()) return null ;
    		
    		int idx = str.indexOf(beanSeparatorChar) ;
    		
    		String name = idx==-1 ? str  : str.substring(0, idx) ;
			String labl = idx==-1 ? null : str.substring(idx+1) ;
			
			return new Key(name, labl) ;
    	} 
    }
}