/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.util.*;

import javax.swing.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JActionPanel extends JPanel
{
	final protected Map<Action,JButton> actionMap = new HashMap<Action,JButton>() ;
	
	private int orientation = SwingConstants.HORIZONTAL ;
	
	public JActionPanel()
	{
		this(SwingConstants.HORIZONTAL) ;
	}
	
	public JActionPanel(int orientation)
	{
		this.orientation = orientation ;
		
		setLayout(new BoxLayout(this, orientation==SwingConstants.VERTICAL ? BoxLayout.PAGE_AXIS : BoxLayout.LINE_AXIS)) ;
		add(Box.createGlue()) ;
	}
	
	public JActionPanel(Action... actions)
	{
		this(SwingConstants.HORIZONTAL, actions) ;
	}

	public JActionPanel(int orientation, Action... actions)
	{
		this(orientation, Arrays.asList(actions)) ;
	}
	
	public JActionPanel(Collection<Action> actions)
	{
		this(SwingConstants.HORIZONTAL, actions) ;
	}
	
	public JActionPanel(int orientation, Collection<Action> actions)
	{
		this(orientation) ;
		
		for(Action action : actions)
			addAction(action) ;
	}
	
	final public void addAction(Action action)
	{
		if(getComponentCount()>0)
			add(orientation==SwingConstants.VERTICAL ? Box.createVerticalStrut(5) : Box.createHorizontalStrut(5)) ;
		
		JButton jb = new JButton(action) ;
		add(jb) ;
		
		actionMap.put(action, jb) ;
	} 
	
	final public Action getAction(String actionName)
	{
		for(Action action : actionMap.keySet())
		{	if(actionName.equals(action.getValue(Action.ACTION_COMMAND_KEY)))
				return action ; }
		
		return null ;
	}
	
	final public boolean containsAction(Action action)
	{
		return actionMap.containsKey(action) ;
	}
	
	final public boolean containsAction(String actionName)
	{
		return getAction(actionName)!=null ;
	}
	
	final public JButton getButton(Action action)
	{
		return actionMap.get(action) ; 
	}
	
	final public JButton getButton(String actionName)
	{
		Action action = getAction(actionName) ;
		return action==null ? null : actionMap.get(action) ;
	}
}