/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.EventListenerList;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JCoolTabbed extends JPanel implements ItemListener
{
	final static private Insets TAB_INSETS = new Insets(2,2,2,2) ;

	final protected EventListenerList listenerList = new EventListenerList() ;	// ToggleButton listeners proxy 
	
	final private Map<JToggleButton,JComponent> tabMapping ;
	
	private JToggleButton selectedTab = null ;
	
	private JToolBar tabs ;
	private JViewport viewport ;
	
	public JCoolTabbed()
	{
		super() ;
		
		tabMapping = new HashMap<JToggleButton,JComponent>() ;
			
		tabs = new JToolBar(SwingConstants.VERTICAL) ;
		tabs.setFloatable(false) ;
		
		viewport = new JViewport() ;
		
		setLayout(new BorderLayout()) ;
		add(tabs, BorderLayout.WEST) ;
		add(viewport, BorderLayout.CENTER) ;
	}

	final public void addTab(Action action, JComponent component)
	{
		String actionName = (String)action.getValue(Action.NAME) ;
		Icon actionIcon = (Icon)action.getValue(Action.SMALL_ICON) ;
		
		// TODO: gestion des actions clavier
		
		addTab(actionName, actionIcon, component) ;
	}
	
	final public void addTab(String name, Icon icon, JComponent component)
	{
		JIconList iconList = new JIconList(icon, new JTextIcon(name)) ;
		JToggleButton togg = new JToggleButton(new JIcon(iconList, JIcon.ROTATE_LEFT)) ;
		
		togg.setName(name) ;
		togg.setMargin(TAB_INSETS) ;
		togg.addItemListener(this) ;
		
		tabMapping.put(togg, component) ;
		
		tabs.add(togg) ;
	}
	
	final public void removeTab(String name)
	{
		JToggleButton tab = null ;
		
		for(JToggleButton tb : tabMapping.keySet())
		{
			if(!tb.getName().equals(name)) continue ;	
			tab = tb ;	// tab found
			break ;
		}
		
		if(tab!=null)
			tabMapping.remove(tab) ;
	}
	
	final public String getSelectedTab()
	{
		return selectedTab==null ? null : selectedTab.getName() ;
	}
	
	final public void setSelectedTab(String name)
	{
		for(JToggleButton tb : tabMapping.keySet())
		{
			if(!tb.getName().equals(name)) continue ;	
			tb.setSelected(true) ;
			break ;
		}
	}
	
	final protected void fireItemStateChanged(ItemEvent e)
	{
		ItemListener[] arr = listenerList.getListeners(ItemListener.class) ;
		for(ItemListener l : arr)
			l.itemStateChanged(e) ;
	}
	
	final public void addItemListener(ItemListener l)
	{
		listenerList.add(ItemListener.class, l) ;
	}
	
	final public void removeItemListener(ItemListener l)
	{
		listenerList.remove(ItemListener.class, l) ;
	}
	
	/*
	 * ItemListener
	 */
	final public void itemStateChanged(ItemEvent e)
	{
		if(e.getStateChange()==ItemEvent.SELECTED)
		{
			JToggleButton tab = (JToggleButton)e.getItem() ;
			if(selectedTab!=null)
				selectedTab.setSelected(false) ;
			
			selectedTab = tab ;
			viewport.setView(tabMapping.get(e.getItem())) ;
		}
		else
		{
			selectedTab = null ;
			viewport.setView(null) ;
		}
		
		fireItemStateChanged(e) ;
	}
}