/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.azuki.common.action.ActionFactory;
import org.azuki.common.icon.CommonIcons;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDialogPanel extends JPanel
{
	final static public String OK_ACTION = "okAction" ;
	final static public String CANCEL_ACTION = "cancelAction" ;
	
	final static public String[] OK_CANCEL_ACTIONS = {OK_ACTION, CANCEL_ACTION} ;
	
	final protected ActionListener dialogActionAdapter = new DialogActionAdapter() ;
	
	final private Action okAction = ActionFactory.create(OK_ACTION, "OK", new ImageIcon(getClass().getResource("/"+CommonIcons.APPLY_16)), dialogActionAdapter) ;
	final private Action cancelAction = ActionFactory.create(CANCEL_ACTION, "Cancel", new ImageIcon(getClass().getResource("/"+CommonIcons.CANCEL_16)), dialogActionAdapter) ;
	
	protected Dimension preferredDialogSize ; 
	
	protected JActionPanel dialogActions ;
	protected JDialog dialog ;
	
	protected JComponent dialogHeader = null ;
	
	protected Object dispositionConstraints = BorderLayout.CENTER ;
	
	protected boolean validated = false ;
	
	public JDialogPanel()
	{}
	
	public JDialogPanel(LayoutManager layoutManager)
	{
		super(layoutManager) ;
	}
	
	public JComponent getDialogHeader()
	{
		return dialogHeader ;
	}
	
	public void setDialogHeader(JComponent dialogHeader)
	{
		this.dialogHeader = dialogHeader ;
	}
	
	public void clearDialogHeader()
	{
		setDialogHeader(null) ;
	}
	
	public Dimension getPreferredDialogSize(Dimension preferredDialogSize)
	{
		return preferredDialogSize ;
	}
	
	public void setPreferredDialogSize(Dimension preferredDialogSize)
	{
		this.preferredDialogSize = preferredDialogSize ;
	}
	
	public Object getDispositionConstraints()
	{
		return dispositionConstraints ;
	}
	
	public void setDispositionConstraints(Object dispositionConstraints)
	{
		this.dispositionConstraints = dispositionConstraints ;
	}
	
	final private void initCloseAction()
	{
		String closeActionName = "close" ;
		Action closeAction = new AbstractAction(closeActionName)
		{
			final public void actionPerformed(ActionEvent e)
			{
				dialog.dispatchEvent(new WindowEvent(dialog, WindowEvent.WINDOW_CLOSING)) ;
			}
		} ;
		
		dialog.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), closeActionName);
		dialog.getRootPane().getActionMap().put(closeActionName, closeAction) ;
	}
	
	public boolean showDialog(Component parent, String title)
	{
		return showDialog(parent, title, OK_CANCEL_ACTIONS, OK_ACTION) ;
	}
	
	protected boolean showDialog(Component parent, String title, String[] actions, String defaultAction)
	{
		dialogActions = new JActionPanel() ;
		for(String action : actions)
		{
			if(action.equals(OK_ACTION)) dialogActions.addAction(okAction) ;
			else if(action.equals(CANCEL_ACTION)) dialogActions.addAction(cancelAction) ;
		}
		
		JPanel dialogPane = new JPanel(new BorderLayout()) ;
		dialogPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
		dialogPane.add(this, dispositionConstraints) ;
		dialogPane.add(dialogActions, BorderLayout.SOUTH) ;
		
		if(preferredDialogSize!=null) dialogPane.setPreferredSize(preferredDialogSize) ;
		
		dialog = new JDialog(JOptionPane.getFrameForComponent(parent)) ;
		dialog.setTitle(title) ;
		dialog.setModal(true) ;
		
		dialog.getContentPane().setLayout(new BorderLayout()) ;
		dialog.getContentPane().add(dialogPane, BorderLayout.CENTER) ;
		if(dialogHeader!=null)
			dialog.getContentPane().add(dialogHeader, BorderLayout.NORTH) ;
		
		dialog.getRootPane().setDefaultButton(dialogActions.getButton(defaultAction)) ;
		
		initCloseAction() ;
		preShow() ;
		
		dialog.pack() ;
		
		dialog.setLocationRelativeTo(parent) ;
		dialog.setVisible(true) ;
		
		return validated ;
	}
	
	// Overload
	protected void preShow(){}
	
	/**
	 * Dialog action handler
	 */
	final class DialogActionAdapter implements ActionListener
	{
		/*
		 * ActionListener
		 */
		final public void actionPerformed(ActionEvent e)
		{
			String action = e.getActionCommand() ;
			validated = action==OK_ACTION ;
			
			if(dialog!=null) dialog.setVisible(false) ;
		}
	}
}