/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;

import org.azuki.common.widget.model.FilteredListModel;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JFilteredList<T> extends JDialogPanel implements DocumentListener, MouseListener
{
	final private DefaultListDataListener defaultListDataListener = new DefaultListDataListener() ;
	
	protected JLabel filterLabel ;
	protected JTextField filterField ;
	protected JPanel filterPanel ;
	
	protected FilteredListModel<T> listModel ;
	protected JList list ;
	protected JScrollPane listScroll ;
	
	public JFilteredList()
	{
		this(new ArrayList<T>()) ;
	}
	
	public JFilteredList(List<T> items)
	{
		super(new BorderLayout(5,5)) ;
		
		filterField = new JTextField() ;
		filterField.getDocument().addDocumentListener(this) ;
		
		filterLabel = new JLabel("Filter") ;
		filterLabel.setLabelFor(filterField) ;
		
		filterPanel = new JPanel(new BorderLayout(2,2)) ;
		filterPanel.add(filterLabel, BorderLayout.WEST) ;
		filterPanel.add(filterField, BorderLayout.CENTER) ;
		
		listModel = new FilteredListModel<T>(items) ;
		listModel.addListDataListener(defaultListDataListener) ;
		
		list = new JList(listModel) ;
		list.setFont(list.getFont().deriveFont(Font.PLAIN)) ;
		list.addMouseListener(this) ;
		
		listScroll = new JScrollPane(list) ;
		listScroll.setPreferredSize(new Dimension(320,200)) ;
		
		add(filterPanel, BorderLayout.NORTH) ;
		add(listScroll, BorderLayout.CENTER) ;
		
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
	}
	
	final public JList getJList()
	{
		return list ;
	}
	
	final public void setFilter(Filter<T> filter)
	{
		listModel.setFilter(filter) ;
	}
	
	final public void setCellRenderer(ListCellRenderer cellRenderer)
	{
		list.setCellRenderer(cellRenderer) ;
	}
	
	final public void setModel(List<T> items)
	{
		if(listModel!=null)
			listModel.removeListDataListener(defaultListDataListener) ;
		
		listModel = new FilteredListModel<T>(items) ;
		listModel.setFilterText(filterField.getText()) ;
		listModel.addListDataListener(defaultListDataListener) ;
		
		list.setModel(listModel) ;
	}
	
	final public FilteredListModel<T> getModel()
	{
		return listModel ;
	}
	
	final public void setSelectionMode(int mode)
	{
		list.setSelectionMode(mode) ;
	}
	
	final public void setSelectedItem(T t)
	{
		list.setSelectedValue(t, true) ;
	}
	
	@SuppressWarnings("unchecked")
	final public T getSelectedItem()
	{
		return (T)list.getSelectedValue() ;
	} 
	
	@SuppressWarnings("unchecked")
	final public List<T> getSelectedItems()
	{
		List<T> items = new ArrayList<T>() ;
		for(Object o : list.getSelectedValues())
			items.add((T)o) ;
		return items ;
	}
	
	final public List<T> getFilteredItems()
	{
		return listModel.getFilteredItems() ;
	}
	
	/*
	 * DocumentListener
	 */
	public void changedUpdate(DocumentEvent e)
	{
		listModel.setFilterText(filterField.getText()) ;
	}

	public void insertUpdate(DocumentEvent e)
	{
		listModel.setFilterText(filterField.getText()) ;
	}

	public void removeUpdate(DocumentEvent e)
	{
		listModel.setFilterText(filterField.getText()) ;
	}
	
	/*
	 * MouseListener
	 */
	final public void mouseClicked(MouseEvent e)
	{
		if(list.getSelectionModel().isSelectionEmpty()) return ;
		if(e.getClickCount()!=2 || dialog==null || !dialog.isVisible()) return ;
		
		validated = true ;
		dialog.setVisible(false) ;
	}
	final public void mouseEntered(MouseEvent e)
	{}
	final public void mouseExited(MouseEvent e)
	{}
	final public void mousePressed(MouseEvent e)
	{}
	final public void mouseReleased(MouseEvent e)
	{}
	
	/*
	 * Inner classes
	 */
	final class DefaultListDataListener implements ListDataListener
	{
		/*
		 * ListDataListener
		 */
		final public void contentsChanged(ListDataEvent e)
	    {
			list.clearSelection() ;
		}

		final public void intervalAdded(ListDataEvent e)
	    {}

		final public void intervalRemoved(ListDataEvent e)
	    {}
	}
	
	/*
	 * Filter interface
	 */
	public interface Filter<T>
	{
		public boolean match(T t, String text) ;
	} 
}