/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.azuki.common.action.Actions;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JFoldingPane extends JPanel
{
	final static public String PROPERTY_EXPANDED = "expanded" ;
	
	final static public Icon defaultOpenIcon = UIManager.getIcon("Tree.expandedIcon") ;
	final static public Icon defaultClosedIcon = UIManager.getIcon("Tree.collapsedIcon") ;
	
	final private MouseAdapter mouseAdapter = new MouseAdapter() ;
	
	final protected JLabel foldLabel ;
	
	final protected JToolBar foldTools ;
	
	final protected JComponent fold ; 
	
	final protected JView view ;
	
	protected String text ;
	protected Icon icon ;
	
	protected Component component ;
	
	protected Icon openIcon = defaultOpenIcon ;
	protected Icon closeIcon = defaultClosedIcon ;
	
	protected boolean expanded = false ;
	
	public JFoldingPane(Action action, Component component)
	{
		this(Actions.getLabel(action), Actions.getIcon(action), component) ;
	}
	
	public JFoldingPane(String text, Component component)
	{
		this(text, null, component) ;
	}
	
	public JFoldingPane(String text, Icon icon, Component component)
	{
		this.text = text ;
		this.icon = icon ;
		
		this.component = component ;
		
		foldLabel = new JLabel(text, icon==null ? closeIcon : new JIconList(closeIcon, icon), SwingConstants.LEFT) ;
		foldLabel.addMouseListener(mouseAdapter) ;
		
		foldTools = new JToolBar(SwingConstants.HORIZONTAL) ;
		foldTools.setFloatable(false) ;
		foldTools.setOpaque(false) ;
		foldTools.setBorder(BorderFactory.createEmptyBorder()) ;
		
		fold = new JPanel(new BorderLayout()) ;
		fold.add(foldLabel, BorderLayout.CENTER) ;
		fold.add(foldTools, BorderLayout.EAST) ;
		
		view = new JView() ;
		
		setLayout(new BorderLayout()) ;
		add(fold, BorderLayout.NORTH) ;
		add(view, BorderLayout.CENTER) ;
	}
	
	final public void setText(String text)
	{
		foldLabel.setText(text) ;
	}
	
	final public void setOpenIcon(Icon openIcon)
	{
		this.openIcon = openIcon ;
	}
	
	final public void setCloseIcon(Icon closeIcon)
	{
		this.closeIcon = closeIcon ;
	}
	
	final public JComponent getFoldComponent()
	{
		return fold ;
	}
	
	final public Component getComponent()
	{
		return component ;
	}
	
	final public void addTool(JComponent c)
	{
		foldTools.add(c) ;
	}
	
	final public void addTool(Action a)
	{
		JButton but = foldTools.add(a) ;
		but.setBorder(BorderFactory.createEmptyBorder()) ;
		but.setMargin(new Insets(1,1,1,1)) ;
	}
	
	final public boolean isExpanded()
	{
		return expanded ;
	}
	
	final public void setExpanded(boolean expanded)
	{
		boolean oldExpanded = this.expanded ;
		
		Icon tico = expanded ? openIcon : closeIcon ;
		if(icon!=null) tico = new JIconList(tico, icon) ;
		
		foldLabel.setIcon(tico) ;
		
		view.setComponent(expanded ? component : null) ;
		
		this.expanded = expanded ;
		firePropertyChange(PROPERTY_EXPANDED, oldExpanded, expanded) ;
	}
	
	/*
	 * Expansion mouse adapter
	 */
	final protected class MouseAdapter implements MouseListener
	{
		final public void mouseClicked(MouseEvent e)
		{
			setExpanded(!expanded) ;
		}
		final public void mouseEntered(MouseEvent e) {}
		final public void mouseExited(MouseEvent e) {}
		final public void mousePressed(MouseEvent e) {}
		final public void mouseReleased(MouseEvent e) {}
	}
}