/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;

import org.azuki.common.widget.model.GenericListModel;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JGenericList<T> extends JDialogPanel implements MouseListener 
{
	protected GenericListModel<T> listModel ;
	protected JList list ;
	protected JScrollPane listScroll ;
	
	public JGenericList()
	{
		this(new ArrayList<T>()) ;
	}
	
	public JGenericList(List<T> items)
	{
		super(new BorderLayout(5,5)) ;
		
		listModel = new GenericListModel<T>(items) ;
		
		list = new JList(listModel) ;
		list.setFont(list.getFont().deriveFont(Font.PLAIN)) ;
		list.addMouseListener(this) ;
		
		listScroll = new JScrollPane(list) ;
		
		add(listScroll, BorderLayout.CENTER) ;
		
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5)) ;
	}
	
	final public JList getJList()
	{
		return list ;
	}
	
	final public void setModel(List<T> items)
	{
		listModel = new GenericListModel<T>(items) ;
		list.setModel(listModel) ;
	}
	
	final public GenericListModel<T> getModel()
	{
		return listModel ;
	}
	
	final public List<T> getItems()
	{
		return listModel.getItems() ;
	}
	
	final public void setSelectionMode(int mode)
	{
		list.setSelectionMode(mode) ;
	}
	
	final public void setSelectedItem(T t)
	{
		list.setSelectedValue(t, false) ;
	}
	
	@SuppressWarnings("unchecked")
	final public T getSelectedItem()
	{
		return (T)list.getSelectedValue() ;
	} 
	
	@SuppressWarnings("unchecked")
	final public List<T> getSelectedItems()
	{
		List<T> items = new ArrayList<T>() ;
		for(Object o : list.getSelectedValues())
			items.add((T)o) ;
		return items ;
	}
	
	final public void setCellRenderer(ListCellRenderer cellRenderer)
	{
		list.setCellRenderer(cellRenderer) ;
	}
	
	/*
	 * MouseListener
	 */
	final public void mouseClicked(MouseEvent e)
	{
		if(list.getSelectionModel().isSelectionEmpty()) return ;
		if(e.getClickCount()!=2 || dialog==null || !dialog.isVisible()) return ;
		
		validated = true ;
		dialog.setVisible(false) ;
	}
	final public void mouseEntered(MouseEvent e)
	{}
	final public void mouseExited(MouseEvent e)
	{}
	final public void mousePressed(MouseEvent e)
	{}
	final public void mouseReleased(MouseEvent e)
	{}
}