/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import javax.swing.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JHeaderPanel extends JPanel
{
	protected JLabel lablIcon ;
	protected JLabel lablTitl ;
	protected JLabel lablSubt ;
	
	public JHeaderPanel()
	{
		this(null, null, null) ;
	}
	
	public JHeaderPanel(String title)
	{
		this(title, null, null) ;
	}
	
	public JHeaderPanel(String title, String subt)
	{
		this(title, subt, null) ;
	}
	
	public JHeaderPanel(String title, Icon icon)
	{
		this(title, null, icon) ;
	}
	
	public JHeaderPanel(String title, String subtitle, Icon icon)
	{
		// labels
		lablTitl = new JLabel(title) ;
		lablTitl.setFont(lablTitl.getFont().deriveFont(Font.BOLD)) ;
		
		lablSubt = new JLabel(subtitle) ;
		lablSubt.setFont(lablSubt.getFont().deriveFont(Font.PLAIN).deriveFont(10.0f)) ;
		
		lablIcon = new JLabel(icon) ;
		
		JPanel leftPanel = new JPanel() ;
		leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.PAGE_AXIS)) ;
		leftPanel.add(lablTitl) ;
		leftPanel.add(lablSubt) ;
		
		// main panel
		JPanel mainPanel = new JPanel(new BorderLayout(10,10)) ;
		mainPanel.setBorder(BorderFactory.createEmptyBorder(7,7,7,7)) ;
		mainPanel.add(leftPanel, BorderLayout.CENTER) ;
		mainPanel.add(lablIcon, BorderLayout.EAST) ;
		
		// sets background properties
		setBackground(Color.white) ;
		mainPanel.setOpaque(false) ;
		leftPanel.setOpaque(false) ;
		
		// disposing components
		setLayout(new BorderLayout()) ;
		add(mainPanel, BorderLayout.NORTH) ;
		add(new JSeparator(), BorderLayout.SOUTH) ;
	}
	
	final public void setTitle(String title)
	{
		lablTitl.setText(title) ;
	}
	
	final public void setSubtitle(String subtitle)
	{
		lablSubt.setText(subtitle) ;
	}
	
	final public void setIcon(Icon icon)
	{
		lablIcon.setIcon(icon) ;
	}
}