/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;
import java.util.*;

import javax.swing.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class JIconList implements Icon
{
	final static public int DEFAULT_ICON_PADDING = 2 ;	// pixels 
	
	final protected java.util.List<Icon> iconList ;
	
	private int orientation ;	// SwingConstants.HORIZONTAL / SwingConstants.VERTICAL
	
	private int width = 0 ;
	private int height = 0 ;
	
	private int iconPadding = DEFAULT_ICON_PADDING ;
	
	/**
	 * TODO
	 * 	- rotation des icones en fonction de l'orientation ??
	 */
	
	public JIconList()
	{
		this(SwingConstants.HORIZONTAL) ;
	}
	
	public JIconList(int orientation)
	{
		this(new ArrayList<Icon>(), orientation) ;
	}
	
	public JIconList(Icon... icon)
	{
		this(SwingConstants.HORIZONTAL, icon) ;
	}
	
	public JIconList(int orientation, Icon... icon)
	{
		this(Arrays.asList(icon), orientation) ;
	}
	
	public JIconList(java.util.List<Icon> iconList, int orientation)
	{
		this.iconList = new ArrayList<Icon>() ;
		this.iconList.addAll(iconList) ;
		
		this.orientation = isValidOrientation(orientation) ? orientation : SwingConstants.HORIZONTAL ;
		
		computeUnits() ;
	}
	
	final private boolean isValidOrientation(int orientation)
	{
		return orientation==SwingConstants.HORIZONTAL || orientation==SwingConstants.VERTICAL ;
	}
	
	final public void add(Icon icon)
	{
		iconList.add(icon) ;
		computeUnits() ;
	}
	
	final public void addAll(Collection<Icon> icons)
	{
		iconList.addAll(icons) ;
		computeUnits() ;
	}
	
	final public void remove(Icon icon)
	{
		iconList.remove(icon) ;
		computeUnits() ;
	}
	
	final public void removeAll(Collection<Icon> icons)
	{
		iconList.removeAll(icons) ;
		computeUnits() ;
	}
	
	final private void computeUnits()
	{
		width = 0 ;
		height = 0 ;
		
		boolean isHorizontal = orientation==SwingConstants.HORIZONTAL ;
		boolean first = true ;
		
		for(Icon icon : iconList)
		{
			if(first)
			{
				first = false ;	
			}
			else
			{
				if(isHorizontal) width += iconPadding ;
				else			 height += iconPadding ;
			}
			
			if(isHorizontal)
			{
				width += icon.getIconWidth() ;
				height = Math.max(height, icon.getIconHeight()) ;
			}
			else
			{
				width = Math.max(width, icon.getIconWidth()) ;
				height += icon.getIconHeight() ;
			}
		}
	}
	
	final public int getOrientation()
	{
		return orientation ;
	}
	
	final public void setOrientation(int orientation)
	{
		if(!isValidOrientation(orientation)) return ;
		
		this.orientation = orientation ;
		computeUnits() ;
	}
	
	final public int getIconPadding()
	{
		return iconPadding ;
	}
	
	final public void setIconPadding(int iconPadding)
	{
		if(iconPadding<0) return ;
		
		this.iconPadding = iconPadding ;
		computeUnits() ;
	}
	
	/*
	 * Icon
	 */
	final public int getIconWidth()
	{
		return width ;
	}
	
	final public int getIconHeight()
	{
		return height ;
	}

	final public void paintIcon(Component c, Graphics g, int x, int y)
	{
		for(Icon icon : iconList)
		{
			int icx = orientation==SwingConstants.HORIZONTAL ? x : x+(width-icon.getIconWidth())/2 ;
			int icy = orientation==SwingConstants.VERTICAL ? y : y+(height-icon.getIconHeight())/2 ;
			
			icon.paintIcon(c, g, icx, icy) ;
			
			if(orientation==SwingConstants.HORIZONTAL)
				x += icon.getIconWidth()+iconPadding ;
			else
				y += icon.getIconHeight()+iconPadding ;
		}
	}
}