/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget ;

import java.awt.*;

import javax.swing.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JSplashScreen extends JWindow
{
	final private Runnable wact = new WaitAction() ;
	final private Runnable cact = new CloseAction() ;
	
	final private JLabel label ;
	final private int delay ;
	
	public JSplashScreen(Icon icon, int delay)
	{
		this.delay = delay ;
		
		label = new JLabel(null, icon, SwingConstants.CENTER) ;
		label.setBackground(Color.white) ;
		label.setBorder(BorderFactory.createLineBorder(Color.black)) ;
		
		setLayout(new BorderLayout()) ;
		add(label, BorderLayout.CENTER) ;
	}
	
	final public void splash()
	{
		pack() ;
		setLocationRelativeTo(null) ;
		setVisible(true) ;
		
		Thread thrd = new Thread(wact, "SplashThread") ;
        thrd.start() ;
	}
	
	/*
	 * Show the window during the specified delay
	 */
	final private class WaitAction implements Runnable
	{
		final public void run()
		{
			try
			{
				Thread.sleep(delay) ;
				
				SwingUtilities.invokeAndWait(cact) ;
			}
			catch(Exception ex)
			{
				//ex.printStackTrace() ;
            }
		}
	}
	
	/*
	 * Close the window
	 */
	final private class CloseAction implements Runnable
	{
		final public void run()
		{
			 setVisible(false) ;
			 dispose() ;
		}
	}
}