/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget.document ;

import java.util.*;

import javax.swing.text.*;

import org.azuki.common.widget.document.filter.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JDocument extends PlainDocument
{
	final protected List<IDocumentFilter> filters = new ArrayList<IDocumentFilter>() ;
	
	private boolean fltr = true ;
	
	
	final public boolean isFiltersEnabled()
	{
		return this.fltr ;
	}
	
	final public void setEnableFilters(boolean b)
	{
		this.fltr = b ;
	}
	
	final public boolean isFiltered()
	{
		return !filters.isEmpty() ;
	}
	
	final public void addFilter(IDocumentFilter filter)
	{
		filters.add(filter) ;
	}
	
	final public List<IDocumentFilter> getFilters()
	{
		return Collections.unmodifiableList(filters) ;
	}
	
	@Override
	public void insertString(int offs, String str, AttributeSet a)
	throws BadLocationException
	{
		if(str==null) return ;
		
		if(fltr)
		{
			for(IDocumentFilter fltrs : filters)
				if(!fltrs.accept(str)) return ;	// not a valid value
		}
		
		super.insertString(offs, str, a) ;
	}
}