/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget.field ;

import java.util.*;

import javax.swing.*;
import javax.swing.text.*;

import org.azuki.common.widget.document.*;
import org.azuki.common.widget.document.filter.*;
import org.azuki.common.widget.field.event.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public abstract class AbstractTextField extends JTextField implements IField
{
	protected String fieldName ;
	protected String initialValue ;
	
	private Vector<IFieldListener> listeners = new Vector<IFieldListener>() ;
	
	protected AbstractTextField()
	{
		this(null) ;
	}
	
	protected AbstractTextField(String text)
	{
		this(text, 0) ;
	}
	
	protected AbstractTextField(String text, int column)
	{
		super(null, text, column) ;
		
		this.initialValue = text ;
		
		setInputVerifier(new Verifier()) ;
	}
	
	final public boolean isFiltersEnabled()
	{
		Document doc = getDocument() ;
		return doc instanceof JDocument && ((JDocument)doc).isFiltersEnabled() ;
	}
	
	final public void setEnableFilters(boolean b)
	{
		Document doc = getDocument() ;
		if(doc instanceof JDocument)
			((JDocument)doc).setEnableFilters(b) ;
	}
	
	final public boolean isFiltered()
	{
		Document doc = getDocument() ;
		return doc instanceof JDocument && ((JDocument)doc).isFiltered() ;
	}
	
	final public void addFilter(IDocumentFilter filter)
	{
		Document doc = getDocument() ;
		if(doc instanceof JDocument)
			((JDocument)doc).addFilter(filter) ;	
	}
	
	final public List<IDocumentFilter> getFilters()
	{
		Document doc = getDocument() ;
		if(doc instanceof JDocument)
			return ((JDocument)doc).getFilters() ;
		
		return Collections.emptyList() ;
	}
	
	@Override
	protected Document createDefaultModel()
	{
		return new JDocument() ;
	}
	
	//FIXME: pas top --> on devrait passer par un listener ... quoique !!
	final public boolean hasValueChanged()
	{
		return !getFieldValue().equals(initialValue) ;
	}
	
	final public void notifyValueChanged()
	{
		fireValueChanged(new FieldEvent(this)) ;
		initialValue = getFieldValue() ;
	}
	
	final private void fireValueChanged(FieldEvent e)
	{
		for(IFieldListener l : listeners)
			l.valueChanged(e) ;
	}
	
	/*
	 * IField
	 */
	final public JComponent getUIComponent()
	{
		return this ;
	}
	
	final public void setInitialValue(String initialValue)
	{
		setEnableFilters(false) ;
		
		this.initialValue = initialValue ;
		setFieldValue(initialValue) ;
		
		setEnableFilters(true) ;
	}
	
	final public String getInitialValue()
	{
		return initialValue ;
	}
	
	final public void setFieldName(String fieldName)
	{
		this.fieldName = fieldName ;
	}
	
	final public String getFieldName()
	{
		return fieldName ;
	}
	
	public void setFieldValue(String fieldValue)
	{
		setText(fieldValue) ;
		// notifyValueChange() ;
		// attention la notification ce fait dans l'InputVerifier
	}
	
	public String getFieldValue()
	{
		return getText() ;
	}
	
	final public void addFieldListener(IFieldListener l)
	{
		listeners.add(l) ;
	}
	
	final public void removeFieldListener(IFieldListener l)
	{
		listeners.removeElement(l) ;
	}
	
	/*
	 * Inner class
	 */
	final protected class Verifier extends InputVerifier
	{
		@Override
		final public boolean verify(JComponent input)
		{
			AbstractTextField field = (AbstractTextField)input ;
			if(field.hasValueChanged())
				field.notifyValueChanged() ;
			return true ;
		}
	}
}