/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget.field ;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import org.azuki.common.widget.field.event.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

@SuppressWarnings("serial")
public class JSpinnerField extends JSpinner implements IField, ChangeListener
{
	protected String fieldName ;
	protected String initialValue ;
	
	private Vector<IFieldListener> listeners = new Vector<IFieldListener>() ;
	
	public JSpinnerField()
	{
		this(new SpinnerNumberModel()) ;
	}
	
	public JSpinnerField(SpinnerModel model)
	{
		super(model) ;
		model.addChangeListener(this) ;
	}
	
//	final public boolean hasValueChanged()
//	{
//		return !getFieldValue().equals(initialValue) ;
//	}
	
	final protected void notifyValueChanged()
	{
		fireValueChanged(new FieldEvent(this)) ;
		initialValue = getFieldValue() ;
	}
	
	final private void fireValueChanged(FieldEvent e)
	{
		for(IFieldListener l : listeners)
			l.valueChanged(e) ;
	}
	
	/*
	 * IField
	 */
	final public JComponent getUIComponent()
	{
		return this ;
	}
	
	final public void setInitialValue(String initialValue)
	{
		this.initialValue = initialValue ;
		setFieldValue(initialValue) ;
	}
	
	final public String getInitialValue()
	{
		return initialValue ;
	}
	
	final public void setFieldName(String fieldName)
	{
		this.fieldName = fieldName ;
	}
	
	final public String getFieldName()
	{
		return fieldName ;
	}
	
	final public void setFieldValue(String fieldValue)
	{
		//TODO: traiter les cas : DATE et LIST !!!
		SpinnerModel model = getModel() ;
		if(model instanceof SpinnerNumberModel)
			setValue(new Integer(fieldValue)) ;
		else
			setValue(fieldValue) ;		
	}
	
	final public String getFieldValue()
	{
		return String.valueOf(getValue()) ;
	}

	final public void addFieldListener(IFieldListener l)
	{
		listeners.add(l) ;
	}
	
	final public void removeFieldListener(IFieldListener l)
	{
		listeners.removeElement(l) ;
	}

	/*
	 * ChangeListener
	 */
	final public void stateChanged(ChangeEvent e)
	{
		notifyValueChanged() ;
	}
}