/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.common.widget.model ;

import java.util.*;

import javax.swing.ListModel;
import javax.swing.event.*;

/**
 * @author Yoann Perrin (yoann.perrin@gmail.com)
 */

public class GenericListModel<T> implements ListModel
{
	final protected EventListenerList listenerList = new EventListenerList() ;
	
	final protected List<T> items ;
	
	public GenericListModel(List<T> items)
	{
		this.items = new ArrayList<T>(items) ;
	}
		
	public int indexOf(T t)
	{
		return items.indexOf(t) ;
	}
	
	public List<T> getItems()
	{
		return Collections.unmodifiableList(items) ;
	}
	
	public void addItem(int index, T t)
	{
		items.add(index, t) ;
		fireIntervalAdded(this, index, index) ;
	}
	
	public void addItem(T t)
	{
		items.add(t) ;
		
		int index = items.size()-1 ;
		fireIntervalAdded(this, index, index) ;
	}
	
	public T removeItem(int index)
	{
		T t = items.remove(index) ;
		fireIntervalRemoved(this, index, index) ;
		return t ;
	}
	
	public boolean removeItem(T t)
	{
		return removeItem(indexOf(t))!=null ;
	}
	
	final protected void modelChanged()
	{
		fireContentsChanged(this, -1, -1) ;
	}
	
	final protected void fireContentsChanged(Object source, int index0, int index1)
    {
		ListDataEvent event = new ListDataEvent(source, ListDataEvent.CONTENTS_CHANGED, index0, index1) ;
		notifyChange(event) ;
    }
	
    final protected void fireIntervalAdded(Object source, int index0, int index1)
    {
    	ListDataEvent event = new ListDataEvent(source, ListDataEvent.INTERVAL_ADDED, index0, index1) ;
		notifyChange(event) ;
    }

    final protected void fireIntervalRemoved(Object source, int index0, int index1)
    {
    	ListDataEvent event = new ListDataEvent(source, ListDataEvent.INTERVAL_REMOVED, index0, index1) ;
		notifyChange(event) ;
    }
    
    final private void notifyChange(ListDataEvent event)
    {
    	ListDataListener[] arr = listenerList.getListeners(ListDataListener.class) ;
		for(ListDataListener ldl : arr)
			ldl.contentsChanged(event) ;
    }
	
	/*
	 * ListModel
	 */
	final public int getSize()
	{
		return items.size() ;
	}
	
	final public T getElementAt(int index)
	{
		return items.get(index) ;
	}

	final public void addListDataListener(ListDataListener l)
	{
		listenerList.add(ListDataListener.class, l) ;
	}
	
	final public void removeListDataListener(ListDataListener l)
	{
		listenerList.remove(ListDataListener.class, l) ;
	}
}