/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.aop ;


/**
 *  All aspect bean must implement this interface in order to 
 *  create a code advice. 
 *
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

public interface IAspect<T>
{
	/**
	 * Creates a code advice that will be bound with the target bean.
	 * The method must return null or throws a exception if the aspect
	 * cannot be applied. For example, if the target bean does not match the contract.
	 *
	 * @param  ipc  The pointcut definition interface
	 * @return      The code advice object. If the method return null, then the 
	 * 				framework considers that the aspect cannot be applied to the 
	 * 				targeted object.
	 * @throws      Exception. In this case, the aspect cannot be applied (same as returning null).  
	 */	
	 public T createAdvice(IPointcut ipc)
	 throws Exception;
}