/*
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.command;

/**
 * This interface must be implemented by a object willing to be notifed
 * when the <code>ICommand</code>'s <code>process</code> method execution has been completed. 
 *      
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

public interface ICallback<T>
{
	/** 
	 * Called by the framework when an a <code>ICommand</code>'s <code>process</code>
	 * method execution has been completed without throwing an Exception.
	 *
	 * @param res The result object returned by the <code>ICommand</code>'s <code>process</code> method.
	 */
	public void setResult(T res) ;
	
	
	/** 
	 * Called by the framework when an a <code>ICommand</code>'s <code>process</code>
	 * method execution throws an Exception.
	 *
	 * @param e The exception thrown by the <code>ICommand</code>'s <code>process</code> method.
	 */
	public void setError(Exception e) ;
}