/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.injection;

import java.lang.annotation.*;

import org.azuki.framework.command.ICallback;

/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */

public interface IMethodProxy<T>
{
	public boolean exists() ;
	
	//-----------------------------------------------
	
	public Integer getBeanID()
	throws Exception ;
	
	public String getMethod() ;
	
	public Class[] getParameterType() ; 
	
	//-----------------------------------------------
	
	public boolean isAsynchronous() ;
	
	public boolean isForked() ;

	public T service(Object... args) // performes a call(), invoke() or send() depending on constructor
	throws Exception ;

	//-----------------------------------------------
	
	public T call(Object... args)    // Synchronous
	throws Exception ;

	public T invoke(Object... args)  // Synchronous, Context forked (new context is used for the called method)
	throws Exception ;

	public void send(Object... args)  // Asynchronous, forked
	throws Exception ;

	public void send(ICallback cback, Object... args) // Asynchronous, forked and callback
	throws Exception ;

	// These annotations is only used by the weaver
	//---------------------------------------------
	@Retention(RetentionPolicy.RUNTIME)
	@Target(ElementType.FIELD)
	public @interface DependencyInjectionInfo
	{
		// Description of the needed method.
		String desc() default "" ; 
		
		// Parameter that will be passed by the invoker
		Class[] param() default {} ;
		
		// If this field is true the weaver will check if
		// the dependency injection has been done.
		boolean mandatory() default true ;
	}
}
