/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  
 *   Contact : support@azuki-framework.org
 */

package org.azuki.framework.injection;


/**
 * Provides informations on the invocation process currently performed on a bean method.
 *
 * @author Robert Bakic (robert.bakic@gmail.com)
 */

public interface InvocationTarget<T>
{
	/**
	 * Gets the instance of the targeted bean.
	 * @return The bean instance
	 */	
	public Object getBean() ;
	
	/**
	 * Gets the name of the method currently called.
	 * @return The method name
	 */	
	public String getMethodName() ;
	
	/**
	 * Gets the parameters arguments passed in this method call.
	 * @return The method arguments
	 */	
	public Object[] getParameters() ;

	
	/**
	 * Changes the parameters arguments passed to the method.
	 * @param args The new parameters arguments.
	 */	
	public void setParameters(Object[] args) ;

	/**
	 * The code advice must called this method in order to forward the method call 
	 * to the destination object or to the next code advice in the chain of control.

	 * @return The result of the targeted method call.
	 */	
	public T proceed() throws Exception;
}
