/*
 *   Created the 2 April 2006
 *
 *   Azuki Framework
 *   Copyright 2006 the original author or authors.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *   
 *   Contact : support@azuki-framework.org
 */

package org.azuki.kernel.metadata ;

import org.azuki.common.framework.BeanDefinition;
import org.azuki.common.util.Util;
import org.azuki.framework.aop.IContract;
import org.azuki.framework.bean.IMetadata;
import org.azuki.framework.command.*;
import org.azuki.framework.context.Log;


/**
 *   @author Robert Bakic (robert.bakic@gmail.com)
 */
public class BW extends PCC implements IMetadata
{
	private IQueue myQ ;
	final public IQueue getQueue()
	{ 
		if(myQ==null) // Default : no dedicated queue
		   myQ=Command.mainQueue() ;
		return myQ;
	}
	final public void setQueue(IQueue q) { myQ = q ;} // Can be modified by queuing aspect
	
	final private Integer bid ;
	final private String  name ;
	final private String  labl ;
	
	final private BeanDefinition bdef ;
	
	final private String  univ ;
	final private String  sche ;

	private Object bean ;
	
	public BW(Integer bid, String name, String labl, BeanDefinition bdef, String univ, String sche) 
	{
		this.bid  = bid  ;
		this.name = name ;
		this.labl = labl ;
		
		this.bdef = bdef ;
		
		this.univ = univ ;
		this.sche = sche ;
	}	
	
	// LIFECYCLE
	////////////////
	
	// The real implemention 
	final public void createInstance()
	{
		try
		{		
			StringBuilder sb = new StringBuilder("(").append(bid).append(',')
			.append(Util.oneString(name, labl)).append(")");
			
			Log.info(sb.toString());
	
			bean = getBeanClass().newInstance() ;
		}
		catch(Throwable t)
		{
			Log.severe("Instanciation ERROR: "+Log.getStack(t)) ;
		}
	}

	
	@Override
	final public void createAdvices()
	{
		if(bean==null)
		{	Log.info(new StringBuilder("Bean (").append(bid).append(") has not been created.").toString()) ;
			return ; }
		
		StringBuilder sb = new StringBuilder("(").append(bid).append(',')
		.append(Util.oneString(name, labl)).append(")");

		Log.info(sb.toString()) ;

		super.createAdvices() ;
		
		Log.info("\n") ;
	}
	
	// END LIFECYCLE

	final public Integer getID()    { return bid; }
	final public String  getName()  { return name; }
	final public String  getLabel() { return labl; }
	
	final public boolean isAspect()     { return bdef.isAspect(); }
	final public IContract getContract(){ return isAspect() ? bdef.getAspectDefinition().getContractObject() : null; }
	
	final public Object getBean() { return bean; }	

	final public String getSchema()   { return sche; }	
	final public String getUniverse() { return univ; }
	
	final public Class  getBeanClass()   { return bdef.getBeanClass(); }
	final public String getVersion()     { return bdef.getVersion(); }
	final public String getDescription() { return bdef.getDescription(); }
	
	final public String getIcon()  { return bdef.getIcon() ; }
	final public String getGroup() { return bdef.getGroup() ; }
	
	@Override
	final public String toString()
	{
		return new StringBuilder('#').append(bid).append('(').append(Util.oneString(name, labl)).append(')').toString() ;
	}
}
